
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

//#include <MPEG4-AAC_Decoder/include/interface.h>
#include <MPEG4-AACDecoder/include/interface.h>
#include "config.h"
#include "ad_internal.h"
#include "arc_common.h"

//-------------------------------------------------------------------------------------------

static ad_info_t info = 
{
  "AAC",
  "arcaac",
  "ARC International",
  "arcaac",
  "-*-"
};

LIBAD_EXTERN(arcaac)

//-------------------------------------------------------------------------------------------

extern int fakemono;

static IttiamAudioAacDecoder_t ObjAacDec;
static IttiamAudioAacDecoder_t* pObjAacDec;
static void *pMemPtr;
static void **origAddress;
static AACDecodeState safeState;
static IttiamAudioAacDecoderFrameInfo_t *frameInfo;
static int totalFrames;
static int totalIPBytes;

//-------------------------------------------------------------------------------------------

static void freeMemory(IttiamAudioAacDecoderMemInfo_t *table, int numAllocated, void **origAddress)
{
  int i;

  if (table != 0)
  {
    for(i = 0; i < numAllocated; i++)
    {
      if(table[i].location == EXTERNAL)
      {
	if (origAddress[i] != 0)
	{
	  free(origAddress[i]);
	  origAddress[i] = 0;
	}
      }
    }

    free(table);
    table = 0;
  }

  if (origAddress != 0)
  {
    free(origAddress);
    origAddress = 0;
  }
}

extern int debug_audio;	// Raymond 2007/03/22

AACDecodeState *hDecoder;

//-------------------------------------------------------------------------------------------

static int decodeAACHeader(sh_audio_t *sh)
{
  IttiamAudioAacDecoderConfig_t *config;

  IttiamAudioAacDecoderSetConfig(pObjAacDec,
				 AUDIO_AAC_DECODER_INIT,
				 (Word32)NULL,
				 (Word32)NULL);

  IttiamAudioAacDecoderSetConfig(pObjAacDec,
				 AUDIO_AAC_DECODER_NUM_OF_PCM_OUTPUT_BITS,
				 0,
				 FMT_16BIT /*OUTPUT_FORMAT*/);

  pMemPtr = IttiamAudioAacDecoderGetConfig(pObjAacDec,
					   AUDIO_AAC_DECODER_HEADER_DECODE,
					   0);

  frameInfo = (IttiamAudioAacDecoderFrameInfo_t *)pMemPtr;

  if (frameInfo->error < 0)
  {
    if(debug_audio)
      fprintf(stderr, "AAC Decoder: Error %d decoding header\n",frameInfo->error);
    return -1;
  }

  if(frameInfo->bytesconsumed == -5)
  {
    fprintf(stderr, "AAC Decoder: ADTS Sync-Word Not Found\n");
    return -1;
  }

  if(frameInfo->bytesconsumed < 0)
  {
    fprintf(stderr, "AAC Decoder: Header decode failed\n");
    return -1;
  }

  config = (IttiamAudioAacDecoderConfig_t *) IttiamAudioAacDecoderGetConfig(pObjAacDec,
					   AUDIO_AAC_DECODER_CONFIG,
					   0);

  sh->samplerate = config->defSampleRate;
  sh->channels   = config->defNumChannels;
  sh->samplesize = ((config->outputFormat == FMT_24BIT) ? 3 : 2);
//  fprintf(stderr,"AAC Decoder: sample rate is %d, %d channels, %d byte samples\n",sh->samplerate,sh->channels,sh->samplesize);
  //   sh->i_bps = (sh->samplerate * sh->channels * sh->samplesize); // Roughly speaking
  sh->i_bps = 0;
  return 0;
}

//-------------------------------------------------------------------------------------------

static int preinit(sh_audio_t *sh)
{
  sh->audio_out_minsize=32*36*2*2; //4608;
  return 1;
}

//-------------------------------------------------------------------------------------------

static int init(sh_audio_t *sh)
{
  int i;
  int rc;
  unsigned char frameLengthFlag;

  bufusage = 0;
  pInpBuffer = inputBuffer;
  pObjAacDec = &ObjAacDec;

  dec_audio_sh=sh; 

  IttiamAudioAacDecoderSetConfig(pObjAacDec,
				 AUDIO_AAC_DECODER_INPBUF_ADDR,
				 (void*)pInpBuffer,
				 INP_BUF_IDX);

  IttiamAudioAacDecoderGetConfig(pObjAacDec,
				 AUDIO_AAC_DECODER_GET_NUMBER_MEMTABS,
				 (Word32)NULL);

  pObjAacDec->memTable = (IttiamAudioAacDecoderMemInfo_t *)calloc(pObjAacDec->numEntries,
								  sizeof(IttiamAudioAacDecoderMemInfo_t));
  origAddress = (void **)calloc(pObjAacDec->numEntries, sizeof(void *));

  if ((pObjAacDec->memTable == NULL) || (origAddress == NULL))
  {
    fprintf(stderr, "AAC Decoder: Memory allocation for MDT failed\n");
    return -1;
  }

  IttiamAudioAacDecoderSetConfig(pObjAacDec,
				 AUDIO_AAC_DECODER_FILL_MEMTABS,
				 pObjAacDec,
				 0);

  for (i = 0; i < pObjAacDec->numEntries; i++)
  {
    IttiamAudioAacDecoderMemInfo_t *mem = &pObjAacDec->memTable[i];

    if (mem->location == EXTERNAL)
    {
      if (mem->memType == PERSISTENT)
	origAddress[i] = (void *)malloc(mem->size + mem->alignment); //, sizeof(Word8));
      else
	origAddress[i] = (void *)realloc_scratch(mem->size + mem->alignment);  // ??

      if (origAddress[i] == NULL)
      {
	fprintf(stderr, "AAC Decoder: Memory allocation failed\n");
	return 0;
      }
      mem->baseAddress = alignMem(origAddress[i], mem->alignment);
    }
    else if (mem->location == INTERNALX)
    {
      origAddress[i] = (void *)xBuffer;
      mem->baseAddress = alignMem(origAddress[i], mem->alignment);
    }
    else
    {
      origAddress[i] = (void *)yBuffer;
      mem->baseAddress = alignMem(origAddress[i], mem->alignment);
    }
  }

  IttiamAudioAacDecoderSetConfig(pObjAacDec,
				 AUDIO_AAC_DECODER_OUTBUF_ADDR,
				 sh->a_out_buffer,
				 OUT_BUF_IDX);

  read_audio_from_demuxer(MIN_STREAMSIZE * MAX_CHANNELS);

  if (decodeAACHeader(sh) == -1)
  {
    IttiamAudioAacDecoderSetConfig(pObjAacDec,
				   AUDIO_AAC_DECODER_INPBUF_ADDR,
				   (void*)sh->codecdata,
				   INP_BUF_IDX);

    if (decodeAACHeader(sh) == -1)
      return 0;

    IttiamAudioAacDecoderSetConfig(pObjAacDec,
				   AUDIO_AAC_DECODER_INPBUF_ADDR,
				   (void*)pInpBuffer,
				   INP_BUF_IDX);
  }
  else
  {
    shuffle_buffer(frameInfo->bytesconsumed);
  }

  hDecoder = (AACDecodeState *)(pObjAacDec->memTable[pObjAacDec->iPersistExt].baseAddress);	// Raymond 2007/06/04
  
  totalFrames = 0;
  totalIPBytes = 0;
  memcpy(&safeState,pObjAacDec->memTable[pObjAacDec->iPersistExt].baseAddress,sizeof(AACDecodeState));

  return 1;
}

//-------------------------------------------------------------------------------------------

static void uninit(sh_audio_t *sh)
{
  int i;

  if (pObjAacDec)
    for (i = 0; i < pObjAacDec->numEntries; i++)
    {
      IttiamAudioAacDecoderMemInfo_t *mem = &pObjAacDec->memTable[i];
      if (mem->location == EXTERNAL)
      {
	if (mem->memType == PERSISTENT)
	  freeMemory(pObjAacDec->memTable, i, origAddress[i]);
      }
    }

  free(origAddress);
}

//-------------------------------------------------------------------------------------------

static int control(sh_audio_t *sh, int cmd, void* arg, ...)
{
  int shift;

  switch(cmd)
  {
    case ADCTRL_RESYNC_STREAM:
    case ADCTRL_SKIP_FRAME:
      bufusage = 0;
      IttiamAudioAacDecoderSetConfig(pObjAacDec,
				     AUDIO_AAC_DECODER_INIT,
				     (Word32)NULL,
				     (Word32)NULL);
      memcpy(pObjAacDec->memTable[pObjAacDec->iPersistExt].baseAddress,&safeState,sizeof(AACDecodeState));
      return CONTROL_TRUE;
  }
  return CONTROL_UNKNOWN;
}

//-------------------------------------------------------------------------------------------

static int decode_audio(sh_audio_t *sh_audio,   // context
			unsigned char *buf,     // output buffer in context
			int minlen,             // available space in output device
			int maxlen)             // available space in context output buffer
{
  int error = -1;
  int len = 0;

  IttiamAudioAacDecoderSetConfig(pObjAacDec,
				 AUDIO_AAC_DECODER_OUTBUF_ADDR,
				 buf,
				 OUT_BUF_IDX);

  // Raymond 2008/06/13
  IttiamAudioAacDecoderSetConfig(pObjAacDec,
				   AUDIO_AAC_DECODER_INPBUF_ADDR,
				   (void*)(pInpBuffer),
				   INP_BUF_IDX);

  if (read_audio_from_demuxer(MIN_STREAMSIZE * MAX_CHANNELS)) 
//  if (len = read_audio_from_mp4_demuxer(MIN_STREAMSIZE * MAX_CHANNELS)) 
  {
    error = IttiamAudioAacDecoderExe(pObjAacDec);

#if 1	// Raymond 2007/06/04
 	if(hDecoder->MP4_AAC == 1 && sh_audio->samples_size > 0 && totalFrames < sh_audio->samples_size )	
	{
		// Raymond 2007/06/06
		if( sh_audio->seek_audio_frame >= 0 )
		{
			totalFrames = sh_audio->seek_audio_frame;
			sh_audio->seek_audio_frame = -1;
		}
		
		shuffle_buffer(sh_audio->samples[totalFrames].size);
		if( sh_audio->samples[totalFrames].size != frameInfo->bytesconsumed )
		{
//			printf("[%d] AAC sample_size = %d, bytesconsumed = %d\n", 
//				totalFrames, sh_audio->samples[totalFrames].size, frameInfo->bytesconsumed);
		}
	}
	else
#endif
    shuffle_buffer(frameInfo->bytesconsumed);

    totalFrames++;
    totalIPBytes += frameInfo->bytesconsumed;

    // FIXME the following is only good for CBR audio...
    if ((totalFrames * frameInfo->samples)/sh_audio->samplerate)
      sh_audio->i_bps = (totalIPBytes * 8) / ((totalFrames * frameInfo->samples)/sh_audio->samplerate) * 1000;
  }

  if(debug_audio == 1 && frameInfo->error != 0 )
  fprintf(stderr,"Back from decode, error is %d, %d bytes consumed, %d samples produced for %d channels\n",frameInfo->error,frameInfo->bytesconsumed,frameInfo->samples,sh_audio->channels);

  return ((frameInfo->error == 0) ? (frameInfo->samples * sh_audio->samplesize) : 0);
}

//-------------------------------------------------------------------------------------------

