
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "config.h"

#include "ad_internal.h"
#include <WMA-Decoder/include/interface.h>
#include <WMA-Decoder/include/wmaudio.h>
#include <WMA-Decoder/include/wmaudioType.h> // For tWMAFileStateInternal
#include "arc_common.h"

//-------------------------------------------------------------------------------------------

static ad_info_t info = 
{
  "WMA",
  "arcwma",
  "ARC International",
  "arcwma",
  "-*-"
};

LIBAD_EXTERN(arcwma)

//-------------------------------------------------------------------------------------------

extern int fakemono;
extern int errno;

static void *pMemPtr;
static void **origAddress;
static int numEntries;
static memTabs *table;
static tHWMAFileState      g_state;
static tWMAFileHeader      g_hdr;
static short outbuf[2048];
static int fileOffset = 0;
static int numReads = 0;
static int dropFrame = 0;
//Robert 2007-11-21 rename verbose -> wma_verbose, Fix GCC4.2.1 compile error
static int wma_verbose = 0;

extern int read_circular;   // Raymond 2007/10/02

#define MAX_STATE_SIZE 35266
#define BYTES_PER_SAMPLE 2

//-------------------------------------------------------------------------------------------

static int decodeWMAHeader(sh_audio_t *sh)
{
  tWMAFileStatus rc;
  Word32  numBytes;
  demuxer_t *d;
  tWMAFileStateInternal* curr_state = (tWMAFileStateInternal*) &g_state;

  d = sh->ds->demuxer;

  rc = WMAFileCheckParams(&g_state, &numBytes);

  if(rc != cWMA_NoErr)
  {
    if (wma_verbose) fprintf(stderr,"The file is not a valid WMA file");
    return -1;
  }

  if (numBytes > MAX_STATE_SIZE)
  {
    if (wma_verbose) fprintf(stderr,"Not enough memory allocated for state");
    return -1;
  }

  rc = WMAFileDecodeInit(&g_state, table[0].baseAddress, table);
  if(rc != cWMA_NoErr)
  {
    if (wma_verbose) fprintf(stderr,"Cannot initialize the WMA decoder");
    return -1;
  }

  rc = WMAFileDecodeInfo(&g_state, &g_hdr);
  if(rc != cWMA_NoErr)
  {
    if (wma_verbose) fprintf(stderr,"Failed to retrieve information");
    return -1;
  }

  sh->samplerate = g_hdr.sample_rate;
  sh->channels   = g_hdr.num_channels;
  sh->samplesize = g_hdr.bits_per_sample/8;
  sh->i_bps = curr_state->hdr_parse.nAvgBytesPerSec; // Raymond 2007/12/07
//  sh->i_bps = (sh->samplerate * sh->channels * sh->samplesize)/8; // Roughly speaking

  if (wma_verbose) fprintf(stderr,"\n\n****** Sample rate %d, channels %d, samplesize %d ******\n\n",sh->samplerate,sh->channels,sh->samplesize);
  fflush(stderr);

  if(g_hdr.has_DRM)
  {
    fprintf(stderr,"DRM not supported");
    return -1;
  }

  return 0;
}

//-------------------------------------------------------------------------------------------

static int preinit(sh_audio_t *sh)
{
  sh->audio_out_minsize=32*36*2*2; //4608;
  return 1;
}

//-------------------------------------------------------------------------------------------

static int init(sh_audio_t *sh)
{
  int i;
  int error;

  bufusage = 0;
  pInpBuffer = inputBuffer;
  dec_audio_sh=sh;

  numEntries = WMADecoderNumAlloc();
  origAddress = (void **)calloc(numEntries, sizeof(void *));
  table = (memTabs *)calloc(numEntries, sizeof(memTabs));

  if ((origAddress == NULL) || (table == NULL))
  {
    if (wma_verbose) fprintf(stderr, "Memory allocation for MDT failed");
    return -1;
  }
  WMADecoderFillMDT(table, MAX_STATE_SIZE);

  memset ((void *)&g_state, 0, sizeof(g_state));
  memset ((void *)&g_hdr, 0, sizeof(g_hdr));

  for (i = 0; i < numEntries; i++)
  {
    memTabs *mem = &table[i];

    if (mem->location == EXTERNAL)
    {
      if (mem->memType == PERSISTENT)
	origAddress[i] = (void *)malloc(mem->size + mem->alignment);
      else
	mem->baseAddress = (void *)realloc_scratch(mem->size + mem->alignment);

      if (origAddress[i] == NULL)
      {
	fprintf(stderr, "Memory allocation failed");
	return -1;
      }
      mem->baseAddress = alignMem(origAddress[i], mem->alignment);
    }
    else if (mem->location == INTERNALX)
    {
      origAddress[i] = xBuffer;
      mem->baseAddress = alignMem(origAddress[i], mem->alignment);
    }
    else
    {
      origAddress[i] = yBuffer;
      mem->baseAddress = alignMem(origAddress[i], mem->alignment);
    }
  }

  error = decodeWMAHeader(sh);

  return error ? 0 : 1;
}

//-------------------------------------------------------------------------------------------

static void uninit(sh_audio_t *sh)
{
  int i;
  for (i = 0; i < numEntries; i++)
  {
    memTabs *mem = &table[i];
    if (mem->location == EXTERNAL)
    {
      if (mem->memType == PERSISTENT)
	if (origAddress[i])
	{
	  free(origAddress[i]);
	  origAddress[i] = 0;
	}
    }
  }

  if (origAddress) free(origAddress);
  origAddress = 0;
  if (table) free(table);
  table = 0;
}

//-------------------------------------------------------------------------------------------

static int control(sh_audio_t *sh,int cmd,void* arg, ...)
{
  tWMAFileStateInternal* curr_state;
  demuxer_t *d;
  int pos;

  curr_state = (tWMAFileStateInternal*) &g_state;
  d = sh->ds->demuxer;
  if (d->audio && d->audio->asf_packet) pos = d->audio->asf_packet->pos;
  if (wma_verbose) fprintf(stderr,"\nPosition is, %d\n",pos);

  switch(cmd)
  {
    case ADCTRL_SKIP_FRAME:
    case ADCTRL_RESYNC_STREAM:
      curr_state->current_position = pos;
      curr_state->hdr_parse.currPacketOffset = 0;
      curr_state->hdr_parse.nextPacketOffset = pos;
      dropFrame = 6;
      return CONTROL_TRUE;
  }
  return CONTROL_UNKNOWN;
}

//-------------------------------------------------------------------------------------------

static int decode_audio(sh_audio_t *sh_audio,   // context
			unsigned char *buf,     // output buffer in context
			int minlen,             // available space in output device
			int maxlen)             // available space in context output buffer
{
  tWMAFileStatus rc;
  int decodedSamples;

  rc = WMAFileDecodeRawData(&g_state, &outbuf, &decodedSamples);

  if(rc != cWMA_NoErr)
  {
    if (rc == cWMA_BrokenFrame)
    {
      if (wma_verbose) fprintf(stderr,"Broken frame");
    }
    else if (rc == cWMA_NoMoreFrames || rc == cWMA_Failed)
    {
      if (wma_verbose) fprintf(stderr,"End of file?");
      dec_audio_sh->ds->eof = 1;
    }
    else if (rc == cWMA_Internal)
    {
      if (wma_verbose) fprintf(stderr,"Decoding failed with internal error (rc=%d)", rc);
    }
    else
    {
      if (wma_verbose) fprintf(stderr,"Decoding failed (rc=%d)", rc);
    }
    return 0;
  }
  
  if (dropFrame)
  {
    --dropFrame;
    return 0;
  }

  memcpy((sh_audio->a_buffer + (sh_audio->a_buffer_len)),&outbuf,(decodedSamples * sh_audio->channels * sh_audio->samplesize));

  return (decodedSamples * sh_audio->channels * sh_audio->samplesize);
}

//-------------------------------------------------------------------------------------------

#if 1	// Raymond 2007/10/25

int WMACallbackDataReader (int fileOffset, unsigned char *buffer, int minSize)
{
  demux_stream_t *ds;
  demux_packet_t *packet;
  int bufferOffset = 0;
  int currentFileOffset, availableSize;
  int ret = 0;
  unsigned char *bptr = NULL;

  if (dec_audio_sh == NULL)
    return (0);

  ds = dec_audio_sh->ds;

  	// Raymond 2007/11/21
  	if( fileOffset >= 0 && fileOffset + minSize < ds->demuxer->header_size )
  	{
//  		printf("header size = %d, minSize = %d\n", ds->demuxer->header_size, minSize);
  		bptr = ds->demuxer->header + fileOffset;
		memcpy(&buffer[bufferOffset], bptr, minSize);
		return 1;
  	}

	// Raymond 2007/11/21
  	if( minSize == 0 ) 
  	{  	
  		if( read_circular == 0 )
  		{
			read_audio_packet_from_demuxer(32768);
			packet = ds->current;
			
		  	if( packet != NULL )
		  	{	
//		  		printf("\nfileOffset = %d, minSize == 0\n", fileOffset);
//				printf("[minSize == 0] packet->pos = %d, packet->len = %d\n", packet->pos, packet->len);
				tWMAFileStateInternal* curr_state = (tWMAFileStateInternal*) &g_state;
				curr_state->hdr_parse.nextPacketOffset = packet->pos;				
		  	}
  		}
		
		return 1;	
	  }
  
// Raymond 2007/10/02	  
if( read_circular == 1 )
{
	bptr = stream_seek_phy(ds->demuxer->stream, fileOffset);
	memcpy(&buffer[bufferOffset], bptr, minSize);
	return 1;
}
else
{
  //packet = ds->first;
  packet = ds->current;		// Raymond 2007/11/21
  while (packet != NULL)
  {
//  	printf("packet->pos = %d\n", packet->pos);
//	printf("packet->pos + packet->len = %d\n", packet->pos + packet->len);
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
//    	printf("ds->current: fileOffset = %d, minSize = %d\n", fileOffset, minSize);
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
      else
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],availableSize);
	minSize -= availableSize;
	fileOffset += availableSize;
        bufferOffset += (fileOffset - packet->pos);
      }
    }
    packet = packet->next;
  }
  
  packet = ds->asf_packet;
  if (packet != NULL)
  {
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
//      printf("asf_packet: fileOffset = %d, minSize = %d\n", fileOffset, minSize);
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
    }
  }

  printf("fileOffset = %d, minSize = %d\n", fileOffset, minSize);	// Raymond 2007/11/21 - this should not happen!
  
  int fd = dup(ds->demuxer->stream->fd);
  if (fd == -1)
  {
    return (0);
  }
  if (lseek(fd,fileOffset,SEEK_SET) != -1)
  {
    ret = (read(fd,buffer,minSize) == minSize)?1:0;
  }
  close(fd);
  lseek(ds->demuxer->stream->fd,ds->demuxer->stream->pos,SEEK_SET);
  return (ret);
}	// Raymond 2007/10/02	

}

#else

int WMACallbackDataReader (int fileOffset, unsigned char *buffer, int minSize)
{
  demux_stream_t *ds;
  demux_packet_t *packet;
  int bufferOffset = 0;
  int currentFileOffset, availableSize;
  int ret = 0;

  if (dec_audio_sh == NULL)
    return (0);

  ds = dec_audio_sh->ds;

  packet = ds->first;
  while (packet != NULL)
  {
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
      else
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],availableSize);
	minSize -= availableSize;
	fileOffset += availableSize;
        bufferOffset += (fileOffset - packet->pos);
      }
    }
    packet = packet->next;
  }
  packet = ds->asf_packet;
  if (packet != NULL)
  {
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
    }
  }
  
  int fd = dup(ds->demuxer->stream->fd);
  if (fd == -1)
  {
    return (0);
  }
  if (lseek(fd,fileOffset,SEEK_SET) != -1)
  {
    ret = (read(fd,buffer,minSize) == minSize)?1:0;
  }
  close(fd);
  lseek(ds->demuxer->stream->fd,ds->demuxer->stream->pos,SEEK_SET);
  return (ret);
}
#endif

//-------------------------------------------------------------------------------------------

UWord32 WMAFileGetData (tWMAFileStateInternal*  pInt, 
			UWord32        Uoffset,
			UWord32        num_bytes,
			unsigned char   **ppData)
{
  int bytes_read;
  int i;

  memset(pInpBuffer,0,INPUT_BUF_SIZE);
  bytes_read = (WMACallbackDataReader (Uoffset, pInpBuffer, num_bytes) ? num_bytes : 0);
  *ppData = pInpBuffer;
  pInt->inpPtr = pInpBuffer;
  
  return bytes_read;
}

//-------------------------------------------------------------------------------------------

