#ifdef BUILD_SK2851		//Barry 2008-05-09


// Utility library for the ARC Pixel Graphics Unit
// ARC International, 2005.

//----------------------------------------------------------------------------------------------------

#include <pgu_lib.h>

//----------------------------------------------------------------------------------------------------

#define DEFAULT_FB "/dev/fb0"

//----------------------------------------------------------------------------------------------------

static int reset_on_quit = 0;
static int fbfd = -1;
YUV_planes planes;
static unsigned char* baseptr = -1;
//Robert 2007-11-21 rename yuv -> mp_yuv, Fix GCC4.2.1 compile error
struct yuv_info mp_yuv;


//----------------------------------------------------------------------------------------------------

int  pgu_open_fb()
{
   if((fbfd = open(DEFAULT_FB, O_RDWR)) == -1) {
    fprintf(stderr,"Can\'t open %s: %s\n", DEFAULT_FB, strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------

void  pgu_close_fb()
{
/*   if (reset_on_quit) */
/*   { */
/*     printf("Resetting PGU\n"); */
/*     if (ioctl(fbfd,  ARCPGUFB_SET_PGU_STATE, &safe_pgu) == -1) { */
/*       fprintf(stderr,"close_fb: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno)); */
/*       return -1; */
/*     } */
/*   } */

  if (fbfd != -1) 
    close(fbfd);
  fbfd = -1;
//  return 0;
}

//----------------------------------------------------------------------------------------------------

unsigned char*  pgu_map_buffer(int length, int offset)
{
  if (fbfd == -1) return -1;

  baseptr = mmap(0, length, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, offset); 
  
  if((long) baseptr == -1) {
    fprintf(stderr,"Couldn\'t map memory area: %s\n", strerror(errno));
    return -1;
  }

  return baseptr;
}

//----------------------------------------------------------------------------------------------------

int  pgu_unmap_buffer(unsigned char *ptr, int length)
{
  baseptr = -1;
  if (ptr) 
  {
    return munmap(ptr, length);
  }

  return -1;
}

//----------------------------------------------------------------------------------------------------

unsigned int  pgu_get_yuv_base(unsigned char bufno)
{
	return ioctl(fbfd, ARCPGUFB_GET_YUV_BASE, bufno); 
}

int  pgu_get_yuv_buffers(unsigned char* ptr)
{	
  int i;

  if (ioctl(fbfd, ARCPGUFB_GETYUV, &mp_yuv) == -1) 
  {
    fprintf(stderr,"Can\'t get YUV info: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
    return -1;
  }

  for (i = 0; i < mp_yuv.num_buffers; i++)
  {
    planes.Y[i] = ptr + ioctl(fbfd, ARCPGUFB_GET_YUV_BASE, i); 
    planes.U[i] = planes.Y[i] + (mp_yuv.u_offset);
    planes.V[i] = planes.Y[i] + (mp_yuv.v_offset);
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------
//-- Flowbee 2008-03-13
//int  pgu_set_primary_mode(int newmode)
int  pgu_set_primary_mode(unsigned long newmode)
{
#if 1
  if ((newmode != INPUT_FORMAT_422) && (newmode != INPUT_FORMAT_420) && (newmode != INPUT_FORMAT_RGB888))
    return -1;

  //  return  pgu_mask_and_set_statctrl(STATCTRL_PIX_FMT_MASK,newmode);
  if (ioctl(fbfd,  ARCPGUFB_SET_MAIN_MODE, &newmode) == -1) 
  {
    fprintf(stderr,"Can\'t set PGU control reg: ARCPGUFB_SET_MAIN_MODE ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
#endif
}

//----------------------------------------------------------------------------------------------------
int pgu_set_origin(unsigned int x, unsigned int y)
{
    unsigned long xy_offset;
    xy_offset = ENCODE_RES(x, y);

    if (ioctl(fbfd, ARCPGUFB_SET_ORIGIN, &xy_offset) == -1)
    {
        fprintf(stderr,"arcpgufb: ARCPGUFB_SET_ORIGIN ioctl failed due to %s\n", strerror(errno));
        return -1;
    }
    return 0;
}

//----------------------------------------------------------------------------------------------------

int  pgu_switch_to_buffer(int bufno)
{
  if (ioctl(fbfd, ARCPGUFB_SWITCH_YUV, &bufno) == -1)
  {
    fprintf(stderr,"ARCPGUFB_SWITCH ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------
int  pgu_set_resolution(unsigned int x, unsigned int y)
{
	unsigned long disp_res;

	disp_res = ENCODE_RES(x, y);
	
  if (ioctl(fbfd, ARCPGUFB_SET_DISP_RES, &disp_res) == -1)
  {
    fprintf(stderr,"ARCPGUFB_SET_DISP_RES ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

int  pgu_set_yuv_resolution(unsigned int x, unsigned int y)
{
	unsigned long disp_res;

	disp_res = ENCODE_RES(x, y);
	
  if (ioctl(fbfd, ARCPGUFB_SET_DISP_RES, &disp_res) == -1)
  {
    fprintf(stderr,"ARCPGUFB_SET_DISP_RES ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------
int  pgu_stop_pgu()
{
  int busy = 1;
  unsigned long statctrl;

  if (fbfd == -1) return -1;
  if (ioctl(fbfd,  ARCPGUFB_STOP_DISPLAY) == -1) {
    fprintf(stderr,"Can\'t stop PGU: ARCPGUFB_STOP_DISPLAY ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0; 
}

//----------------------------------------------------------------------------------------------------
int  pgu_start_pgu()
{
  if (ioctl(fbfd,  ARCPGUFB_START_DISPLAY) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_START_DISPLAY ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0; 
}

//----------------------------------------------------------------------------------------------------
int pgu_set_primary_buffer(unsigned long bufno)
{
  if (ioctl(fbfd,  ARCPGUFB_SET_RGB_BUF, &bufno) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_RGB_BUF ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}
//----------------------------------------------------------------------------------------------------
//Barry 2008-06-18
int pgu_enable_osd(unsigned long _en)
{
	if(ioctl(fbfd, ARCPGUFB_ENABLE_OSD, &(_en)) == -1)
	{
		fprintf("ARCPGUFB_ENABLE_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

int pgu_select_osd(unsigned long _idx, unsigned long _en)
{
	unsigned long _val;
	_val = (_idx & OSD_SEL_MASK) + (_en & 0x01);
	if(ioctl(fbfd, ARCPGUFB_SELECT_OSD, &(_val)) == -1)
	{
		fprintf("ARCPGUFB_SELECT_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

int pgu_set_osd_start_position(unsigned long _idx, unsigned long _x, unsigned long _y)
{
	unsigned long _val;
	_val = (((_idx & OSD_SEL_MASK) >> OSD_SEL_SHIFT) << 24) + ((_x & 0x0fff) << 12) + (_y & 0x0fff);
	if(ioctl(fbfd, ARCPGUFB_SET_OSD_START_POSITION, &(_val)) == -1)
	{
		fprintf("ARCPGUFB_SET_OSD_START_POSITION ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

int pgu_set_osd_end_position(unsigned long _idx, unsigned long _x, unsigned long _y)
{
	unsigned long _val;
	_val = (((_idx & OSD_SEL_MASK) >> OSD_SEL_SHIFT) << 24) + ((_x & 0x0fff) << 12) + (_y & 0x0fff);
	if(ioctl(fbfd, ARCPGUFB_SET_OSD_END_POSITION, &(_val)) == -1)
	{
		fprintf("ARCPGUFB_SET_OSD_END_POSITION ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
//Barry 2008-06-20
int  audio_pgu_get_yuv_buffers()
{	
  int i;
  
  if (ioctl(fbfd, ARCPGUFB_GETYUV, &mp_yuv) == -1) 
  {
    fprintf(stderr,"Can\'t get YUV info: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}
//----------------------------------------------------------------------------------------------------
#if 0	//Barry 2008-08-18
int  pgu_reset()
{
  if (ioctl(fbfd, ARCPGUFB_RESET) == -1)
  {
    fprintf(stderr,"ARCPGUFB_RESET ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------
// Raymond 2007/08/08

static int fd_time = -1;

int  pgu_open_fb_for_time()
{
   	if((fd_time = open(DEFAULT_FB, O_RDWR)) == -1) 
   	{
    		fprintf(stderr,"Can\'t open %s: %s\n", DEFAULT_FB, strerror(errno));
    		return -1;
  	}

  	return 0;
}

void  pgu_close_fb_for_time()
{
  	if (fd_time != -1) 
    		close(fd_time);
  	fd_time = -1;
}

void set_mplayer_time(int mplayer_time)
{
	int res;

	if (fd_time == -1) 
		return;
	
	res = ioctl(fd_time, ARCPGUFB_SET_MPLAYER_PLAY_TIME, &mplayer_time);
	
	if (res) 
	{
		fprintf(stderr,"set_mplayer_time failed\n");
    	}
}

// Raymond 2007/08/23
void set_mplayer_duration(int mplayer_duration)
{
	int res;

	if (fd_time == -1) 
		return;
	
	res = ioctl(fd_time, ARCPGUFB_SET_MPLAYER_DURATION, &mplayer_duration);
	
	if (res) 
	{
		fprintf(stderr,"set_mplayer_duration failed\n");
    	}
}

// Raymond 2008/05/20
void set_mplayer_status(int _status)
{
	int res;

	if (fd_time == -1) 
		return;
	
	res = ioctl(fd_time, ARCPGUFB_SET_MPLAYER_STATUS, &_status);
	
	if (res) 
	{
		fprintf(stderr,"set_mplayer_status failed\n");
    	}
}

//----------------------------------------------------------------------------------------------------
// Raymond 2007/11/23
int pgu_set_mplayer_advertisment(unsigned long _yesno)
{
	if (fd_time == -1) 
		return;

	if(ioctl(fd_time, ARCPGUFB_MPLAYER_ADVERTISMENT, &(_yesno)) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_MPLAYER_ADVERTISMENT ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//-----------------------------------------------------------------------------
//Barry 2007-08-20	AVI with subtitle
int pgu_get_display_index()
{
	unsigned long _idx = 0xff;

	if(ioctl(fbfd, ARCPGUFB_GET_DISP_INDEX, &(_idx)) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_GET_DISP_INDEX ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return ((int)_idx);
}

//-----------------------------------------------------------------------------
//Barry 2007-11-27	-hdtv option
int pgu_set_display_index(unsigned long _idx)
{
	if(ioctl(fbfd, ARCPGUFB_SET_DISP_INDEX, &(_idx)) == -1) 
	{
		fprintf("ARCPGUFB_GET_DISP_INDEX ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//-----------------------------------------------------------------------------
int  pgu_enable_overlay(int yesno)
{
  	if (yesno){
		if(ioctl(fbfd, ARCPGUFB_ENABLE_OL) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_ENABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else{
		if(ioctl(fbfd, ARCPGUFB_DISABLE_OL) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_DISABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	return 0;
//	return  pgu_mask_and_set_statctrl(STATCTRL_OL_EN_MASK, (yesno ? STATCTRL_OL_EN_MASK : 0)); 
}

//----------------------------------------------------------------------------------------------------
//Barry 2007-12-24
int pgu_show_mplayer_speed(char _time[])
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_VIDEO_SPEED, _time) == -1) 
	{
		fprintf("ARCPGUFB_SHOW_VIDEO_SPEED ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

int pgu_clear_mplayer_speed(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_VIDEO_SPEED) == -1) 
	{
		fprintf("ARCPGUFB_CLEAR_VIDEO_SPEED ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
//Barry 2008-01-07
int pgu_get_mplayer_time_on_off(void)
{
 	unsigned long _on;
 	
	if(ioctl(fbfd, ARCPGUFB_GET_VIDEO_TIME_ONOFF, &_on) == -1) 
	{
		fprintf("ARCPGUFB_GET_VIDEO_TIME_ONOFF ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return ((int)_on);
}

//----------------------------------------------------------------------------------------------------
int pgu_clear_mplayer_subtitle()
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_VIDEO_SUBTITLE) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_SHOW_TERAWINS_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_show_mplayer_subtitle(char _s[][31])
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_VIDEO_SUBTITLE, _s) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_SHOW_TERAWINS_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//-----------------------------------------------------------------------------
int pgu_enable_tv_osd(int yesno)
{
	if (yesno){
		if(ioctl(fbfd, ARCPGUFB_ENABLE_TV_OSD) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_ENABLE_TV_OSD ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else{
		/* Flowbee@960424 */		
		if(ioctl(fbfd, ARCPGUFB_DISABLE_TV_OSD) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_DISABLE_TV_OSD ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int pgu_set_video_yuv_res(struct _disp_scalar_info *p)
{
  if (ioctl(fbfd,  ARCPGUFB_SET_VIDEO_YUV_RES, p) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_VIDEO_YUV_RES ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}
#endif


#else	//SK2850


// Utility library for the ARC Pixel Graphics Unit

// ARC International, 2005.

//----------------------------------------------------------------------------------------------------

#include <pgu_lib.h>

//----------------------------------------------------------------------------------------------------

#define DEFAULT_FB "/dev/fb0"

//----------------------------------------------------------------------------------------------------

static int reset_on_quit = 0;
static int fbfd = -1;
static arc_pgu safe_pgu;
static arc_pgu user_pgu;
YUV_planes planes;
static unsigned char* baseptr = -1;
//Robert 2007-11-21 rename yuv -> mp_yuv, Fix GCC4.2.1 compile error
struct yuv_info mp_yuv;

//----------------------------------------------------------------------------------------------------

int  pgu_mask_and_set_statctrl(unsigned long mask, unsigned long newbits)
{
  unsigned long statctrl;

  if (fbfd == -1) return -1;
  if (ioctl(fbfd,  ARCPGUFB_GET_STATCTRL, &statctrl) == -1) 
  {
    fprintf(stderr,"Can\'t set PGU control reg: ARCPGUFB_GET_STATCTRL ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  statctrl &= ~mask;
  statctrl |= (newbits & mask);
  //  statctrl &= ~STATCTRL_REG_DISP_EN_MASK;

  if (ioctl(fbfd,  ARCPGUFB_SET_STATCTRL, &statctrl) == -1) 
  {
    fprintf(stderr,"Can\'t set PGU control reg: ARCPGUFB_SET_STATCTRL ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------

int pgu_get_statctrl(unsigned long *dest)
{
  if (ioctl(fbfd,  ARCPGUFB_GET_STATCTRL, dest) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_GET_STATCTRL ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0; 
}

//----------------------------------------------------------------------------------------------------
//Barry 2007-08-20	AVI with subtitle
int pgu_get_display_index()
{
	unsigned long _idx = 0xff;

	if(ioctl(fbfd, ARCPGUFB_GET_DISP_INDEX, &(_idx)) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_GET_DISP_INDEX ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return ((int)_idx);
}

//-----------------------------------------------------------------------------
//Barry 2007-11-27	-hdtv option
int pgu_set_display_index(unsigned long _idx)
{
	if(ioctl(fbfd, ARCPGUFB_SET_DISP_INDEX, &(_idx)) == -1) 
	{
		fprintf("ARCPGUFB_GET_DISP_INDEX ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//-----------------------------------------------------------------------------
int pgu_clear_mplayer_subtitle()
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_MPLAYER_SUBTITLE) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_SHOW_TERAWINS_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_show_mplayer_subtitle(char _s[][31])
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_MPLAYER_SUBTITLE, _s) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_SHOW_TERAWINS_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_enable_tv_osd(int yesno)
{
	if (yesno){
		if(ioctl(fbfd, ARCPGUFB_ENABLE_TV_OSD) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_ENABLE_TV_OSD ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else{
		/* Flowbee@960424 */		
		if(ioctl(fbfd, ARCPGUFB_DISABLE_TV_OSD) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_DISABLE_TV_OSD ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	return 0;
}
//-----------------------------------------------------------------------------
int  pgu_open_fb()
{
   if((fbfd = open(DEFAULT_FB, O_RDWR)) == -1) {
    fprintf(stderr,"Can\'t open %s: %s\n", DEFAULT_FB, strerror(errno));
    return -1;
  }

  if (ioctl(fbfd,  ARCPGUFB_GET_PGU_STATE, &safe_pgu) == -1) {
    fprintf(stderr,"Can\'t open framebuffer: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------

void  pgu_close_fb()
{
/*   if (reset_on_quit) */
/*   { */
/*     printf("Resetting PGU\n"); */
/*     if (ioctl(fbfd,  ARCPGUFB_SET_PGU_STATE, &safe_pgu) == -1) { */
/*       fprintf(stderr,"close_fb: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno)); */
/*       return -1; */
/*     } */
/*   } */

  if (fbfd != -1) 
    close(fbfd);
  fbfd = -1;
//  return 0;
}

//----------------------------------------------------------------------------------------------------

unsigned char*  pgu_map_buffer(int length, int offset)
{
  if (fbfd == -1) return -1;

  baseptr = mmap(0, length, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, offset); 
  
  if((long) baseptr == -1) {
    fprintf(stderr,"Couldn\'t map memory area: %s\n", strerror(errno));
    return -1;
  }

  return baseptr;
}

//----------------------------------------------------------------------------------------------------

int  pgu_unmap_buffer(unsigned char *ptr, int length)
{
  baseptr = -1;
  if (ptr) 
  {
    return munmap(ptr, length);
  }

  return -1;
}

//----------------------------------------------------------------------------------------------------

unsigned int  pgu_get_yuv_base(unsigned char bufno)
{
	return ioctl(fbfd, ARCPGUFB_GET_YUV_BASE, bufno); 
}

#if 0	//Barry 2008-07-30
int  pgu_get_yuv_buffers(unsigned char* ptr)
{	
  int i;

  if (ioctl(fbfd, ARCPGUFB_GETYUV, &mp_yuv) == -1) 
  {
    fprintf(stderr,"Can\'t get YUV info: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
    return -1;
  }

  for (i = 0; i < mp_yuv.num_buffers; i++)
  {
    planes.Y[i] = ptr + ioctl(fbfd, ARCPGUFB_GET_YUV_BASE, i); 
    planes.U[i] = planes.Y[i] + (mp_yuv.u_offset);
    planes.V[i] = planes.Y[i] + (mp_yuv.v_offset);
  }
  return 0;
}
#endif

//----------------------------------------------------------------------------------------------------

int  pgu_set_primary_mode(int newmode)
{
  if ((newmode != STATCTRL_PIX_FMT_RGB555) && (newmode != STATCTRL_PIX_FMT_YUV420))
    return -1;

  //  return  pgu_mask_and_set_statctrl(STATCTRL_PIX_FMT_MASK,newmode);
  if (ioctl(fbfd,  ARCPGUFB_SET_MAIN_MODE, &newmode) == -1) 
  {
    fprintf(stderr,"Can\'t set PGU control reg: ARCPGUFB_SET_MAIN_MODE ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
}

//----------------------------------------------------------------------------------------------------

int  pgu_set_overlay_mode(int newmode)
{
  if ((newmode != STATCTRL_OL_FMT_RGBA4444) && (newmode != STATCTRL_OL_FMT_RGBA5551) && (newmode != STATCTRL_OL_FMT_RGB555))
    return -1;

  return  pgu_mask_and_set_statctrl(STATCTRL_OL_FMT_MASK,newmode);
}

//----------------------------------------------------------------------------------------------------

int  pgu_enable_overlay(int yesno)
{
  	if (yesno){
		if(ioctl(fbfd, ARCPGUFB_ENABLE_OL) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_ENABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else{
		if(ioctl(fbfd, ARCPGUFB_DISABLE_OL) == -1) 
		{
			fprintf(stderr, "Can\'t get PGU: ARCPGUFB_DISABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	return 0;
//	return  pgu_mask_and_set_statctrl(STATCTRL_OL_EN_MASK, (yesno ? STATCTRL_OL_EN_MASK : 0)); 
}

//----------------------------------------------------------------------------------------------------

int pgu_set_origin(unsigned int x, unsigned int y)
{
#ifdef CONFIG_ENABLE_CENTERING 
  unsigned long xy_offset;
  xy_offset = ENCODE_PGU_DIMS(x, y);

  if (ioctl(fbfd, ARCPGUFB_SET_ORIGIN, &xy_offset) == -1) { 
    fprintf(stderr,"arcpgufb: ARCPGUFB_SET_ORIGIN ioctl failed due to %s\n", strerror(errno)); 
    return -1; 
  }
#endif
  return 0;
}

//----------------------------------------------------------------------------------------------------

int  pgu_switch_to_buffer(int bufno)
{
  if (ioctl(fbfd, ARCPGUFB_SWITCH_YUV, &bufno) == -1)
  {
    fprintf(stderr,"ARCPGUFB_SWITCH ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------

int  pgu_reset()
{
  if (ioctl(fbfd, ARCPGUFB_RESET) == -1)
  {
    fprintf(stderr,"ARCPGUFB_RESET ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------

int  pgu_set_resolution(unsigned int x, unsigned int y)
{
  arc_pgu *pgu_ptr;
  //Barry 2007-04-13 Flowbee@960213 modified
//  pgu_stop_pgu();

  if (! (pgu_ptr =  pgu_get_pgu())) return -1;

  pgu_ptr->frm_dim = ENCODE_PGU_DIMS(x-1, y-1);
  pgu_ptr->stride = x*2;

//  pgu_set_pgu(pgu_ptr);
//  return  pgu_start_pgu();
  return pgu_set_pgu(pgu_ptr);
}

int  pgu_set_yuv_resolution(unsigned int x, unsigned int y)
{
  arc_pgu *pgu_ptr;
  pgu_stop_pgu();

  if (! (pgu_ptr =  pgu_get_pgu())) return -1;

  pgu_ptr->frm_dim = ENCODE_PGU_DIMS(x-1, y-1);
  pgu_ptr->stride = x;
  pgu_ptr->cru_frm_st = x*y;
  pgu_ptr->cru_stride = x>>1;
  pgu_ptr->crv_frm_st = pgu_ptr->cru_frm_st + ((x>>1)*(y>>1));
  pgu_ptr->crv_stride = x>>1;

  pgu_set_pgu(pgu_ptr);
  return  pgu_start_pgu();
}
//----------------------------------------------------------------------------------------------------

int pgu_get_colkey(unsigned long *dest)
{
  if (ioctl(fbfd,  ARCPGUFB_GET_OL_COLKEY, dest) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_GET_COLKEY ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0; 
}

//----------------------------------------------------------------------------------------------------

int  pgu_set_colkey(unsigned long newkey)
{
  if (ioctl(fbfd,  ARCPGUFB_SET_OL_COLKEY, &newkey) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_COLKEY ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0; 
}

//----------------------------------------------------------------------------------------------------

unsigned long pgu_set_clkcfg(unsigned long pxlclk, unsigned long sysclk)
{
  arc_pgu tmppgu;
  unsigned long clk_div, high;

  if (ioctl(fbfd,  ARCPGUFB_GET_PGU_STATE, &tmppgu) == -1) {
    fprintf(stderr,"Set CLK_CFG: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return 0xffffffff;
  }

  clk_div = 1;
  while (sysclk/clk_div > pxlclk) 
    clk_div++;

  if (clk_div > 1) clk_div--;
  high = clk_div/2;
  tmppgu.clk_cfg = (clk_div | (high << 16));

  if (ioctl(fbfd,  ARCPGUFB_SET_PGU_STATE, &tmppgu) == -1) {
    fprintf(stderr,"Set CLK_CFG: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return 0xffffffff;
  }

  return tmppgu.clk_cfg;
}

//----------------------------------------------------------------------------------------------------

int  pgu_stop_pgu()
{
  int busy = 1;
  unsigned long statctrl;

  if (fbfd == -1) return -1;
  if (ioctl(fbfd,  ARCPGUFB_STOP_DISPLAY) == -1) {
    fprintf(stderr,"Can\'t stop PGU: ARCPGUFB_STOP_DISPLAY ioctl failed due to %s\n", strerror(errno));
    return -1;
  }

  while (busy) {
    if (ioctl(fbfd,  ARCPGUFB_GET_STATCTRL, &statctrl) == -1) 
    {
      fprintf(stderr,"Can\'t stop PGU: ARCPGUFB_GET_STATCTRL ioctl failed due to %s\n", strerror(errno));
      return -1;
    }
    busy = statctrl & STATCTRL_DISP_BUSY_MASK;
  }

  return 0; 
}

//----------------------------------------------------------------------------------------------------

int  pgu_start_pgu()
{
  if (ioctl(fbfd,  ARCPGUFB_START_DISPLAY) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_START_DISPLAY ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0; 
}

//----------------------------------------------------------------------------------------------------

int  pgu_set_ol_startend(int start, int end)
{
  if (ioctl(fbfd,  ARCPGUFB_SET_OL_START, &start) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_OL_START ioctl failed due to %s\n", strerror(errno));
    return -1;
  }

  if (ioctl(fbfd,  ARCPGUFB_SET_OL_END, &end) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_OL_END ioctl failed due to %s\n", strerror(errno));
    return -1;
  }

  return 0;
}

//----------------------------------------------------------------------------------------------------

int pgu_set_ol_offset(unsigned long offset)
{
  if (ioctl(fbfd,  ARCPGUFB_SET_OL_OFFSET, &offset) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_OL_OFFSET ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------

int pgu_set_primary_buffer(unsigned long bufno)
{
  if (ioctl(fbfd,  ARCPGUFB_SET_RGB_BUF, &bufno) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_RGB_BUF ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}

//----------------------------------------------------------------------------------------------------

arc_pgu*  pgu_get_pgu()
{
  if (ioctl(fbfd,  ARCPGUFB_GET_PGU_STATE, &user_pgu) == -1) {
    fprintf(stderr,"Can\'t get PGU: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return 0;
  }

  return &user_pgu;
}

//----------------------------------------------------------------------------------------------------

int  pgu_set_pgu(arc_pgu* newpgu)
{
  if (ioctl(fbfd,  ARCPGUFB_SET_PGU_STATE, newpgu) == -1) {
    fprintf(stderr,"Can\'t set PGU: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return -1;
  }

  return 0;
}

//----------------------------------------------------------------------------------------------------

void pgu_blank_rgb(unsigned long base_offset, int width, int height)
{
  if (baseptr != -1)
    memset(baseptr + base_offset,0,(width*height*2));
}

//----------------------------------------------------------------------------------------------------

int get_hsync_cycle(arc_pgu* tmppgu)
{
  if (! tmppgu) return -1;

  return (tmppgu->hsync_cfg & PGU_SYNC_REGS_CYC_MASK);
}

//----------------------------------------------------------------------------------------------------

int get_hsync_backporch(arc_pgu* tmppgu)
{
  if (! tmppgu) return -1;

  return ((tmppgu->hsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET);
}

//----------------------------------------------------------------------------------------------------

int get_hsync_width(arc_pgu* tmppgu)
{
  if (! tmppgu) return -1;

  return ((tmppgu->hsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET);
}

//----------------------------------------------------------------------------------------------------

unsigned long set_hsync(int cycle, int backporch, int width)
{
  arc_pgu tmppgu;

  if (ioctl(fbfd,  ARCPGUFB_GET_PGU_STATE, &tmppgu) == -1) {
    fprintf(stderr,"Set HSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return 0xffffffff;
  }

  tmppgu.hsync_cfg = (((width << PGU_SYNC_REGS_WIDTH_OFFSET) & PGU_SYNC_REGS_WIDTH_MASK)
		  | ((backporch << PGU_SYNC_REGS_PORCH_OFFSET) & PGU_SYNC_REGS_PORCH_MASK)
		  | (cycle & PGU_SYNC_REGS_CYC_MASK));

  if (ioctl(fbfd,  ARCPGUFB_SET_PGU_STATE, &tmppgu) == -1) {
    fprintf(stderr,"Set HSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return 0xffffffff;
  }

  return tmppgu.hsync_cfg;
}

//----------------------------------------------------------------------------------------------------

int get_vsync_cycle(arc_pgu* tmppgu)
{
  if (! tmppgu) return -1;

  return (tmppgu->vsync_cfg & PGU_SYNC_REGS_CYC_MASK);
}

//----------------------------------------------------------------------------------------------------

int get_vsync_backporch(arc_pgu* tmppgu)
{
  if (! tmppgu) return -1;

  return ((tmppgu->vsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET);
}

//----------------------------------------------------------------------------------------------------

int get_vsync_width(arc_pgu* tmppgu)
{
  if (! tmppgu) return -1;

  return ((tmppgu->vsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET);
}

//----------------------------------------------------------------------------------------------------

unsigned long set_vsync(int cycle, int backporch, int width)
{
  arc_pgu tmppgu;

  if (ioctl(fbfd,  ARCPGUFB_GET_PGU_STATE, &tmppgu) == -1) {
    fprintf(stderr,"Set VSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return 0xffffffff;
  }

  tmppgu.vsync_cfg = (((width << PGU_SYNC_REGS_WIDTH_OFFSET) & PGU_SYNC_REGS_WIDTH_MASK)
		  | ((backporch << PGU_SYNC_REGS_PORCH_OFFSET) & PGU_SYNC_REGS_PORCH_MASK)
		  | (cycle & PGU_SYNC_REGS_CYC_MASK));

  if (ioctl(fbfd,  ARCPGUFB_SET_PGU_STATE, &tmppgu) == -1) {
    fprintf(stderr,"Set VSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return 0xffffffff;
  }

  return tmppgu.vsync_cfg;
}

//----------------------------------------------------------------------------------------------------

void  pgu_dump_controller_regs(arc_pgu *pguptr)
{
  printf("Offset\tReg ID\t\tContents\tSummary\n---------------------------------------------\n\n");
  printf("0x00\tID\t\t0x%08x\tIdentity\n",pguptr->pgu_id);
  printf("0x08\tCLK_CTRL\t0x%08x\tClock Cfg - Divider %d, Pulse width %d\n",pguptr->clk_cfg,
	 (pguptr->clk_cfg & CLK_CFG_REG_CLK_DIV_MASK),GET_CLK_CFG_REG_CLK_HIGH_VAL(pguptr->clk_cfg));
  printf("0x10\tDISP_DIM\t0x%08x\tDisplay dimensions - X %d, Y %d\n",pguptr->disp_dim, 
	 pguptr->disp_dim & DISP_DIM_REG_PGU_X_RES_MASK, ((pguptr->disp_dim >> 16) & DISP_DIM_REG_PGU_X_RES_MASK));
  printf("0x18\tHSYNC_CFG\t0x%08x\tHSync - Width %d, Back porch %d, Pulse width %d\n",pguptr->hsync_cfg,
	 ((pguptr->hsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET),
	 ((pguptr->hsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET),
	 (pguptr->hsync_cfg & PGU_SYNC_REGS_CYC_MASK));
  printf("0x20\tVSYNC_CFG\t0x%08x\tVSync - Width %d, Back porch %d, Pulse width %d\n",pguptr->vsync_cfg,
	 ((pguptr->vsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET),
	 ((pguptr->vsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET),
	 (pguptr->vsync_cfg & PGU_SYNC_REGS_CYC_MASK));
  printf("0x28\tFRM_ST\t\t0x%08x\tMain start address\n",pguptr->frm_start);
  printf("0x30\tSTRIDE\t\t0x%08x\tMain stride\n",pguptr->stride);
  printf("0x38\tFRM_DIM\t\t0x%08x\tFrame dimensions - X %d, Y %d\n",pguptr->frm_dim,
	 (FRM_DIM_REG_FRM_WTH_MASK & pguptr->frm_dim),
	 (FRM_DIM_REG_FRM_LNS_MASK & (pguptr->frm_dim >> 16)));
  printf("0x40\tSTAT_CTRL\t0x%08x\tStatus/Control\n",pguptr->statctrl);

  printf("\t\t\t\t\tDISP_EN\t%s\n",((pguptr->statctrl & STATCTRL_REG_DISP_EN_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tI_EN\t%s\n",((pguptr->statctrl & STATCTRL_INTR_EN_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tI_CLR\t%s\n",((pguptr->statctrl & STATCTRL_INTR_CLR_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tCNST\t%s\n",((pguptr->statctrl & STATCTRL_CONT_B_MASK) ? "ON" : "OFF"));

  printf("\t\t\t\t\tMode\t");
  if (! (pguptr->statctrl & STATCTRL_PIX_FMT_MASK)) printf("RGB555\n");
  else if (pguptr->statctrl & STATCTRL_PIX_FMT_YUV420) printf("YUV420\n");
  else printf("Unrecognised\n");

  printf("\t\t\t\t\tBL\t%s\n",((pguptr->statctrl & STATCTRL_BACKLIGHT_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tX2\t%s\n",((pguptr->statctrl & STATCTRL_X2_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tRS\t%s\n",((pguptr->statctrl &  STATCTRL_REV_LN_SCAN_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tHPOL\t%s\n",((pguptr->statctrl & STATCTRL_HPOL_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tVPOL\t%s\n",((pguptr->statctrl & STATCTRL_VPOL_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tDPOL\t%s\n",((pguptr->statctrl & STATCTRL_DPOL_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tOL_EN\t%s\n",((pguptr->statctrl & STATCTRL_OL_EN_MASK) ? "ON" : "OFF"));

  printf("\t\t\t\t\tOL Mode\t");
  if (! (pguptr->statctrl & STATCTRL_OL_FMT_MASK)) printf("RGBA4444\n");
  else if (pguptr->statctrl & STATCTRL_OL_FMT_RGBA5551) printf("RGBA5551\n");
  else if (pguptr->statctrl & STATCTRL_OL_FMT_RGB555) printf("RGB555\n");
  else printf("Unrecognised\n");

  printf("\t\t\t\t\tDB\t%s\n",((pguptr->statctrl & STATCTRL_DISP_BUSY_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tBB\t%s\n",((pguptr->statctrl & STATCTRL_BU_BUSY_MASK) ? "ON" : "OFF"));
  printf("\t\t\t\t\tI_FLG\t%s\n",((pguptr->statctrl & STATCTRL_INTR_FLAG) ? "ON" : "OFF"));

  printf("0x48\tCRU_FRM_ST\t0x%08x\tU offset\n",pguptr->cru_frm_st);
  printf("0x50\tCRU_STRIDE\t0x%08x\tU stride\n",pguptr->cru_stride);
  printf("0x58\tCRV_FRM_ST\t0x%08x\tV offset\n",pguptr->crv_frm_st);
  printf("0x60\tCRV_STRIDE\t0x%08x\tV stride\n",pguptr->crv_stride);

#ifdef CONFIG_ENABLE_ARCPGU_OVERLAY
  printf("0x68\tOL_FRM_ST\t0x%08x\tOverlay start address\n",pguptr->ol_frm_st);
  printf("0x70\tOL_STRIDE\t0x%08x\tOverlay stride\n",pguptr->ol_stride);
  printf("0x78\tOL_SCN_LINE\t0x%08x\tOverlay start/end line - Start %d, End %d\n",pguptr->ol_scn_line,
	 (FRM_DIM_REG_FRM_WTH_MASK & pguptr->ol_scn_line),
	 (FRM_DIM_REG_FRM_LNS_MASK & (pguptr->ol_scn_line >> 16)));
  printf("0x80\tOL_COL_KEY\t0x%08x\tOverlay colour key\n",pguptr->ol_col_key);
#else
  printf("Overlay disabled\n");
#endif

#ifdef CONFIG_ENABLE_CENTERING
  printf("0x88\tDISPLAY_OFFSET\t0x%08x\tDisplay offset - X %d, Y %d\n",pguptr->disp_dim, 
	 pguptr->centreoffsets & DISP_DIM_REG_PGU_X_RES_MASK, ((pguptr->centreoffsets >> 16) & DISP_DIM_REG_PGU_X_RES_MASK));
#else
  printf("Display offset disabled\n");
#endif

  printf("\n\n");
}

//----------------------------------------------------------------------------------------------------

void  pgu_display_controller()
{
  arc_pgu pgu;

  if (ioctl(fbfd,  ARCPGUFB_GET_PGU_STATE, &pgu) == -1) {
    fprintf(stderr,"Can\'t display PGU: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
    return ;
  }

  pgu_dump_controller_regs(&pgu);
}

//----------------------------------------------------------------------------------------------------

// Raymond 2007/08/08

static int fd_time = -1;

int  pgu_open_fb_for_time()
{
   	if((fd_time = open(DEFAULT_FB, O_RDWR)) == -1) 
   	{
    		fprintf(stderr,"Can\'t open %s: %s\n", DEFAULT_FB, strerror(errno));
    		return -1;
  	}

  	return 0;
}

void  pgu_close_fb_for_time()
{
  	if (fd_time != -1) 
    		close(fd_time);
  	fd_time = -1;
}

void set_mplayer_time(int mplayer_time)
{
	int res;

	if (fd_time == -1) 
		return;
	
	res = ioctl(fd_time, ARCPGUFB_SET_MPLAYER_PLAY_TIME, &mplayer_time);
	
	if (res) 
	{
		fprintf(stderr,"set_mplayer_time failed\n");
    	}
}

// Raymond 2007/08/23
void set_mplayer_duration(int mplayer_duration)
{
	int res;

	if (fd_time == -1) 
		return;
	
	res = ioctl(fd_time, ARCPGUFB_SET_MPLAYER_DURATION, &mplayer_duration);
	
	if (res) 
	{
		fprintf(stderr,"set_mplayer_duration failed\n");
    	}
}

// Raymond 2008/05/20
void set_mplayer_status(int _status)
{
	int res;

	if (fd_time == -1) 
		return;
	
	res = ioctl(fd_time, ARCPGUFB_SET_MPLAYER_STATUS, &_status);
	
	if (res) 
	{
		fprintf(stderr,"set_mplayer_status failed\n");
    	}
}


//----------------------------------------------------------------------------------------------------
// Raymond 2007/11/23
int pgu_set_mplayer_advertisment(unsigned long _yesno)
{
	if (fd_time == -1) 
		return;

	if(ioctl(fd_time, ARCPGUFB_MPLAYER_ADVERTISMENT, &(_yesno)) == -1) 
	{
		fprintf(stderr, "ARCPGUFB_MPLAYER_ADVERTISMENT ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//-----------------------------------------------------------------------------
//Barry 2007-12-24
int pgu_show_mplayer_speed(char _time[])
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_VIDEO_SPEED, _time) == -1) 
	{
		fprintf("ARCPGUFB_SHOW_VIDEO_SPEED ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

int pgu_clear_mplayer_speed(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_VIDEO_SPEED) == -1) 
	{
		fprintf("ARCPGUFB_CLEAR_VIDEO_SPEED ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
//Barry 2008-01-07
int pgu_get_mplayer_time_on_off(void)
{
 	unsigned long _on;
 	
	if(ioctl(fbfd, ARCPGUFB_GET_VIDEO_TIME_ONOFF, &_on) == -1) 
	{
		fprintf("ARCPGUFB_GET_VIDEO_TIME_ONOFF ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return ((int)_on);
}

//-----------------------------------------------------------------------------
//Barry 2008-06-20
//int  audio_pgu_get_yuv_buffers()
int  pgu_get_yuv_buffers()		//Barry 2008-07-30
{	
  int i;
  
  if (ioctl(fbfd, ARCPGUFB_GETYUV, &mp_yuv) == -1) 
  {
    fprintf(stderr,"Can\'t get YUV info: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
    return -1;
  }
  return 0;
}
//----------------------------------------------------------------------------------------------------

#endif	//SK2850
