// vd_h264.c

#include <stdio.h>
#include <string.h>
#include <signal.h>

#include <fcntl.h>
#include <sys/mman.h>
#include <errno.h>


#include "vd_internal.h"
#include "../libvo/arc_pgu.h"
//#include "H264Decoder.h"
#include "arc_codecs.h"

#include <sys/mman.h>
#include <sys/types.h>

//static int InitOutput (void);
static void transmit (char *data,int sz);

//create the interface structure for this codec
//which contains pointers to the info struct above, and all below functions
//init, uninit, control, decode (in that order)
static vd_info_t info = {
  "H.264 Decoder", 	// codec long name
  "h264",		// codec short name (driver name in codecs.conf)
  "pbb",		// maintainer
  "pbb",		// author
  "test"		// additional comments
};
LIBVD_EXTERN(h264)

extern volatile void *memBase0;	//quick hack to get access to the virtual base address

//global static variables
static unsigned long *virtualBufferAddresses;
static unsigned long *physicalBufferAddresses;
static mp_image_t *get_next_frame (sh_video_t *);
static int initialised = 0;
static vd_frameinfo_t frameInfo = {0, NULL,get_next_frame};
static int fdMem = -1;
static void *SDM_StartAddr = (void *)-1;
static int waitForIDR = 0;
static sh_video_t *sh_ptr = NULL;
static arc_frame_data frameData;
static int flushedOutput = 0;
static mp_image_t *CreateMpiImage (sh_video_t *);

static char nalbuf[10240];
static int startlen;
static int startptr;
static int buflen;


static void SetSizes (unsigned long, unsigned long,int);
static int InitOutput (int);

#define BE_16(x) (((unsigned char *)(x))[0] <<  8 | \
		  ((unsigned char *)(x))[1])
#define BE_24(x) (((unsigned char *)(x))[0] << 16 | \
		  ((unsigned char *)(x))[1] <<  8 | \
		  ((unsigned char *)(x))[2])
#define BE_32(x) (((unsigned char *)(x))[0] << 24 | \
		  ((unsigned char *)(x))[1] << 16 | \
		  ((unsigned char *)(x))[2] <<  8 | \
		  ((unsigned char *)(x))[3])

/******************************************************************************
   Access to the external function interface structure
******************************************************************************/
extern arc_codec_functions h264_functions;


static void SetSizes (unsigned long w, unsigned long h, int num_buffers)
{
  int i;
  //check for initialisation
  if (initialised == 0)
  {
    fprintf(stderr,"\nInitialising display: %u:%u with %u buffers\n",w,h,num_buffers);    
    mpcodecs_config_vo(sh_ptr,w,h,IMGFMT_YV12); //YUV420 format
#if defined __ARC_SHARED_BUFFER__
    //set the addresses of all buffers
    if (!InitOutput(num_buffers))
      return;
#endif
    initialised = 1;
  }
}

// get the next frame from the saved image buffer (if present)
// form a mplayer image buffer and fill it with the data
// then update the structure for the remaining frames (if any)
static mp_image_t *get_next_frame (sh_video_t *sh)
{
  mp_image_t *mpi = NULL;
  //check that there are frames left
  if (frameInfo.numFrames == 0)
    return (NULL);
  frameInfo.numFrames--;
  //get an image buffer to copy the frame data to
  mpi = CreateMpiImage (sh);
  return (mpi);
}

static int control (sh_video_t *sh, int cmd, void *arg, ...)
{
  if (cmd == VDCTRL_RESYNC_STREAM)
    frameInfo.numFrames = 0;
  return (h264_functions.control(cmd,arg));
}

static int init (sh_video_t *sh)
{
  int i = 0;
  short s;
  unsigned long physaddr;
  struct yuv_info yuv;
  fprintf(stderr,"\nInit:\n");

  flushedOutput = 0;
  h264_functions.set_sizes = (fp_set_sizes)SetSizes;
  memset(&frameData,0,sizeof(arc_frame_data));
        

  fdMem = -1;
  //open a memory device to get access to the SDM memory
  fdMem = open("/dev/mem",O_RDWR|O_SYNC);
  if (fdMem < 0)
  {
    fprintf(stderr,"\nFailed to open /dev/mem");
    return (0);
  }
  SDM_StartAddr = mmap((void *)0,32768,PROT_READ|PROT_WRITE,MAP_SHARED,fdMem,0xC0F00000);
  if (SDM_StartAddr == (void *)-1)
  {
    fprintf(stderr,"\nFailed on mmap [%s]",strerror(errno));
    return (0);
  }
  fprintf(stderr,"\nMapped address is 0x%.8X",SDM_StartAddr);
  if (!h264_functions.set_buffer_addresses((unsigned long)SDM_StartAddr,NULL,NULL) ||
      !h264_functions.init (ARC_H264,NULL,0,0,0,0))
  {
    fprintf(stderr,"\nFailed to construct the decoder");
    return (0);
  }
  mpcodecs_vd_h264.frameInfo = &frameInfo;
  sh_ptr = sh;
  initialised = 0;
  startlen = 0;
  startptr = 0;
  buflen = 0;

  return (1);
}

static void uninit (sh_video_t *sh)
{
  int i;
  h264_functions.uninit();
  if (SDM_StartAddr != (void *)-1)
    munmap(SDM_StartAddr,32768);
  if (fdMem >= 0)
    close(fdMem);
  if (virtualBufferAddresses) 
      free(virtualBufferAddresses);
  if (physicalBufferAddresses)
      free(physicalBufferAddresses);
}

// The avcC header is pre-parsed owing to an awkward length field field in the midst of a set of NALs
static mp_image_t *decode_avc1_hdr(sh_video_t *sh, void *data, int len, int flags)
{
  int i,j,numframes = 0;
  int result;
  unsigned char *input = (unsigned char *)data;
  int index = 0;
  int startIndex = -1;
  int endIndex = -1;
  mp_image_t *mpi = NULL;
  struct demuxer_st *demuxer = sh->ds->demuxer;
  int declen;
 
  for (i = 0; i < demuxer->numNalOffsets; i++)
  {
    declen = demuxer->nalOffsets[i+1]-demuxer->nalOffsets[i]-2; // FIXME need to take into account nal length field length - varies between header and stream

    fprintf(stderr,"\n\n");
    fprintf(stderr,"\nFirst\n");
    for (j = demuxer->nalOffsets[i]; j < demuxer->nalOffsets[i]+4; j++)
      fprintf(stderr,"%02x ", input[j]);
    fprintf(stderr,"\nLast:\n");
    for (j = demuxer->nalOffsets[i]+declen-4; j < demuxer->nalOffsets[i]+declen; j++)
      fprintf(stderr,"%02x ", input[j]);
    fprintf(stderr,"\n");

    if ((result = h264_functions.decode(&input[demuxer->nalOffsets[i]],declen,0,&numframes, flags)) < 0)
    {
      fprintf(stderr,"\nDecode failed, return code is %d\n",result);
      return (NULL);
    }
    //check the number of decoded frames
    if (numframes > 0)
      frameInfo.numFrames += numframes;
  }
  demuxer->numNalOffsets = 0;
  free(demuxer->nalOffsets);
  demuxer->nalOffsets = NULL;
  demuxer->headerConsumed = 1;
}

// Length-based parsing for AVC1 format
static mp_image_t *decode_avc1(sh_video_t *sh, void *data, int len, int flags)
{
  int i,j,numframes = 0;
  int result;
  unsigned char *input = (unsigned char *)data;
  int index = 0;
  int startIndex = -1;
  int endIndex = -1;
  mp_image_t *mpi = NULL;
  struct demuxer_st *demuxer = sh->ds->demuxer;
  int decptr = 0;

  memcpy(nalbuf+buflen,input,len);
  buflen += len;

  while (len > 0)
  {
    int declen;

    if (startptr)
      declen = startlen;
    else
      switch(demuxer->nalLengthLen)
      {
        case 1:
	  declen = nalbuf[0];
	  decptr += 1;
	  break;
        case 2:
	  declen = BE_16(nalbuf);
	  decptr += 2;
	  break;
        case 3:
	  declen = BE_24(nalbuf);
	  decptr += 3;
	  break;
        case 4:
	  declen = BE_32(nalbuf);
	  decptr += 4;
	  break;
        default:
	  fprintf(stderr,"Unexpected NAL length value %d\n",demuxer->nalLengthLen);
	  break;
      }

    fprintf(stderr,"\n\n");
    fprintf(stderr,"\nFirst at %d:\n", decptr);
    for (j = decptr; j < decptr+4; j++)
      fprintf(stderr,"%02x ", input[j]);
    fprintf(stderr,"\nLast at %d (start %d + length %d):\n", decptr+declen, decptr, declen);
    for (j = decptr+declen-4; j < decptr+declen; j++)
      fprintf(stderr,"%02x ", input[j]);
    fprintf(stderr,"\n");
    
    if (decptr+declen > buflen) {
      fprintf(stderr,"****** Can\'t fully decode this NAL, need %d at %d, saving %d of %d bytes for next time ******\n",declen,decptr,buflen-decptr,buflen);
      //      memcpy(nalbuf,input+decptr,len-decptr);
      startlen = declen;
      break;
    }
    else
      startlen = 0;

    if ((result = h264_functions.decode(&(nalbuf[decptr]),declen,0,&numframes,flags)) < 0)
    {
      fprintf(stderr,"\nDecode failed, return code is %d\n",result);
      return (NULL);
    }
    //check the number of decoded frames
    if (numframes > 0)
      frameInfo.numFrames += numframes;
    decptr += declen;
    len -= declen;
  }

  if (decptr<buflen) {
    memcpy(nalbuf,nalbuf+decptr,buflen-decptr);
    buflen -= decptr;
  }
  else
    buflen = 0;
  startptr = buflen?1:0;
  buflen -= decptr;

  mpi = get_next_frame(sh);  
  return (mpi);  
}

static mp_image_t *decode (sh_video_t *sh, void *data, int len, int flags)
{
  int i,numframes = 0;
  int result;
  //fprintf(stderr,"\nIn the H.264 codec\n\n");
  unsigned char *input = (unsigned char *)data;
  //split down into individual NAL payloads
  int index = 0;
  int startIndex = -1;
  int endIndex = -1;
  int zeros = 0;
  mp_image_t *mpi = NULL;
  struct demuxer_st *demuxer = sh->ds->demuxer;


  //fprintf(stderr,"\nIn decode len = %d, %u %u %u %u ",len,input[0],input[1],input[2],input[3]);
  //check the length, if -ve, insert a dummy payload
  if (len >= 0)
  {
    int numNals = demuxer->numNalOffsets+1;

    if (demuxer->file_format == DEMUXER_TYPE_MOV)
    {
      if (! demuxer->headerConsumed) 
	return decode_avc1_hdr(sh,data,len,flags);
      else
	return decode_avc1(sh, data, len, flags);
    }

    if ((demuxer->file_format == DEMUXER_TYPE_RTP) && ((len >= 4) && (input[0] == 0) && (input[1] == 0) && (input[2] == 1)))
    {
      //need to find NALs in the provided data
      unsigned long value = 0x000000FF;
      //find the starter of a NAL
      int nalCount = 0;
      for (i=3; i<len; i++)
      {
        value <<= 8;
        value |= input[i];
        if ((value & 0x00FFFFFF) == 0x00000001)
	{
	  fprintf(stderr,"\nAdding index %u",i-2);
          if (nalCount != 0)
	  {
            demuxer->nalOffsets = (int *)malloc(sizeof(int));
            demuxer->nalOffsets[0] = i-3;
            demuxer->numNalOffsets = 1;
          }
          else
	  {
            int j;
            int *tmp = (int *)malloc((demuxer->numNalOffsets+1) * sizeof(int));
            for (j=0; j<demuxer->numNalOffsets;j++)
              tmp[j] = demuxer->nalOffsets[j];
            free(demuxer->nalOffsets);
            demuxer->nalOffsets = tmp;
            demuxer->nalOffsets[demuxer->numNalOffsets++] = i-2;
          }
          //found a start of a NAL
          nalCount++;
        }
      }
      numNals += nalCount;
    }
    //    fprintf(stderr,"\nNumNals is %u\n",numNals);
    if (numNals == 1)
    {
      //only one NAL, the offset is 3 into the buffer to allow for 0,0,0,1
      if ((len >= 4) && (input[0] == 0) && (input[1] == 0) && (input[2] == 1))
      {
/* 	fprintf(stderr,"001 found\n"); */
/* 	fprintf(stderr,"\nFirst\n"); */
/* 	for (i = 0; i < 4; i++) */
/* 	  fprintf(stderr,"%02x ", input[i+3]); */
/* 	fprintf(stderr,"\nLast:\n"); */
/* 	for (i = 4; i > 0; i--) */
/* 	  fprintf(stderr,"%02x ", input[len-i-3]); */
/* 	fprintf(stderr,"\n"); */
	
	result = h264_functions.decode(&input[3],len-3,0,&numframes, flags);
      }
      else
      {
/* 	fprintf(stderr,"001 NOT found\n"); */
/* 	fprintf(stderr,"\nFirst\n"); */
/* 	for (i = 0; i < 4; i++) */
/* 	  fprintf(stderr,"%02x ", input[i]); */
/* 	fprintf(stderr,"\nLast:\n"); */
/* 	for (i = 4; i > 0; i--) */
/* 	  fprintf(stderr,"%02x ", input[len-i]); */
/* 	fprintf(stderr,"\n"); */
	
	result = h264_functions.decode(input,len,0,&numframes, flags);
      }
      if (result < 0)
      {
        fprintf(stderr,"\nDecode failed, return code is %d\n",result);
        return (NULL);
      }
      //check the number of decoded frames
      if (numframes > 0)
        frameInfo.numFrames += numframes;
    }
    else
    {
      int i,j,incvalue;
      //multiple NALs
      startIndex = 3;
      for (i=0; i<numNals;i++)
      {
        incvalue = 0;
        if (i != (numNals-1))
	{
	  endIndex = demuxer->nalOffsets[i];
          if (input[endIndex-1] == 0)
	  {
            endIndex--;
            incvalue = 1;
	  }
        }
	else
	  endIndex = len;
        //
/* 	fprintf(stderr,"\nFirst (endindex is %u)\n",endIndex); */
/* 	for (j = startIndex; j < (startIndex+4); j++) */
/* 	  fprintf(stderr,"%02x ", input[j]); */
/* 	fprintf(stderr,"\nLast:\n"); */
/* 	for (j = 4; j > 0; j--) */
/* 	  fprintf(stderr,"%02x ", input[endIndex-j]); */
/*         //fprintf(stderr,"%02x ", input[endIndex+1]); */
/* 	fprintf(stderr,"\n"); */
        result = h264_functions.decode(&input[startIndex],endIndex-startIndex,0,&numframes, flags);
        if (result < 0)
        {
          fprintf(stderr,"\nDecode failed, return code is %d\n",result);
          return (NULL);
        }
        //check the number of decoded frames
        if (numframes > 0)
          frameInfo.numFrames += numframes;
        startIndex = endIndex + 3 + incvalue;
      }
      //clear the offset data
      demuxer->numNalOffsets = 0;
      free(demuxer->nalOffsets);
      demuxer->nalOffsets = NULL;
    }
  }
  else //if len is -ve insert a dummy payload to flush remaining frames
  {
    if (flushedOutput == 0)
    {
      //flush any remaining frames
      unsigned char endstream[1] = {11};
      unsigned char *udata = (unsigned char *)endstream;
      flushedOutput = 1;
      result = h264_functions.decode(endstream,1,0,&numframes, flags);
      if (result >= 0)
      {
        //check the number of decoded frames
        if (numframes > 0)
          frameInfo.numFrames += numframes;
      }
    }
    //otherwise the codec has already decoded a flush stream header, but there may be some more frames...
  }
  fprintf(stderr,"\nreturning from decode");
  //if (numframes > 1)
  //  fprintf(stderr,"\nGetNextFrame (%u)\n",frameInfo.numFrames);
  mpi = get_next_frame(sh);  
  return (mpi);
}


static mp_image_t *CreateMpiImage (sh_video_t *sh)
{

  mp_image_t * mpi = NULL;
#if defined __ARC_SHARED_BUFFER__
  mpi = (mp_image_t *)malloc(sizeof(mp_image_t));
  if (mpi == NULL)
  {
    fprintf(stderr,"\nFailed in Malloc for mpi");
    return (NULL);
  }
  memset(mpi,0,sizeof(mp_image_t));
  h264_functions.get_frame_data(&frameData);
  mpi->flags = MP_IMGFLAG_DIRECT | MP_IMGFLAG_PLANAR | MP_IMGFLAG_YUV;
  mpi->width = sh->disp_w;
  mpi->height = sh->disp_h;
  mpi->w = frameData.width;
  mpi->h = frameData.height;
  mpi->x = frameData.xOffset;
  mpi->y = frameData.yOffset;
  mpi->stride[0] = frameData.stride[0];
  mpi->stride[1] = frameData.stride[1];
  mpi->stride[2] = frameData.stride[2];
  mpi->planes[0] = frameData.planes[0];
  mpi->planes[1] = frameData.planes[1];
  mpi->planes[2] = frameData.planes[2];
  mpi->num_planes = 3;
  mpi->priv = (void *)frameData.index;
  //fprintf(stderr,"\nY: 0x%.8X, U: 0x%.8X,V: 0x%.8X",mpi->planes[0],mpi->planes[1],mpi->planes[2]);
#else
  mpi = mpcodecs_get_image(sh,MP_IMGTYPE_TEMP,(MP_IMGFLAG_PLANAR | MP_IMGFLAG_YUV), sh->disp_w, sh->disp_h);
  f_h264_copy_data(picture,mpi->planes[0]);
#endif

  //fprintf(stderr,"\nsh disp_w %u disp_h %u",sh->disp_w,sh->disp_h);

  return (mpi);
}

#if defined __ARC_SHARED_BUFFER__
static int InitOutput (int numBuffers)
{
  int i;
  short s;
  unsigned long physaddr;
  struct yuv_info yuv;
  //get the YUV buffer information
  int fdFrameDevice = open("/dev/fb0",O_RDWR);
  if (fdFrameDevice == -1)
  {
    fprintf(stderr,"arcpgufb: can't open /dev/fb0: %s\n", strerror(errno));
    return -1;
  }
  if (ioctl(fdFrameDevice, ARCPGUFB_GETYUV, &yuv) == -1)
  {
    fprintf(stderr,"\nH264: ARCPGUFB_GETYUV ioctl failed due to %s", strerror(errno));
    return (0);
  }
  //and the related physical address information
  physaddr = yuv.phys_base;
  // fprintf(stderr,"\nPhysical base is 0x%X",physaddr);
  //check the number of buffers is correct
  if (yuv.num_buffers < numBuffers)
  {
    fprintf(stderr,"\nH264: Number of driver buffers (%u) < %u",yuv.num_buffers,numBuffers);
    return (0);
  }
  //calculate the start addresses of all buffers
  virtualBufferAddresses = malloc((yuv.num_buffers+1) * sizeof(long));
  physicalBufferAddresses = malloc((yuv.num_buffers+1) * sizeof(long));
  for (i=0; i <yuv.num_buffers; i++)
  {
    int offset = ioctl(fdFrameDevice, ARCPGUFB_GET_YUV_BASE, i);
    virtualBufferAddresses[i] = (unsigned long)memBase0 + offset;
    physicalBufferAddresses[i] = physaddr + offset;
    //fprintf(stderr,"\nVirtual: 0x%.8X, Physical: 0x%.8X",virtualBufferAddresses[i],physicalBufferAddresses[i]);
  }
  //set the tail data (used for range checking)
  virtualBufferAddresses[yuv.num_buffers] = virtualBufferAddresses[yuv.num_buffers-1] + yuv.yuv_size;
  physicalBufferAddresses[yuv.num_buffers] = physicalBufferAddresses[yuv.num_buffers-1] + yuv.yuv_size;
  //initialise the codec memory controller
  h264_functions.set_buffer_addresses((unsigned long)SDM_StartAddr,virtualBufferAddresses,physicalBufferAddresses);
  close(fdFrameDevice);
  return (1);
}
#endif

