#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <errno.h>
#include <assert.h>

#include "vd_internal.h"
#include "../libvo/arc_pgu.h"
#include "arc_codecs.h"

#include "libvo/video_out.h"	// Raymond 2007/01/02 - for HW scalar	

static vd_info_t info = {
  "MS VC-1 Decoder", 	// codec long name
  "msvc1",		// codec short name (driver name in codecs.conf)
  "pbb",		// maintainer
  "pbb",		// author
  "test"		// additional comments
};

//create the interface structure for this codec
//which contains pointers to the info struct above, and all below functions
//init, uninit, control, decode (in that order)
LIBVD_EXTERN(msvc1)

// real codec size (filled by fileformat parser) - Raymond
extern int real_w;
extern int real_h;    

extern volatile void *memBase0;	//quick hack to get access to the virtual base address

static unsigned long *virtualBufferAddresses;
static unsigned long *physicalBufferAddresses;
static mp_image_t *get_next_frame (sh_video_t *);
static int initialised = 0;
static vd_frameinfo_t frameInfo = {0, NULL,get_next_frame};
static sh_video_t *streamHeader = NULL;
static int fdMem = -1;
static sh_video_t *sh_ptr = NULL;
static void *SDM_StartAddr = (void *)-1;
static arc_frame_data frameData;

extern int read_circular;   // Raymond 2007/10/02

static mp_image_t *CreateMpiImage (sh_video_t *);
static int InitOutput (void);
static void SetSizes (unsigned long w, unsigned long h);
int CallbackDataReader (int, unsigned char *, int);

/******************************************************************************
   Access to the external function interface structure
******************************************************************************/
extern arc_codec_functions vc1_functions;
#if defined __ARC_SHARED_BUFFER__
#undef __ARC_SHARED_BUFFER__
#endif

static void SetSizes (unsigned long w, unsigned long h)
{
  int i;
  //check for initialisation
  if (initialised == 0)
  {
//    fprintf(stderr,"\nInitialising display: %d:%d\n",w,h);
    mpcodecs_config_vo(sh_ptr,w,h,IMGFMT_YV12); //YUV420 format

    // Set decoder buffer resolution - Raymond 2007/01/02
    if( use_DecBuffer == 1 )
        SetDecBufferResolution(w, h);
	
#if defined __ARC_SHARED_BUFFER__
    //set the addresses of all buffers
    if (!InitOutput())
      return;
#endif
    initialised = 1;
  }
}

// get the next frame from the saved image buffer (if present)
// form a mplayer image buffer and fill it with the data
// then update the structure for the remaining frames (if any)
static mp_image_t *get_next_frame (sh_video_t *sh)
{
  mp_image_t *mpi = NULL;
  fprintf(stderr,"\nGetNextFrame");
  //check that there are frames left
  if (frameInfo.numFrames == 0)
    return (NULL);
   //get an image buffer to copy the frame data to
  mpi = CreateMpiImage (sh);
  if (mpi == NULL)
    return (NULL);
  //update the remaining frame information
  frameInfo.numFrames--;
  if (frameInfo.numFrames == 0)
    frameInfo.frameData = NULL;
  //else
  //  frameInfo.frameData = &frameInfo.frameData[1];
  return (mpi);
}

static int control (sh_video_t *sh, int cmd, void *arg, ...)
{
  return (vc1_functions.control(cmd,arg));
}

//Codec initialisation
static int init (sh_video_t *sh)
{
//  fprintf(stderr,"\nInit:\n");

  vc1_functions.read_callback = CallbackDataReader;
  vc1_functions.set_sizes = (fp_set_sizes)SetSizes;
  memset(&frameData,0,sizeof(arc_frame_data));

  streamHeader = sh;
  sh->context = (void *)-1;
  fdMem = -1;
  //open a memory device to get access to the SDM memory
  fdMem = open("/dev/mem",O_RDWR|O_SYNC);
  if (fdMem < 0)
  {
    fprintf(stderr,"\nFailed to open /dev/mem");
    return (0);
  }
  SDM_StartAddr = mmap((void *)0,32768,PROT_READ|PROT_WRITE,MAP_SHARED,fdMem,0xC0F00000);
  if (SDM_StartAddr == (void *)-1)
  {
    fprintf(stderr,"\nFailed on mmap [%s]",strerror(errno));
    return (0);
  }

	// Raymond 2004/04/16
	if( HW_Scalar == 1 )
	{  		
		int ret = InitPhyMem(fdMem);
		if (ret == 0)
      		{
       		fprintf(stderr,"\nFailed on InitPhyMem [%s]",  strerror(errno));
        		return (0);
      		}	
	}
  //
  initialised = 0;
  sh_ptr = sh;
  if (!vc1_functions.set_buffer_addresses((unsigned long)SDM_StartAddr,NULL,NULL) ||
      !vc1_functions.init (ARC_VC1,(unsigned char *)sh->ds->demuxer->header,sh->ds->demuxer->header_size,sh->ds->demuxer->header_offset,0,0))
    return (0);
  sh->context = (void *)NULL;
  sh_ptr = sh;
  mpcodecs_vd_msvc1.frameInfo = &frameInfo;
  return (1);
}

static void uninit (sh_video_t *sh)
{
//  fprintf(stderr,"\nUninit:\n");
  vc1_functions.uninit();
  if (SDM_StartAddr != (void *)-1)
    munmap(SDM_StartAddr,32768);

  // Raymond 2004/04/16
  if( HW_Scalar == 1 )
    ClosePhyMem();	
  
  if (fdMem >= 0)
    close(fdMem);
  if (virtualBufferAddresses) 
      free(virtualBufferAddresses);
  if (physicalBufferAddresses)
      free(physicalBufferAddresses);
}


static mp_image_t *decode (sh_video_t *sh, void *data, int len, int flags)
{
  int i;
  int numframes = 0;
  int result = 0;
  mp_image_t *mpi = NULL;
  streamHeader = sh;
  //
  if (len < 0)
  {
    frameInfo.numFrames = 0;
    return (NULL);
  }
  result = vc1_functions.decode(sh->ds->buffer,sh->ds->buffer_size,(int)(sh->ds->pos),&numframes, flags);
  if (result < 0)
  {
    fprintf(stderr,"\nDecode failed, return code is %d\n",result);
    return (NULL);
  }
  //check the number of decoded frames
  if (numframes > 0)
  {
    //check for initialisation
    if (initialised == 0)
    {
      fprintf(stderr,"\nUninitialised!");
      return (NULL);
    }
    //only generate the output frame buffer if not done so already
    mpi = CreateMpiImage(sh);
    if (mpi == NULL)
      return (NULL);
    //set the number of remaining frames
    numframes--;
    frameInfo.numFrames += numframes;
    if (numframes == 0)
      frameInfo.frameData = (void **)NULL;
  }
  return (mpi);
}

static mp_image_t *CreateMpiImage (sh_video_t *sh)
{
  mp_image_t * mpi = NULL;
#if defined __ARC_SHARED_BUFFER__
  mpi = (mp_image_t *)malloc(sizeof(mp_image_t));
  if (mpi == NULL)
  {
    fprintf(stderr,"\nFailed in Malloc for mpi");
    return (NULL);
  }
  memset(mpi,0,sizeof(mp_image_t));
  vc1_functions.get_frame_data(&frameData);
  mpi->flags = MP_IMGFLAG_DIRECT | MP_IMGFLAG_PLANAR | MP_IMGFLAG_YUV;
  mpi->width = sh->disp_w;
  mpi->height = sh->disp_h;
  mpi->w = frameData.width;
  mpi->h = frameData.height;
  mpi->x = frameData.xOffset;
  mpi->y = frameData.yOffset;
  mpi->stride[0] = frameData.stride[0];
  mpi->stride[1] = frameData.stride[1];
  mpi->stride[2] = frameData.stride[2];
  mpi->planes[0] = frameData.planes[0];
  mpi->planes[1] = frameData.planes[1];
  mpi->planes[2] = frameData.planes[2];
  mpi->num_planes = 3;
  mpi->priv = (void *)frameData.index;
#else
//  mpi = mpcodecs_get_image(sh,MP_IMGTYPE_TEMP,(MP_IMGFLAG_PLANAR | MP_IMGFLAG_YUV), sh->disp_w, sh->disp_h);
  mpi = mpcodecs_get_image(sh,MP_IMGTYPE_TEMP,(MP_IMGFLAG_PLANAR | MP_IMGFLAG_YUV), real_w, real_h);

#if 1	// Raymond 2008/06/27

	if( !((real_w & 0xf) || (real_h & 0xf)) )	
	{
		int ii = 0;
		if( use_DecBuffer == 1 )
		{
			ii = vbuf_dec_alloc();
			mpi->planes[0] = get_dec_yuv_bufN(ii);
			mpi->planes[1] = get_dec_bufN_U(ii);
			mpi->planes[2] = get_dec_bufN_V(ii);
		}
		else
		{
			ii = vbuf_alloc();
			mpi->planes[0] = get_bufN_Y(ii);
			mpi->planes[1] = get_bufN_U(ii);
			mpi->planes[2] = get_bufN_V(ii);
		}
//		printf("VBuf[%d] = %X %X %X\n", ii, mpi->planes[0], mpi->planes[1], mpi->planes[2]);
	}

#endif

  frameData.planes[0] = mpi->planes[0];
  frameData.planes[1] = mpi->planes[1];
  frameData.planes[2] = mpi->planes[2];
  vc1_functions.get_frame_data(&frameData);

#endif
  return (mpi);
}

#if defined __ARC_SHARED_BUFFER__
static int InitOutput (void)
{
  int i;
  short s;
  unsigned long physaddr;
  struct yuv_info yuv;
  fprintf(stderr,"\nInit Output");
  //get the YUV buffer information
  int fdFrameDevice = open("/dev/fb0",O_RDWR);
  if (fdFrameDevice == -1)
  {
    fprintf(stderr,"arcpgufb: can't open /dev/fb0: %s\n", strerror(errno));
    return -1;
  }
  if (ioctl(fdFrameDevice, ARCPGUFB_GETYUV, &yuv) == -1)
  {
    fprintf(stderr,"\nH264: ARCPGUFB_GETYUV ioctl failed due to %s", strerror(errno));
    return (0);
  }
  //and the related physical address information
  physaddr = yuv.phys_base;
  fprintf(stderr,"\nPhysical base is 0x%X, number of buffers is %d",physaddr,yuv.num_buffers);
  //calculate the start addresses of all buffers
  virtualBufferAddresses = malloc((yuv.num_buffers+1) * sizeof(long));
  physicalBufferAddresses = malloc((yuv.num_buffers+1) * sizeof(long));

  for (i=0; i < yuv.num_buffers; i++)
  {
    int offset = ioctl(fdFrameDevice, ARCPGUFB_GET_YUV_BASE, i);
    virtualBufferAddresses[i] = (unsigned long)memBase0 + offset;
    physicalBufferAddresses[i] = physaddr + offset;
    fprintf(stderr,"\nVirtual: 0x%.8X, Physical: 0x%.8X",virtualBufferAddresses[i],physicalBufferAddresses[i]);
  }
  //set the tail data (used for range checking)
  virtualBufferAddresses[yuv.num_buffers] = (unsigned long)memBase0 + yuv.yuv_size;
  physicalBufferAddresses[yuv.num_buffers] = physaddr + yuv.yuv_size;
  //initialise the codec memory controller
  vc1_functions.set_buffer_addresses((unsigned long)SDM_StartAddr,virtualBufferAddresses,physicalBufferAddresses);
  close(fdFrameDevice);
  return (1);
}
#endif

#if 1	// Raymond 2007/10/25

int CallbackDataReader (int fileOffset, unsigned char *buffer, int minSize)
{
  demux_stream_t *ds;
  demux_packet_t *packet;
  int bufferOffset = 0;
  int currentFileOffset, availableSize;
  unsigned char *bptr = NULL;
  
  //check the stream header is set
//  fprintf(stderr,"\nRead Offset:Size 0x%.8X:0x%.8X",fileOffset,minSize);
  if (streamHeader == NULL)
    return (0);
  //determine the demux stream
  ds = streamHeader->ds;

// Raymond 2007/10/02	  
if( read_circular == 1 )
{
	bptr = stream_seek_phy(ds->demuxer->stream, fileOffset);
	memcpy(&buffer[bufferOffset], bptr, minSize);
	return 1;
}	
else
{
  //search for a matching packet
  packet = ds->first;
  while (packet != NULL)
  {
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
      //found (at least a partial) match
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
      else
      {
        //read in what we can from this packet
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],availableSize);
	//and update the indices for the remaining data
	minSize -= availableSize;
	fileOffset += availableSize;
        bufferOffset += (fileOffset - packet->pos);
      }
    }
    packet = packet->next;
  }
  //check the current asf_packet as a 'last resort' before having to read the stream
  packet = ds->asf_packet;
  if (packet != NULL)
  {
    //fprintf(stderr,"\nASF Packet P:L 0x%.8X",packet->pos);
    //fprintf(stderr," 0x%.8X",packet->len);
    //fprintf(stderr," 0x%.8X",fileOffset);
    //if (fileOffset == (packet->pos + packet->len))
    //  fprintf(stderr," %u\n",packet->buffer[fileOffset - packet->pos]);
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
      //found (at least a partial) match
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
      //otherwise the info is just not available in mplayer...
    }
  }

  //force a read
//  fprintf(stderr,"\nASF Read Offset:Size 0x%.8X:0x%.8X",fileOffset,minSize);
  int fd = dup(ds->demuxer->stream->fd);
  if (fd == -1)
  {
    fprintf(stderr,"\ndup failed, %s",strerror(errno));
    return (0);
  }
  lseek(fd,fileOffset,SEEK_SET);
  int ret = (read(fd,buffer,minSize) == minSize)?1:0;
  close(fd);
  //return to the previous file offset
  lseek(ds->demuxer->stream->fd,ds->demuxer->stream->pos,SEEK_SET);
  //return 0 on failure
  
  return (ret);
}
}

#else
int CallbackDataReader (int fileOffset, unsigned char *buffer, int minSize)
{
  demux_stream_t *ds;
  demux_packet_t *packet;
  int bufferOffset = 0;
  int currentFileOffset, availableSize;
  //check the stream header is set
  //fprintf(stderr,"\nRead Offset:Size 0x%.8X:0x%.8X",fileOffset,minSize);
  if (streamHeader == NULL)
    return (0);
  //determine the demux stream
  ds = streamHeader->ds;
  //search for a matching packet
  packet = ds->first;
  while (packet != NULL)
  {
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
      //found (at least a partial) match
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
      else
      {
        //read in what we can from this packet
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],availableSize);
	//and update the indices for the remaining data
	minSize -= availableSize;
	fileOffset += availableSize;
        bufferOffset += (fileOffset - packet->pos);
      }
    }
    packet = packet->next;
  }
  //check the current asf_packet as a 'last resort' before having to read the stream
  packet = ds->asf_packet;
  if (packet != NULL)
  {
    //fprintf(stderr,"\nASF Packet P:L 0x%.8X",packet->pos);
    //fprintf(stderr," 0x%.8X",packet->len);
    //fprintf(stderr," 0x%.8X",fileOffset);
    //if (fileOffset == (packet->pos + packet->len))
    //  fprintf(stderr," %u\n",packet->buffer[fileOffset - packet->pos]);
    if ((fileOffset >= packet->pos) && (fileOffset < (packet->pos + packet->len)))
    {
      //found (at least a partial) match
      availableSize = packet->len - (fileOffset - packet->pos);
      if (availableSize >= minSize)
      {
        memcpy(&buffer[bufferOffset],&packet->buffer[fileOffset - packet->pos],minSize);
	return (1);
      }
      //otherwise the info is just not available in mplayer...
    }
  }
  //force a read
//  fprintf(stderr,"\nASF Read Offset:Size 0x%.8X:0x%.8X",fileOffset,minSize);
  int fd = dup(ds->demuxer->stream->fd);
  if (fd == -1)
  {
    fprintf(stderr,"\ndup failed, %s",strerror(errno));
    return (0);
  }
  lseek(fd,fileOffset,SEEK_SET);
  int ret = (read(fd,buffer,minSize) == minSize)?1:0;
  close(fd);
  //return to the previous file offset
  lseek(ds->demuxer->stream->fd,ds->demuxer->stream->pos,SEEK_SET);
  //return 0 on failure
  return (ret);
}
#endif	// Raymond 2007/10/25

