//  QuickTime MOV file parser by A'rpi
//  additional work by Atmos
//  based on TOOLS/movinfo.c by A'rpi & Al3x
//  compressed header support from moov.c of the openquicktime lib.
//  References: http://openquicktime.sf.net/, http://www.heroinewarrior.com/
//  http://www.geocities.com/SiliconValley/Lakes/2160/fformats/files/mov.pdf
//  (above url no longer works, file mirrored somewhere? ::atmos)
//  The QuickTime File Format PDF from Apple:
//  http://developer.apple.com/techpubs/quicktime/qtdevdocs/PDF/QTFileFormat.pdf
//  (Complete list of documentation at http://developer.apple.com/quicktime/)
//  MP4-Lib sources from http://mpeg4ip.sf.net/ might be usefull fot .mp4
//  aswell as .mov specific stuff.
//  All sort of Stuff about MPEG4:
//  http://www.cmlab.csie.ntu.edu.tw/~pkhsiao/thesis.html
//  I really recommend N4270-1.doc and N4270-2.doc which are exact specs
//  of the MP4-File Format and the MPEG4 Specific extensions. ::atmos

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "config.h"
#include "mp_msg.h"
#include "help_mp.h"

#include "stream.h"
#include "demuxer.h"
#include "stheader.h"

#include "bswap.h"

#include "qtpalette.h"
#include "parse_mp4.h" // .MP4 specific stuff

#ifdef __ARC_MPLAYER__
#include "salloc.h"
#endif

#ifdef MACOSX
#include <QuickTime/QuickTime.h>
#else
#include "../loader/qtx/qtxsdk/components.h"
#endif

#ifdef HAVE_ZLIB
#include <zlib.h>
#endif

#ifndef _FCNTL_H
#include <fcntl.h>
#endif
		
#define BE_16(x) (((unsigned char *)(x))[0] <<  8 | \
		  ((unsigned char *)(x))[1])
#define BE_32(x) (((unsigned char *)(x))[0] << 24 | \
		  ((unsigned char *)(x))[1] << 16 | \
		  ((unsigned char *)(x))[2] <<  8 | \
		  ((unsigned char *)(x))[3])

#define char2short(x,y)	BE_16(&(x)[(y)])
#define char2int(x,y) 	BE_32(&(x)[(y)])

#define MOV_FOURCC(a,b,c,d) ((a<<24)|(b<<16)|(c<<8)|(d))

//#define MOV_MAX_TRACKS 256
#define MOV_MAX_TRACKS 10		// Raymond 2008/03/06

#define MOV_TRAK_UNKNOWN 0
#define MOV_TRAK_VIDEO 1
#define MOV_TRAK_AUDIO 2
#define MOV_TRAK_FLASH 3
#define MOV_TRAK_GENERIC 4
#define MOV_TRAK_CODE 5

// Raymond 2008/03/20
//#define MOV_VIDEO_BUF_SIZE		262144	// 256 * 1024	
//#define MOV_AUDIO_BUF_SIZE	32768	// 32 * 1024	

// Raymond 2008/08/06
int MOV_VIDEO_BUF_SIZE = 262144;	// 256 * 1024	
int MOV_AUDIO_BUF_SIZE = 32768;		// 32 * 1024	

extern int get_preview;	// Raymond 2007/05/11

extern int audio_cache;	// Raymond 2007/07/23

// Raymond 2007/05/02
extern int avsync_mjpeg;	
//extern int mjpeg_skip;		
static int packet_cnt;	

// Raymond 2007/05/03 - skip mjpeg_skip1 per mjpeg_skip2 frame
extern int mjpeg_skip1;	
extern int mjpeg_skip2;

//Barry 2007-10-22 - fast forward / backward
extern int speed_mult;
static int forward_keyframe_idx=0;	
static int backward_keyframe_idx=-1;
extern int is_mjpeg;
int Only_One_I = 0;
int mov_keyframe_size = 0;	//Barry 2007-11-22

#define MOV_READ_PHY	// Raymond 2008/03/20

int mov_readphy = 0;	// Raymond 2008/03/19


#ifdef BUILD_SK2851	// Raymond 2008/08/20
extern int set_real_display;
extern int disp_x; 
extern int disp_y;
#endif

#if 0
typedef struct {
    unsigned int pts; // duration
    unsigned int size;
    off_t pos;
} mov_sample_t;
#endif
typedef struct {
    unsigned int sample; // number of the first sample in the chunk
    unsigned int size;   // number of samples in the chunk
//    int desc;            // for multiple codecs mode - not used
    off_t pos;
    unsigned char* buf;		// Raymond 2008/06/03
} mov_chunk_t;

typedef struct {
    unsigned int first;
    unsigned int spc;
//    unsigned int sdid;
} mov_chunkmap_t;

typedef struct {
    unsigned int num;
    unsigned int dur;
} mov_durmap_t;

typedef struct {
    unsigned int dur;
    unsigned int pos;
    int speed;
    //
    int frames;
    int start_sample;
    int start_frame;
    int pts_offset;
} mov_editlist_t;

typedef struct {
    int id;
    int type;
    off_t pos;
    //
    unsigned int media_handler;
    unsigned int data_handler;
    //
    int timescale;
    unsigned int length;
    int samplesize;  // 0 = variable
    int duration;    // 0 = variable
    int width,height; // for video
    unsigned int fourcc;
    unsigned int nchannels;
    unsigned int samplebytes;
    unsigned int max_sample_size;	// Raymond 2008/03/11	
    unsigned int total_sample_size;	// Raymond 2008/03/07	
    //
    int tkdata_len;  // track data 
    unsigned char* tkdata;
    int stdata_len;  // stream data
    unsigned char* stdata;
    //
    unsigned char* stream_header;
    int stream_header_len; // if >0, this header should be sent before the 1st frame
    //
    int samples_size;
    mov_sample_t* samples;
    int chunks_size;
    mov_chunk_t* chunks;
    int chunkmap_size;
    mov_chunkmap_t* chunkmap;
    int durmap_size;
    mov_durmap_t* durmap;
    int keyframes_size;
    unsigned int* keyframes;
    int editlist_size;
    mov_editlist_t* editlist;
    int editlist_pos;

    // Raymond 2008/03/10
//    unsigned char* trak_buf;
//    unsigned int trak_buf_pos;	
    //
    void* desc; // image/sound/etc description (pointer to ImageDescription etc)
} mov_track_t;

typedef struct {
    off_t moov_start;
    off_t moov_end;
    off_t mdat_start;
    off_t mdat_end;
    int track_db;    
    int timescale; // movie timescale
    int duration;  // movie duration (in movie timescale units)
    mov_track_t* tracks[MOV_MAX_TRACKS];
    // Raymond 2008/03/19	
    unsigned char *mov_vbuf;
    unsigned int mov_vbuf_pos;
    unsigned int mov_vbuf_len;
    unsigned char *mov_abuf;
    unsigned int mov_abuf_pos;
    unsigned int mov_abuf_len;
} mov_priv_t;

// Raymond 

static unsigned char *byteptr;
/* counter of how many bytes got written to the bitstream */
static int bytecnt; 

/* a one byte temporary buffer */
static unsigned char outbfr;

/* counter of how many unused bits left in the byte buffer */
static int outcnt;

static unsigned int mask[9] =
{
	0x00000000, 0x00000001, 0x00000003, 0x00000007,
	0x0000000f, 0x0000001f, 0x0000003f, 0x0000007f,
	0x000000ff
};

// Read N bits from bitstream buffer
unsigned int ReadBits( int nbits )
{
	unsigned int    l = 0 ;
	unsigned int	temp = 0; 

	int		dbit = nbits - outcnt;

	/* nbits <= outcnt */
	if( dbit < 0 )		
	{
		l = (*byteptr) & mask[outcnt] ; 
		l = l >> (-dbit); 
		outcnt = -dbit;
		return l;
	}
	else if ( dbit == 0 )	/* nbits == outcnt */
	{
		l = (*byteptr) & mask[outcnt] ; 

		byteptr++;
		bytecnt++;
		outcnt = 8;
		return l;
	}

	/* nbits > outcnt */
	while( dbit > 0 )
	{
		temp = ((*byteptr) & mask[outcnt]) ; 
		l |= temp << dbit;
					
		nbits = dbit;
		byteptr ++ ;
		bytecnt++;
		outcnt = 8;
		dbit -= 8;
	}
	
	/* final : nbit < 8 */
	temp = ((*byteptr) & mask[outcnt]) ; 
	temp = temp >> (-dbit); 
	l |= temp ;
	outcnt = (-dbit);

	if(outcnt == 0)
	{
		byteptr++;
		bytecnt++;
		outcnt = 8;
	}
	return l;
}

/* Give m4v VO/VOL header and return bitstream width, height */
int GetWidthHeight( unsigned char *bs, int *width, int *height,int buffer_len )
{
	unsigned int tmpvar;

	int i = 0;
	int Verid = 1;
	int TimeIncrementResolution = 0;
	int is_object_layer_identifier = 0;

	bytecnt = 0;
	outbfr	= 0;
	outcnt	= 8;

	for( i = 0 ; i < buffer_len-4 ; i++ )
		if( bs[i] == 0 && bs[i+1] == 0 && bs[i+2] == 1 )
//			if( bs[i+3] >= 0x0 && bs[i+3] < 0x20 )
			if( bs[i+3] < 0x20 )
				break;

//	printf("VO/VOL start at [%d] bytes\n", i);	
	if( i == buffer_len  - 4 )
		return -1;
	
	byteptr = bs + i;
	
	tmpvar = ReadBits(32);
	
	// check VO_START_CODE
	if( tmpvar < 0x100 || tmpvar >= 0x120 )
	{
//		printf("tmpvar = %d\n", tmpvar);	
		return -2;
	}
	
	tmpvar = ReadBits(32);		// VOL_START_CODE
//	if( tmpvar != 0x120 )
	if( tmpvar < 0x120 || tmpvar > 0x12f)	// Raymond 2007/12/19
		return -3;
	
	ReadBits(9);
												
	is_object_layer_identifier = ReadBits(1);	// is_object_layer_identifier ( 1 bit )
	if( is_object_layer_identifier == 1 )
	{
		Verid = ReadBits(4);						
		ReadBits(3);
	}
		
	tmpvar = ReadBits(4);	// insert aspect_ratio_info ( 4 bits ) 
	if( tmpvar == 0xF )
		ReadBits(16);

	tmpvar = ReadBits(1);	// vol_control_parameters ( 1 bit )
	if( tmpvar == 1 )
	{
		ReadBits(3);
		tmpvar = ReadBits(1);	
		if( tmpvar == 1 )
			ReadBits(79);
	}

	tmpvar = ReadBits(2);	// vol_shape (2 bits) 
	if( tmpvar != 0 )
		return -4;			// No support Shape
	
	ReadBits(1);

	TimeIncrementResolution = ReadBits(16);
	
	ReadBits(1);
	
	tmpvar = ReadBits(1);	// fixed_vop_rate ( 1 bit )
	if( tmpvar == 1 )
	{
		int j;
		int p2;		
		int bits = 1;
											
		for( j = 1 ; j <= 16 ; j++ )
		{
			p2 = 1 << j;
			if( TimeIncrementResolution > p2 )
				bits ++;
		}			
		
//		printf("T bits = %d\n", bits );
		ReadBits(bits);
	}
	
	ReadBits(1);

	*width = ReadBits(13);	// vol_width (13 bits) 

	ReadBits(1);

	*height = ReadBits(13);	// vol_height (13 bits) 
	
	return 1;
}

void mov_build_index(mov_track_t* trak,int timescale){
    int i,j,s, cs;
    int last=trak->chunks_size;
    unsigned int pts=0;

#if 0
    if (trak->chunks_size <= 0)
    {
	mp_msg(MSGT_DEMUX, MSGL_WARN, "No chunk offset table, trying to build one!\n");
	
	trak->chunks_size = trak->samples_size; /* XXX: FIXME ! */
	trak->chunks = realloc(trak->chunks, sizeof(mov_chunk_t)*trak->chunks_size);
	
	for (i=0; i < trak->chunks_size; i++)
	    trak->chunks[i].pos = -1;
    }
#endif

    mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV track #%d: %d chunks, %d samples\n",trak->id,trak->chunks_size,trak->samples_size);
    mp_msg(MSGT_DEMUX, MSGL_V, "pts=%d  scale=%d  time=%5.3f\n",trak->length,trak->timescale,(float)trak->length/(float)trak->timescale);

    // process chunkmap:
    i=trak->chunkmap_size;
    while(i>0){
	--i;
	for(j=trak->chunkmap[i].first;j<last;j++){
//	    trak->chunks[j].desc=trak->chunkmap[i].sdid;
	    trak->chunks[j].size=trak->chunkmap[i].spc;
	}
	last=trak->chunkmap[i].first;
    }

#if 0
    for (i=0; i < trak->chunks_size; i++)
    {
	/* fixup position */
	if (trak->chunks[i].pos == -1)
	    if (i > 0)
		trak->chunks[i].pos = trak->chunks[i-1].pos + trak->chunks[i-1].size;
	    else
		trak->chunks[i].pos = 0; /* FIXME: set initial pos */
#endif

    // calc pts of chunks:
    s=0;
    for(j=0;j<trak->chunks_size;j++){
        trak->chunks[j].sample=s;
        s+=trak->chunks[j].size;
    }

    // workaround for fixed-size video frames (dv and uncompressed)
    if(!trak->samples_size && trak->type!=MOV_TRAK_AUDIO){
	trak->samples_size=s;
	// Raymond 2008/03/06
	if(audio_cache == 0)
		trak->samples=salloc(sizeof(mov_sample_t)*s);
	else
		trak->samples=malloc(sizeof(mov_sample_t)*s);
	for(i=0;i<s;i++)
	    trak->samples[i].size=trak->samplesize;
	trak->samplesize=0;
    }

    if(!trak->samples_size){
	// constant sampesize
	if(trak->durmap_size==1 || (trak->durmap_size==2 && trak->durmap[1].num==1)){
	    trak->duration=trak->durmap[0].dur;
	} else mp_msg(MSGT_DEMUX, MSGL_ERR, "*** constant samplesize & variable duration not yet supported! ***\nContact the author if you have such sample file!\n");
	return;
    }

    // calc pts:
    s=0;
    for(j=0;j<trak->durmap_size;j++)
    {
	for(i=0;i<trak->durmap[j].num;i++)
	{
	    if(s < trak->samples_size)	// Raymond 2006/01/10     	
	    trak->samples[s].pts=pts;
	    ++s;
	    pts+=trak->durmap[j].dur;
	}
    }

    // calc sample offsets
    s=0;
    for(j=0;j<trak->chunks_size;j++){
	off_t pos=trak->chunks[j].pos;
	mp_msg(MSGT_DEMUX, MSGL_DBG3, "Chunk %d : size %d\n", j, trak->chunks[j].size);
	cs = 0;
	for(i=0;i<trak->chunks[j].size;i++){
	    trak->samples[s].pos=pos;
	    mp_msg(MSGT_DEMUX, MSGL_DBG3, "Sample %5d: pts=%8d  off=0x%08X  size=%d\n",s,
		trak->samples[s].pts,
		(int)trak->samples[s].pos,
		trak->samples[s].size);
	    pos+=trak->samples[s].size;
	    cs += 	trak->samples[s].size;
           // Raymond 2008/03/11
           if( trak->samples[s].size > trak->max_sample_size )
		   	trak->max_sample_size = trak->samples[s].size;
		   	
	    ++s;
	}
//	mp_msg(MSGT_DEMUX, MSGL_V, "Chunk %d : size %d, len = %d\n", j, trak->chunks[j].size, cs);
    }

    mp_msg(MSGT_DEMUX, MSGL_V, "trak->max_sample_size = %d\n", trak->max_sample_size);	// Raymond 2008/03/11

    // precalc editlist entries
    if(trak->editlist_size>0){
	int frame=0;
	int e_pts=0;
	for(i=0;i<trak->editlist_size;i++){
	    mov_editlist_t* el=&trak->editlist[i];
	    int sample=0;
	    int pts=el->pos;
	    el->start_frame=frame;
	    if(pts<0){
		// skip!
		el->frames=0; continue;
	    }
	    // find start sample
	    for(;sample<trak->samples_size;sample++){
		if(pts<=trak->samples[sample].pts) break;
	    }
	    el->start_sample=sample;
	    el->pts_offset=((long long)e_pts*(long long)trak->timescale)/(long long)timescale-trak->samples[sample].pts;
	    pts+=((long long)el->dur*(long long)trak->timescale)/(long long)timescale;
	    e_pts+=el->dur;
	    // find end sample
	    for(;sample<trak->samples_size;sample++){
		if(pts<=trak->samples[sample].pts) break;
	    }
	    el->frames=sample-el->start_sample;
	    frame+=el->frames;
	    mp_msg(MSGT_DEMUX,MSGL_V,"EL#%d: pts=%d  1st_sample=%d  frames=%d (%5.3fs)  pts_offs=%d\n",i,
		el->pos,el->start_sample, el->frames,
		(float)(el->dur)/(float)timescale, el->pts_offset);
	}
    }

}


int mov_check_file(demuxer_t* demuxer){
    int flags=0;
    int no=0;
    mov_priv_t* priv=malloc(sizeof(mov_priv_t));
    
    mp_msg(MSGT_DEMUX,MSGL_V,"Checking for MOV\n");
    
    memset(priv,0,sizeof(mov_priv_t));
    
    while(1){
	int i;
	int skipped=8;
	off_t len=stream_read_dword(demuxer->stream);
	unsigned int id=stream_read_dword(demuxer->stream);
	if(stream_eof(demuxer->stream)) break; // EOF
	if (len == 1) /* real size is 64bits - cjb */
	{
#ifndef _LARGEFILE_SOURCE
	    if (stream_read_dword(demuxer->stream) != 0)
		mp_msg(MSGT_DEMUX, MSGL_WARN, "64bit file, but you've compiled MPlayer without LARGEFILE support!\n");
	    len = stream_read_dword(demuxer->stream);
#else
	    len = stream_read_qword(demuxer->stream);
#endif
	    skipped += 8;
	}
#if 0
	else if (len == 0) /* deleted chunk */
	{
	    /* XXX: CJB! is this right? - alex */
	    goto skip_chunk;
	}
#endif
	else if(len<8) break; // invalid chunk

//	printf("ID is 0x%x, len is %d\n",id,len);

	switch(id){
	case MOV_FOURCC('f','t','y','p'): {
	  unsigned int tmp;
	  // File Type Box (ftyp): 
	  // char[4]  major_brand	   (eg. 'isom')
	  // int      minor_version	   (eg. 0x00000000)
	  // char[4]  compatible_brands[]  (eg. 'mp41')
	  // compatible_brands list spans to the end of box
#if 1
	  tmp = stream_read_dword(demuxer->stream);
	  switch(tmp) {
	    case MOV_FOURCC('i','s','o','m'):
	      mp_msg(MSGT_DEMUX,MSGL_V,"ISO: File Type Major Brand: ISO Base Media\n");
     	      break;
	    case MOV_FOURCC('m','p','4','1'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: ISO/IEC 14496-1 (MPEG-4 system) v1\n");
     	      break;
	    case MOV_FOURCC('m','p','4','2'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: ISO/IEC 14496-1 (MPEG-4 system) v2\n");
     	      break;
	    case MOV_FOURCC('M','4','A',' '):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: Apple iTunes AAC-LC Audio\n");
     	      break;
	    case MOV_FOURCC('M','4','P',' '):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: Apple iTunes AAC-LC Protected Audio\n");
     	      break;
	    case MOV_FOURCC('q','t',' ',' '):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: Original QuickTime\n");
     	      break;
	    case MOV_FOURCC('3','g','p','1'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: 3GPP Profile 1\n");
     	      break;
	    case MOV_FOURCC('3','g','p','2'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: 3GPP Profile 2\n");
     	      break;
	    case MOV_FOURCC('3','g','p','3'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: 3GPP Profile 3\n");
     	      break;
	    case MOV_FOURCC('3','g','p','4'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: 3GPP Profile 4\n");
     	      break;
	    case MOV_FOURCC('3','g','p','5'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: 3GPP Profile 5\n");
     	      break;
	    case MOV_FOURCC('m','m','p','4'):
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"ISO: File Type Major Brand: Mobile ISO/IEC 14496-1 (MPEG-4 system)\n");
     	      break;
	    default:
	      tmp = be2me_32(tmp);  
	      mp_msg(MSGT_DEMUX,MSGL_WARN,"ISO: Unknown File Type Major Brand: %.4s\n",&tmp);
	  }
	  mp_msg(MSGT_DEMUX,MSGL_V,"ISO: File Type Minor Version: %d\n",
	      stream_read_dword(demuxer->stream));
	  skipped += 8;
	  // List all compatible brands
	  for(i = 0; i < ((len-16)/4); i++) {
	    //tmp = be2me_32(stream_read_dword(demuxer->stream));
	    tmp = stream_read_dword(demuxer->stream);
	    tmp = be2me_32(tmp);
	    mp_msg(MSGT_DEMUX,MSGL_V,"ISO: File Type Compatible Brand #%d: %.4s\n",i,&tmp);
//	    printf("ISO: File Type Compatible Brand #%d: %.4s (value 0x%x, len = %d, stream pos %d)\n",i,&tmp,tmp,len, demuxer->stream->buf_pos);
	    skipped += 4;
	  }
#endif	  
	  } break;
	case MOV_FOURCC('m','o','o','v'):
//	case MOV_FOURCC('c','m','o','v'):
	  mp_msg(MSGT_DEMUX,MSGL_V,"MOV: Movie header found!\n");
	  priv->moov_start=(off_t)stream_tell(demuxer->stream);
	  priv->moov_end=(off_t)priv->moov_start+len-skipped;
	  mp_msg(MSGT_DEMUX,MSGL_DBG2,"MOV: Movie header: start: %x end: %x\n",
	    priv->moov_start, priv->moov_end);
	  skipped+=8;
	  i = stream_read_dword(demuxer->stream)-8;
	  if(stream_read_dword(demuxer->stream)==MOV_FOURCC('r','m','r','a')){
	      int ref=0;
	      skipped+=i;
	      mp_msg(MSGT_DEMUX,MSGL_INFO,"MOV: Reference Media file!!!\n");
	      //set demuxer type to playlist ...
	      demuxer->type=DEMUXER_TYPE_PLAYLIST;
	      while(i>0){
	          int len=stream_read_dword(demuxer->stream)-8;
		  int fcc=stream_read_dword(demuxer->stream);
		  if(len<0) break; // EOF!?
		  i-=8;
//		  printf("i=%d  len=%d\n",i,len);
		  switch(fcc){
		  case MOV_FOURCC('r','m','d','a'):
		      continue;
		  case MOV_FOURCC('r','d','r','f'): {
		      int tmp=stream_read_dword(demuxer->stream);
		      int type=stream_read_dword_le(demuxer->stream);
	              int slen=stream_read_dword(demuxer->stream);
		      //char* s=malloc(slen+1);
		      //stream_read(demuxer->stream,s,slen);
		      
		      //FIXME: also store type & data_rate ?
		      ds_read_packet(demuxer->video, 
			demuxer->stream,
			slen,
			0,
			stream_tell(demuxer->stream),
			0 // no flags 
		      );
		      flags|=4;
		      mp_msg(MSGT_DEMUX,MSGL_V,"Added reference to playlist\n");
		      //s[slen]=0;
		      //mp_msg(MSGT_DEMUX,MSGL_INFO,"REF: [%.4s] %s\n",&type,s);
		      len-=12+slen;i-=12+slen; break;
		    }
		  case MOV_FOURCC('r','m','d','r'): {
		      int flags=stream_read_dword(demuxer->stream);
		      int rate=stream_read_dword(demuxer->stream);
		      mp_msg(MSGT_DEMUX,MSGL_V,"  min. data rate: %d bits/sec\n",rate);
		      len-=8; i-=8; break;
		    }
		  case MOV_FOURCC('r','m','q','u'): {
		      int q=stream_read_dword(demuxer->stream);
		      mp_msg(MSGT_DEMUX,MSGL_V,"  quality index: %d\n",q);
		      len-=4; i-=4; break;
		    }
		  }
		  i-=len;stream_skip(demuxer->stream,len);
	      }
	  }
	  flags|=1;
	  break;
	case MOV_FOURCC('w','i','d','e'):
	  mp_msg(MSGT_DEMUX,MSGL_V,"MOV: 'WIDE' chunk found!\n");
	  if(flags&2) break;
	case MOV_FOURCC('m','d','a','t'):
	  mp_msg(MSGT_DEMUX,MSGL_V,"MOV: Movie DATA found!\n");
	  priv->mdat_start=stream_tell(demuxer->stream);
	  priv->mdat_end=priv->mdat_start+len-skipped;
	  mp_msg(MSGT_DEMUX,MSGL_DBG2,"MOV: Movie data: start: %x end: %x\n",
	    priv->mdat_start, priv->mdat_end);
//	  fprintf(stderr,"MOV: Movie data: start: %x end: %x\n",
//	    priv->mdat_start, priv->mdat_end);
	  flags|=2;
	  if(flags==3){
	    // if we're over the headers, then we can stop parsing here!
	    demuxer->priv=priv;
	    return 1;
	  }
	  break;
	case MOV_FOURCC('f','r','e','e'):
	case MOV_FOURCC('s','k','i','p'):
	case MOV_FOURCC('j','u','n','k'):
	  mp_msg(MSGT_DEMUX,MSGL_DBG2,"MOV: free space (len: %d)\n", len);
	  /* unused, if you edit a mov, you can use space provided by free atoms (redefining it) */
	  break;
	case MOV_FOURCC('p','n','o','t'):
	case MOV_FOURCC('P','I','C','T'):
	  /* dunno what, but we shoudl ignore it */
	  break;
	default:
	  if(no==0){ free(priv); return 0;} // first chunk is bad!
	  id = be2me_32(id);
	  mp_msg(MSGT_DEMUX,MSGL_V,"MOV: unknown chunk: %.4s %d\n",&id,(int)len);
	}
skip_chunk:
	if(!stream_skip(demuxer->stream,len-skipped)) break;
	++no;
    }

    if(flags==3){
	demuxer->priv=priv;
	return 1;
    }
    free(priv);

    if ((flags==5) || (flags==7)) // reference & header sent
        return 1;

    if(flags==1)
	mp_msg(MSGT_DEMUX,MSGL_WARN,"MOV: missing data (mdat) chunk! Maybe broken file...\n");
    else if(flags==2)
	mp_msg(MSGT_DEMUX,MSGL_WARN,"MOV: missing header (moov/cmov) chunk! Maybe broken file...\n");

    return 0;
}

static void lschunks(demuxer_t* demuxer,int level,off_t endpos,mov_track_t* trak){
    mov_priv_t* priv=demuxer->priv;
//    printf("lschunks (level=%d,endpos=%x)\n", level, endpos);
    while(1){
	off_t pos;
	off_t len;
	unsigned int id;
	//
	pos=stream_tell(demuxer->stream);
//	printf("stream_tell==%d\n",pos);
	if(pos>=endpos) return; // END
	len=stream_read_dword(demuxer->stream);
//	printf("len==%d\n",len);
	if(len<8) return; // error
	len-=8;
	id=stream_read_dword(demuxer->stream);
	//
	mp_msg(MSGT_DEMUX,MSGL_DBG2,"lschunks %.4s  %d\n",&id,(int)len);
	//
	if(trak){
	  switch(id){
	    case MOV_FOURCC('m','d','a','t'): {
		mp_msg(MSGT_DEMUX,MSGL_WARN,"Hmm, strange MOV, parsing mdat in lschunks?\n");
		return;
	    }
	    case MOV_FOURCC('f','r','e','e'):
	    case MOV_FOURCC('u','d','t','a'):
		/* here not supported :p */
		break;
	    case MOV_FOURCC('t','k','h','d'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sTrack header!\n",level,"");
		// read codec data
		trak->tkdata_len=len;
		trak->tkdata=salloc(trak->tkdata_len);
		stream_read(demuxer->stream,trak->tkdata,trak->tkdata_len);
/*
0  1 Version
1  3 Flags
4  4 Creation time
8  4 Modification time
12 4 Track ID
16 4 Reserved
20 4 Duration
24 8 Reserved
32 2 Layer
34 2 Alternate group
36 2 Volume
38 2 Reserved
40 36 Matrix structure
76 4 Track width
80 4 Track height
*/
		mp_msg(MSGT_DEMUX,MSGL_V,"tkhd len=%d ver=%d flags=0x%X id=%d dur=%d lay=%d vol=%d\n",
		    trak->tkdata_len, trak->tkdata[0], trak->tkdata[1],
		    char2int(trak->tkdata,12), // id
		    char2int(trak->tkdata,20), // duration
		    char2short(trak->tkdata,32), // layer
		    char2short(trak->tkdata,36)); // volume
		break;
	    }
	    case MOV_FOURCC('m','d','h','d'): {
		unsigned int tmp;
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sMedia header!\n",level,"");
#if 0
		tmp=stream_read_dword(demuxer->stream);
		printf("dword1: 0x%08X (%d)\n",tmp,tmp);
		tmp=stream_read_dword(demuxer->stream);
		printf("dword2: 0x%08X (%d)\n",tmp,tmp);
		tmp=stream_read_dword(demuxer->stream);
		printf("dword3: 0x%08X (%d)\n",tmp,tmp);
		tmp=stream_read_dword(demuxer->stream);
		printf("dword4: 0x%08X (%d)\n",tmp,tmp);
		tmp=stream_read_dword(demuxer->stream);
		printf("dword5: 0x%08X (%d)\n",tmp,tmp);
		tmp=stream_read_dword(demuxer->stream);
		printf("dword6: 0x%08X (%d)\n",tmp,tmp);
#endif
		stream_skip(demuxer->stream,12);
		// read timescale
		trak->timescale=stream_read_dword(demuxer->stream);
		// read length
		trak->length=stream_read_dword(demuxer->stream);
		break;
	    }
	    case MOV_FOURCC('h','d','l','r'): {
		unsigned int tmp=stream_read_dword(demuxer->stream);
		unsigned int type=stream_read_dword_le(demuxer->stream);
		unsigned int subtype=stream_read_dword_le(demuxer->stream);
		unsigned int manufact=stream_read_dword_le(demuxer->stream);
		unsigned int comp_flags=stream_read_dword(demuxer->stream);
		unsigned int comp_mask=stream_read_dword(demuxer->stream);
		int len=stream_read_char(demuxer->stream);
		char* str=malloc(len+1);
		stream_read(demuxer->stream,str,len);
		str[len]=0;
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sHandler header: %.4s/%.4s (%.4s) %s\n",level,"",&type,&subtype,&manufact,str);
		free(str);
		switch(bswap_32(type)){
		case MOV_FOURCC('m','h','l','r'):
		    trak->media_handler=bswap_32(subtype); break;
		case MOV_FOURCC('d','h','l','r'):
		    trak->data_handler=bswap_32(subtype); break;
		default:
		    mp_msg(MSGT_DEMUX,MSGL_V,"MOV: unknown handler class: 0x%X (%.4s)\n",bswap_32(type),&type);
		}
		break;
	    }
	    case MOV_FOURCC('v','m','h','d'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sVideo header!\n",level,"");
		trak->type=MOV_TRAK_VIDEO;
		// read video data
		break;
	    }
	    case MOV_FOURCC('s','m','h','d'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sSound header!\n",level,"");
		trak->type=MOV_TRAK_AUDIO;
		// read audio data
		break;
	    }
	    case MOV_FOURCC('g','m','h','d'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sGeneric header!\n",level,"");
		trak->type=MOV_TRAK_GENERIC;
		break;
	    }
	    case MOV_FOURCC('n','m','h','d'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sGeneric header!\n",level,"");
		trak->type=MOV_TRAK_GENERIC;
		break;
	    }
	    case MOV_FOURCC('s','t','s','d'): {
		int i=stream_read_dword(demuxer->stream); // temp!
		int count=stream_read_dword(demuxer->stream);
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sDescription list! (cnt:%d)\n",level,"",count);
		for(i=0;i<count;i++){
		    off_t pos=stream_tell(demuxer->stream);
		    off_t len=stream_read_dword(demuxer->stream);
		    unsigned int fourcc=stream_read_dword_le(demuxer->stream);
		    /* some files created with Broadcast 2000 (e.g. ilacetest.mov)
		       contain raw I420 video but have a yv12 fourcc */
		    if(fourcc==mmioFOURCC('y','v','1','2')) fourcc=mmioFOURCC('I','4','2','0');
		    if(len<8) break; // error
		    mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*s desc #%d: %.4s  (%d bytes)\n",level,"",i,&fourcc,len-16);
		    if(fourcc!=trak->fourcc && i)
			mp_msg(MSGT_DEMUX,MSGL_WARN,MSGTR_MOVvariableFourCC);
//		    if(!i)	// Raymond 2007/06/05
		    {
			trak->fourcc=fourcc;
			// read type specific (audio/video/time/text etc) header
			// NOTE: trak type is not yet known at this point :(((
			trak->stdata_len=len-8;
			trak->stdata=salloc(trak->stdata_len);	// Raymond 2007/07/03
			stream_read(demuxer->stream,trak->stdata,trak->stdata_len);
		    }
		    if(!stream_seek(demuxer->stream,pos+len)) break;
		}
		break;
	    }
	    case MOV_FOURCC('s','t','t','s'): {
		int temp=stream_read_dword(demuxer->stream);
		int len=stream_read_dword(demuxer->stream);
		int i;
		unsigned int pts=0;
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sSample duration table! (%d blocks)\n",level,"",len);
		trak->durmap=salloc(sizeof(mov_durmap_t)*len);	// Raymond 2007/07/03
		memset(trak->durmap,0,sizeof(mov_durmap_t)*len);
		trak->durmap_size=len;
		for(i=0;i<len;i++){
		    trak->durmap[i].num=stream_read_dword(demuxer->stream);
		    trak->durmap[i].dur=stream_read_dword(demuxer->stream);
		    pts+=trak->durmap[i].num*trak->durmap[i].dur;
		}
		if(trak->length!=pts) mp_msg(MSGT_DEMUX, MSGL_WARN, "Warning! pts=%d  length=%d\n",pts,trak->length);
		break;
	    }
	    case MOV_FOURCC('s','t','s','c'): {
		int temp=stream_read_dword(demuxer->stream);
		int len=stream_read_dword(demuxer->stream);
		int ver = (temp << 24);
		int flags = (temp << 16)|(temp<<8)|temp;
		int i;
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sSample->Chunk mapping table!  (%d blocks) (ver:%d,flags:%ld)\n",
		    level,"",len,ver,flags);
		// read data:
		trak->chunkmap_size=len;
		trak->chunkmap=salloc(sizeof(mov_chunkmap_t)*len);	// Raymond 2007/07/03
		for(i=0;i<len;i++){
		    trak->chunkmap[i].first=stream_read_dword(demuxer->stream)-1;
		    trak->chunkmap[i].spc=stream_read_dword(demuxer->stream);
		    // Raymond 2008/03/7
		    stream_read_dword(demuxer->stream);	
//		    trak->chunkmap[i].sdid=stream_read_dword(demuxer->stream);
		}
		break;
	    }
	    case MOV_FOURCC('s','t','s','z'): {
		int temp=stream_read_dword(demuxer->stream);
		int ss=stream_read_dword(demuxer->stream);
		int ver = (temp << 24);
		int flags = (temp << 16)|(temp<<8)|temp;
		int entries=stream_read_dword(demuxer->stream);
		int i;

		// Raymond 2007/05/11
		//if( get_preview == 1 && trak->type == MOV_TRAK_AUDIO )
		if( get_preview == 1 && trak->type == MOV_TRAK_AUDIO && ss == 0 )	//Barry 2008-08-28
		{
			stream_skip(demuxer->stream, entries << 2);
			break;
		}	
				
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sSample size table! (entries=%d ss=%d) (ver:%d,flags:%ld)\n",
		    level,"",entries,ss,ver,flags);
		trak->samplesize=ss;
		if (!ss) {
		  // variable samplesize
#if 1	// Raymond 2007/07/03	  
		  if(trak->samples)
		  	free(trak->samples);
		  // Raymond 2007/07/23
                  if(audio_cache == 0)
		  	trak->samples=salloc(sizeof(mov_sample_t)*entries);	
		  else
			trak->samples=malloc(sizeof(mov_sample_t)*entries);
#else		  
		  trak->samples=realloc(trak->samples,sizeof(mov_sample_t)*entries);
#endif
		  trak->samples_size=entries;
#if 1	// Raymond 2008/03/07
		for(i=0;i<entries;i++)
		{
			trak->samples[i].size=stream_read_dword(demuxer->stream);
			trak->total_sample_size += trak->samples[i].size;
		}
#else
		  for(i=0;i<entries;i++)
		    trak->samples[i].size=stream_read_dword(demuxer->stream);
#endif		  
		}
		// Raymond 2008/03/10
		else
		{
//			trak->total_sample_size = entries;
			trak->total_sample_size = entries * ss;	// Raymond 2008/03/20
		}
		break;
	    }
	    case MOV_FOURCC('s','t','c','o'): {
		int temp=stream_read_dword(demuxer->stream);
		int len=stream_read_dword(demuxer->stream);
		int i;
		
		// Raymond 2007/05/11
		if( get_preview == 1 && trak->type == MOV_TRAK_AUDIO )
		{
			stream_skip(demuxer->stream, len << 2);
			break;
		}	
		
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sChunk offset table! (%d chunks)\n",level,"",len);
		// extend array if needed:
		if(len>trak->chunks_size){
#if 1	// Raymond 2007/07/03
		    if(trak->chunks)
		  	sfree(trak->chunks);
		    // Raymond 2007/07/23
		    if(audio_cache == 0)
		    	trak->chunks=salloc(sizeof(mov_chunk_t)*len);
		    else
		    	trak->chunks=malloc(sizeof(mov_chunk_t)*len);
#else
		    trak->chunks=realloc(trak->chunks,sizeof(mov_chunk_t)*len);
#endif
		    trak->chunks_size=len;
		}
		// read elements:
		for(i=0;i<len;i++) trak->chunks[i].pos=stream_read_dword(demuxer->stream);
		break;
	    }
	    case MOV_FOURCC('c','o','6','4'): {
		int temp=stream_read_dword(demuxer->stream);
		int len=stream_read_dword(demuxer->stream);
		int i;
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*s64bit chunk offset table! (%d chunks)\n",level,"",len);
		// extend array if needed:
		if(len>trak->chunks_size){
#if 1	// Raymond 2007/07/03
		    if(trak->chunks)
		  	sfree(trak->chunks);
		    // Raymond 2007/07/23
		    if(audio_cache == 0)
		    	trak->chunks=salloc(sizeof(mov_chunk_t)*len);
		    else
			trak->chunks=malloc(sizeof(mov_chunk_t)*len);    
#else			
		    trak->chunks=realloc(trak->chunks,sizeof(mov_chunk_t)*len);
#endif
		    trak->chunks_size=len;
		}
		// read elements:
		for(i=0;i<len;i++)
		{
#ifndef	_LARGEFILE_SOURCE
		    if (stream_read_dword(demuxer->stream) != 0)
			mp_msg(MSGT_DEMUX, MSGL_WARN, "Chunk %d has got 64bit address, but you've MPlayer compiled without LARGEFILE support!\n", i);
		    trak->chunks[i].pos = stream_read_dword(demuxer->stream);
#else
		    trak->chunks[i].pos = stream_read_qword(demuxer->stream);
#endif
		}
		break;
	    }
	    case MOV_FOURCC('s','t','s','s'): {
		int temp=stream_read_dword(demuxer->stream);
		int entries=stream_read_dword(demuxer->stream);
		int ver = (temp << 24);
		int flags = (temp << 16)|(temp<<8)|temp;
		int i;
		mp_msg(MSGT_DEMUX, MSGL_V,"MOV: %*sSyncing samples (keyframes) table! (%d entries) (ver:%d,flags:%ld)\n",
		    level, "",entries, ver, flags);
		trak->keyframes_size=entries;
		trak->keyframes=salloc(sizeof(unsigned int)*entries);	// Raymond 2007/07/03
		for (i=0;i<entries;i++)
		    trak->keyframes[i]=stream_read_dword(demuxer->stream)-1;
//		for (i=0;i<entries;i++) printf("%3d: %d\n",i,trak->keyframes[i]);
		break;
	    }
	    case MOV_FOURCC('m','d','i','a'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sMedia stream!\n",level,"");
		lschunks(demuxer,level+1,pos+len,trak);
		break;
	    }
	    case MOV_FOURCC('m','i','n','f'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sMedia info!\n",level,"");
		lschunks(demuxer,level+1,pos+len,trak);
		break;
	    }
	    case MOV_FOURCC('s','t','b','l'): {
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: %*sSample info!\n",level,"");
		lschunks(demuxer,level+1,pos+len,trak);
		break;
	    }
	    case MOV_FOURCC('e','d','t','s'): {
		mp_msg(MSGT_DEMUX, MSGL_V, "MOV: %*sEdit atom!\n", level, "");
		lschunks(demuxer,level+1,pos+len,trak);
		break;
	    }
	    case MOV_FOURCC('e','l','s','t'): {
		int temp=stream_read_dword(demuxer->stream);
		int entries=stream_read_dword(demuxer->stream);
		int ver = (temp << 24);
		int flags = (temp << 16)|(temp<<8)|temp;
		int i;
	
		mp_msg(MSGT_DEMUX, MSGL_V,"MOV: %*sEdit list table (%d entries) (ver:%d,flags:%ld)\n",
		    level, "",entries, ver, flags);
#if 1
		trak->editlist_size=entries;
		trak->editlist=salloc(trak->editlist_size*sizeof(mov_editlist_t));
		for (i=0;i<entries;i++)
		{
		    int dur=stream_read_dword(demuxer->stream);
		    int mt=stream_read_dword(demuxer->stream);
		    int mr=stream_read_dword(demuxer->stream); // 16.16fp
		    trak->editlist[i].dur=dur;
		    trak->editlist[i].pos=mt;
		    trak->editlist[i].speed=mr;
		    mp_msg(MSGT_DEMUX, MSGL_V,"MOV: %*s  entry#%d: duration: %d  start time: %d  speed: %3.1fx\n",level,"",
			i,
			dur,mt,(float)mr/65536.0f);
		}
#endif
		break;
	    }
	    case MOV_FOURCC('c','o','d','e'):
	    {
	    /* XXX: Implement atom 'code' for FLASH support */
	    }
	    default:
		id = be2me_32(id);
		mp_msg(MSGT_DEMUX,MSGL_V,"MOV: unknown chunk: %.4s %d\n",&id,(int)len);
		break;
	  }//switch(id)
	} else { /* not in track */
	  switch(id) {
	    case MOV_FOURCC('m','v','h','d'): {
		stream_skip(demuxer->stream,12);
		priv->timescale=stream_read_dword(demuxer->stream);
		priv->duration=stream_read_dword(demuxer->stream);
		mp_msg(MSGT_DEMUX, MSGL_V,"MOV: %*sMovie header (%d bytes): tscale=%d  dur=%d\n",level,"",(int)len,
		    (int)priv->timescale,(int)priv->duration);
		break;
	    }
	    case MOV_FOURCC('t','r','a','k'): {
//	    if(trak) printf("MOV: Warning! trak in trak?\n");
	    if(priv->track_db>=MOV_MAX_TRACKS){
		mp_msg(MSGT_DEMUX,MSGL_WARN,MSGTR_MOVtooManyTrk);
		return;
	    }
	    if(!priv->track_db) mp_msg(MSGT_DEMUX, MSGL_INFO, "--------------\n");
	    trak=salloc(sizeof(mov_track_t));
	    memset(trak,0,sizeof(mov_track_t));
	    mp_msg(MSGT_DEMUX,MSGL_V,"MOV: Track #%d:\n",priv->track_db);
	    trak->id=priv->track_db;
	    priv->tracks[priv->track_db]=trak;
	    lschunks(demuxer,level+1,pos+len,trak);

		// Raymond 2007/05/11
//		if( get_preview == 1 && trak->type == MOV_TRAK_AUDIO )
//			break;			
	    // Raymond 2007/05/15
	    if( get_preview == 0 || ( get_preview == 1 && trak->type == MOV_TRAK_VIDEO) )
		if( trak->type == MOV_TRAK_VIDEO	|| trak->type == MOV_TRAK_AUDIO)	// Raymond 2008/03/06
	    		mov_build_index(trak,priv->timescale);
		
	    switch(trak->type){
	    case MOV_TRAK_AUDIO: {
		// Raymond 2007/05/15
		if( get_preview == 1 )
			break;
#if 0				   
		struct {
		   int16_t version;		// 0 or 1 (version 1 is qt3.0+)
		   int16_t revision;		// 0
		   int32_t vendor_id;		// 0
		   int16_t channels;		// 1 or 2  (Mono/Stereo)
		   int16_t samplesize;		// 8 or 16 (8Bit/16Bit)	
		   int16_t compression_id;	// if version 0 then 0
		  				// if version 1 and vbr then -2 else 0
		   int16_t packet_size;		// 0
		   uint16_t sample_rate;	// samplerate (Hz)
		   // qt3.0+ (version == 1)
		   uint32_t samples_per_packet;	// 0 or num uncompressed samples in a packet
		   				// if 0 below three values are also 0
		   uint32_t bytes_per_packet;	// 0 or num compressed bytes for one channel
		   uint32_t bytes_per_frame;	// 0 or num compressed bytes for all channels
		   				// (channels * bytes_per_packet)
		   uint32_t bytes_per_sample;	// 0 or size of uncompressed sample
		   // if samples_per_packet and bytes_per_packet are constant (CBR)
		   // then bytes_per_frame and bytes_per_sample must be 0 (else is VBR)
		   // ---
		   // optional additional atom-based fields
		   // ([int32_t size,int32_t type,some data ],repeat)
		} my_stdata;		  
#endif		
		sh_audio_t* sh=new_sh_audio(demuxer,priv->track_db);
		sh->format=trak->fourcc;

#if 1	// Raymond 2007/06/04	- for MP4 AAC	
		sh->samples_size = trak->samples_size;
		sh->samples = trak->samples;
#endif
		switch( sh->format ) {
		    case 0x726D6173: /* samr */
			/* amr narrowband */
			trak->samplebytes=sh->samplesize=1;
			trak->nchannels=sh->channels=1;
			sh->samplerate=8000;
			break;
			
		    case 0x62776173: /* sawb */
			/* amr wideband */
			trak->samplebytes=sh->samplesize=1;
			trak->nchannels=sh->channels=1;
			sh->samplerate=16000;
			break;

		    default:
			
// assumptions for below table: short is 16bit, int is 32bit, intfp is 16bit
// XXX: 32bit fixed point numbers (intfp) are only 2 Byte!		
// short values are usually one byte leftpadded by zero		
//   int values are usually two byte leftpadded by zero		
//  stdata[]:
//	8   short	version
//	10  short	revision
//	12  int		vendor_id
//	16  short	channels
//	18  short	samplesize
//	20  short	compression_id
//	22  short	packet_size (==0)
//	24  intfp	sample_rate
//     (26  short)	unknown (==0)
//    ---- qt3.0+ (version>=1)
//	28  int		samples_per_packet
//	32  int		bytes_per_packet
//	36  int		bytes_per_frame
//	40  int		bytes_per_sample
// there may be additional atoms following at 28 (version 0)
// or 44 (version 1), eg. esds atom of .MP4 files		
// esds atom:
//      28  int		atom size (bytes of int size, int type and data)
//      32  char[4]	atom type (fourc charater code -> esds)		
//      36  char[]  	atom data (len=size-8)

		trak->samplebytes=sh->samplesize=char2short(trak->stdata,18)/8;
		trak->nchannels=sh->channels=char2short(trak->stdata,16);
		/*printf("MOV: timescale: %d samplerate: %d durmap: %d (%d) -> %d (%d)\n",
		    trak->timescale, char2short(trak->stdata,24), trak->durmap[0].dur,
		    trak->durmap[0].num, trak->timescale/trak->durmap[0].dur,
		    char2short(trak->stdata,24)/trak->durmap[0].dur);*/
		sh->samplerate=char2short(trak->stdata,24);
		if((sh->samplerate < 7000) && trak->durmap) {
		  switch(char2short(trak->stdata,24)/trak->durmap[0].dur) {
		    // TODO: add more cases.
		    case 31:
		      sh->samplerate = 32000; break;
		    case 43:
		      sh->samplerate = 44100; break;
		    case 47:
		      sh->samplerate = 48000; break;
		    default:
		      mp_msg(MSGT_DEMUX, MSGL_WARN,
			  "MOV: unable to determine audio samplerate, "
			  "assuming 44.1kHz (got %d)\n",
			  char2short(trak->stdata,24)/trak->durmap[0].dur);
		      sh->samplerate = 44100;
		  }  
		}  
		}
		mp_msg(MSGT_DEMUX, MSGL_INFO, "Audio bits: %d  chans: %d  rate: %d\n",
		    sh->samplesize*8,sh->channels,sh->samplerate);

		if(trak->stdata_len >= 44 && trak->stdata[9]>=1){
		  mp_msg(MSGT_DEMUX,MSGL_V,"Audio header: samp/pack=%d bytes/pack=%d bytes/frame=%d bytes/samp=%d  \n",
		    char2int(trak->stdata,28),
		    char2int(trak->stdata,32),
		    char2int(trak->stdata,36),
		    char2int(trak->stdata,40));
		  if(trak->stdata_len>=44+8){
		    int len=char2int(trak->stdata,44);
		    int fcc=char2int(trak->stdata,48);
		    // we have extra audio headers!!!
		    printf("Audio extra header: len=%d  fcc=0x%X\n",len,fcc);
		    if((len >= 4) && 
		       (char2int(trak->stdata,52) >= 12) &&
		       (char2int(trak->stdata,52+4) == MOV_FOURCC('f','r','m','a')) &&
		       (char2int(trak->stdata,52+8) == MOV_FOURCC('a','l','a','c')) &&
		       (len >= 36 + char2int(trak->stdata,52))) {
			    sh->codecdata_len = char2int(trak->stdata,52+char2int(trak->stdata,52));
			    mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found alac atom (%d)!\n", sh->codecdata_len);
			    sh->codecdata = (unsigned char *)malloc(sh->codecdata_len);
			    memcpy(sh->codecdata, &trak->stdata[52+char2int(trak->stdata,52)], sh->codecdata_len);
		    } else {
		    sh->codecdata_len = len-8;
		    sh->codecdata = trak->stdata+44+8;
		    }
		  }
		}

		if((trak->stdata[9]==0 || trak->stdata[9]==1) && trak->stdata_len >= 36) { // version 0 with extra atoms
        int adjust = (trak->stdata[9]==1)?48:0;
		    int atom_len = char2int(trak->stdata,28+adjust);
		    switch(char2int(trak->stdata,32+adjust)) { // atom type
		      case MOV_FOURCC('e','s','d','s'): {
			mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found MPEG4 audio Elementary Stream Descriptor atom (%d)!\n", atom_len);
#ifndef BUILD_MPLAYER_LITE	// Raymond 2008/08/18			
			if(atom_len > 8) {
			  esds_t esds; 				  
			  if(!mp4_parse_esds(&trak->stdata[36+adjust], atom_len-8, &esds)) {
			    
			    sh->i_bps = esds.avgBitrate/8; 

//			    printf("######## audio format = %d ########\n",esds.objectTypeId);
			    if(esds.objectTypeId==MP4OTI_MPEG1Audio || esds.objectTypeId==MP4OTI_MPEG2AudioPart3)
				sh->format=0x55; // .mp3

			    // dump away the codec specific configuration for the AAC decoder
			    if(esds.decoderConfigLen){
			    if( (esds.decoderConfig[0]>>3) == 29 )
			    	sh->format = 0x1d61346d; // request multi-channel mp3 decoder
			    sh->codecdata_len = esds.decoderConfigLen;
			    sh->codecdata = (unsigned char *)malloc(sh->codecdata_len);
			    memcpy(sh->codecdata, esds.decoderConfig, sh->codecdata_len);
			    }
			  }
			  mp4_free_esds(&esds); // freeup esds mem
#if 0
	  		  { FILE* f=fopen("esds.dat","wb");
			  fwrite(&trak->stdata[36],atom_len-8,1,f);
			  fclose(f); }
#endif			  
			}
#endif // BUILD_MPLAYER_LITE	// Raymond 2008/08/18			
		      } break;
		      case MOV_FOURCC('a','l','a','c'): {
			mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found alac atom (%d)!\n", atom_len);
			if(atom_len > 8) {
			    // copy all the atom (not only payload) for lavc alac decoder
			    sh->codecdata_len = atom_len;
			    sh->codecdata = (unsigned char *)malloc(sh->codecdata_len);
			    memcpy(sh->codecdata, &trak->stdata[28], sh->codecdata_len);
			}
		      } break;
		      default:
			mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found unknown audio atom %c%c%c%c (%d)!\n",
			    trak->stdata[32+adjust],trak->stdata[33+adjust],trak->stdata[34+adjust],trak->stdata[35+adjust],
			    atom_len);
		    }
		}  
		mp_msg(MSGT_DEMUX, MSGL_INFO, "Fourcc: %.4s\n",&trak->fourcc);
#if 0
		{ FILE* f=fopen("stdata.dat","wb");
		  fwrite(trak->stdata,trak->stdata_len,1,f);
		  fclose(f); }
		{ FILE* f=fopen("tkdata.dat","wb");
		  fwrite(trak->tkdata,trak->tkdata_len,1,f);
		  fclose(f); }
#endif
		// Emulate WAVEFORMATEX struct:
		sh->wf=malloc(sizeof(WAVEFORMATEX));
		memset(sh->wf,0,sizeof(WAVEFORMATEX));
		sh->wf->nChannels=sh->channels;
		sh->wf->wBitsPerSample=(trak->stdata[18]<<8)+trak->stdata[19];
		// sh->wf->nSamplesPerSec=trak->timescale;
#if 1
		// Raymond - fix AMR sample rate bug
		switch( sh->format ) 
		{
		case 0x726D6173: /* samr */
		case 0x62776173: /* sawb */
			sh->wf->nSamplesPerSec = sh->samplerate;
			break;
		default:		
			sh->wf->nSamplesPerSec=(trak->stdata[24]<<8)+trak->stdata[25];
			break;
		}	
#else
		sh->wf->nSamplesPerSec=(trak->stdata[24]<<8)+trak->stdata[25];
#endif		
		if(trak->stdata_len >= 44 && trak->stdata[9]>=1 && char2int(trak->stdata,28)>0){
		//Audio header: samp/pack=4096 bytes/pack=743 bytes/frame=1486 bytes/samp=2
		  sh->wf->nAvgBytesPerSec=(sh->wf->nChannels*sh->wf->nSamplesPerSec*
		      char2int(trak->stdata,32)+char2int(trak->stdata,28)/2)
		      /char2int(trak->stdata,28);
		  sh->wf->nBlockAlign=char2int(trak->stdata,36);
		} else {
		  sh->wf->nAvgBytesPerSec=sh->wf->nChannels*sh->wf->wBitsPerSample*sh->wf->nSamplesPerSec/8;
		  // workaround for ms11 ima4
		  if (sh->format == 0x1100736d && trak->stdata_len >= 36)
		      sh->wf->nBlockAlign=char2int(trak->stdata,36);
		}
		// Selection:
//		if(demuxer->audio->id==-1 || demuxer->audio->id==priv->track_db){
//		    // (auto)selected audio track:
//		    demuxer->audio->id=priv->track_db;
//		    demuxer->audio->sh=sh; sh->ds=demuxer->audio;
//		}
		break;
	    }
	    case MOV_TRAK_VIDEO: {
		int i, entry;
		int flag, start, count_flag, end, palette_count, gray;
		int hdr_ptr = 76;  // the byte just after depth
		unsigned char *palette_map;
		sh_video_t* sh=new_sh_video(demuxer,priv->track_db);
		int depth = trak->stdata[75]|(trak->stdata[74]<<8);
		sh->format=trak->fourcc;

//  stdata[]:
//	8   short	version
//	10  short	revision
//	12  int		vendor_id
//	16  int		temporal_quality
//	20  int		spatial_quality
//	24  short	width
//	26  short	height
//	28  int		h_dpi
//	32  int		v_dpi
//	36  int		0
//	40  short	frames_per_sample
//	42  char[4]	compressor_name
//	74  short	depth
//	76  short	color_table_id
// additional atoms may follow,
// eg esds atom from .MP4 files
//      78  int		atom size
//      82  char[4]	atom type
//	86  ...		atom data

	{	ImageDescription* id=salloc(8+trak->stdata_len);
		trak->desc=id;
		id->idSize=8+trak->stdata_len;
//		id->cType=bswap_32(trak->fourcc);
		id->cType=le2me_32(trak->fourcc);
		id->version=char2short(trak->stdata,8);
		id->revisionLevel=char2short(trak->stdata,10);
		id->vendor=char2int(trak->stdata,12);
		id->temporalQuality=char2int(trak->stdata,16);
		id->spatialQuality=char2int(trak->stdata,20);
		id->width=char2short(trak->stdata,24);
		id->height=char2short(trak->stdata,26);
		id->hRes=char2int(trak->stdata,28);
		id->vRes=char2int(trak->stdata,32);
		id->dataSize=char2int(trak->stdata,36);
		id->frameCount=char2short(trak->stdata,40);
		memcpy(&id->name,trak->stdata+42,32);
		id->depth=char2short(trak->stdata,74);
		id->clutID=char2short(trak->stdata,76);
		if(trak->stdata_len>78)	memcpy(((char*)&id->clutID)+2,trak->stdata+78,trak->stdata_len-78);
		sh->ImageDesc=id;
#if 0
		{   FILE *f=fopen("ImageDescription","wb");
		    fwrite(id,id->idSize,1,f);
		    fclose(f);
		}
#endif
	}

		if(trak->stdata_len >= 86) { // extra atoms found
		  int pos=78;
		  int atom_len;
		  while(pos+8<=trak->stdata_len &&
		    (pos+(atom_len=char2int(trak->stdata,pos)))<=trak->stdata_len){
		   switch(char2int(trak->stdata,pos+4)) { // switch atom type
		    case MOV_FOURCC('g','a','m','a'):
		      // intfp with gamma value at which movie was captured
		      // can be used to gamma correct movie display
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found unsupported Gamma-Correction movie atom (%d)!\n",
			  atom_len);
		      break;
		    case MOV_FOURCC('f','i','e','l'):
		      // 2 char-values (8bit int) that specify field handling
		      // see the Apple's QuickTime Fileformat PDF for more info
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found unsupported Field-Handling movie atom (%d)!\n",
			  atom_len);
		      break;
		    case MOV_FOURCC('m','j','q','t'):
		      // Motion-JPEG default quantization table
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found unsupported MJPEG-Quantization movie atom (%d)!\n",
			  atom_len);
		      break;
		    case MOV_FOURCC('m','j','h','t'):
		      // Motion-JPEG default huffman table
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found unsupported MJPEG-Huffman movie atom (%d)!\n",
			  atom_len);
		      break;
		    case MOV_FOURCC('e','s','d','s'):
		      // MPEG4 Elementary Stream Descriptor header
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found MPEG4 movie Elementary Stream Descriptor atom (%d)!\n", atom_len);
		      // add code here to save esds header of length atom_len-8
		      // beginning at stdata[86] to some variable to pass it
		      // on to the decoder ::atmos
#ifndef BUILD_MPLAYER_LITE	// Raymond 2008/08/18		      
		      if(atom_len > 8) {
      			esds_t esds; 				  
			if(!mp4_parse_esds(trak->stdata+pos+8, atom_len-8, &esds)) {
    
			  if(esds.objectTypeId==MP4OTI_MPEG2VisualSimple || esds.objectTypeId==MP4OTI_MPEG2VisualMain ||
			     esds.objectTypeId==MP4OTI_MPEG2VisualSNR || esds.objectTypeId==MP4OTI_MPEG2VisualSpatial ||
			     esds.objectTypeId==MP4OTI_MPEG2VisualHigh || esds.objectTypeId==MP4OTI_MPEG2Visual422)
			    sh->format=mmioFOURCC('m', 'p', 'g', '2');
			  else if(esds.objectTypeId==MP4OTI_MPEG1Visual)
			    sh->format=mmioFOURCC('m', 'p', 'g', '1');

			  // dump away the codec specific configuration for the AAC decoder
			  trak->stream_header_len = esds.decoderConfigLen;
			  trak->stream_header = (unsigned char *)salloc(trak->stream_header_len);
			  memcpy(trak->stream_header, esds.decoderConfig, trak->stream_header_len);

			  // Raymond - get m4v width and height
//			  printf("Fourcc: %.4s\n",&trak->fourcc);			  
			  if ( trak->fourcc == mmioFOURCC('m','p','4','v') )
			  {
				  int width = 0, height = 0, len;
				  int ret = 1;
				  ret = GetWidthHeight(trak->stream_header, &width, &height, trak->stream_header_len);
				  if ( ret < 0 )
				  	printf("GetWidthHeight() return %d\n", ret);
				  else
				  {
// 				    	printf("M4V size = %3d x %3d\n", width, height);
//				    	printf("Track size = %3d x %3d\n", trak->width, trak->height);
					trak->width  = width;
					trak->height = height;
// 				    	printf("Track size new = %3d x %3d\n", trak->width, trak->height);
				  }
			  }
			}
			mp4_free_esds(&esds); // freeup esds mem
		      }	      
#endif // BUILD_MPLAYER_LITE	// Raymond 2008/08/18			  
		      break;
		    case MOV_FOURCC('a','v','c','C'):
		      // AVC decoder configuration record
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: AVC decoder configuration record atom (%d)!\n", atom_len);
		      if(atom_len > 8) {
		        int i, poffs, cnt;
		        // Parse some parts of avcC, just for fun :)
		        // real parsing is done by avc1 decoder
		        mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC version: %d\n", *(trak->stdata+pos+8));
		        if (*(trak->stdata+pos+8) != 1)
		          mp_msg(MSGT_DEMUX, MSGL_ERR, "MOV: unknown avcC version (%d). Expexct problems.\n", *(trak->stdata+pos+9));
		        mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC profile: %d\n", *(trak->stdata+pos+9));
		        mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC profile compatibility: %d\n", *(trak->stdata+pos+10));
		        mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC level: %d\n", *(trak->stdata+pos+11));
		        mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC nal length size: %d\n", ((*(trak->stdata+pos+12))&0x03)+1);
		        mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC number of sequence param sets: %d\n", cnt = (*(trak->stdata+pos+13) & 0x1f));
		        poffs = pos + 14;
		        for (i = 0; i < cnt; i++) {
		          mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC sps %d have length %d\n", i, BE_16(trak->stdata+poffs));
		          poffs += BE_16(trak->stdata+poffs) + 2;
		        }
		        mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC number of picture param sets: %d\n", *(trak->stdata+poffs));
		        poffs++;
		        for (i = 0; i < cnt; i++) {
		          mp_msg(MSGT_DEMUX, MSGL_V, "MOV: avcC pps %d have length %d\n", i, BE_16(trak->stdata+poffs));
		          poffs += BE_16(trak->stdata+poffs) + 2;
		        }

			// Raymond
			{
				int profile = *(trak->stdata+pos+9);
				if( profile != 66 )	
					fprintf(stderr, "\navcC profile : %d, not baseline profile, will not be supported!\n\n", profile);
			}	
		        // Copy avcC for the AVC decoder
		        // This data will be put in extradata below, where BITMAPINFOHEADER is created
		        trak->stream_header_len = atom_len-8;
		        trak->stream_header = (unsigned char *)salloc(trak->stream_header_len);
		        memcpy(trak->stream_header, trak->stdata+pos+8, trak->stream_header_len);
		      }	      
		      break;
		// Raymond 2007/05/14 - avc1 bit rate	  
		case MOV_FOURCC('b','t','r','t'):		         
		      if(atom_len == 20) {
		        int poffs = pos + 16;
			 int bit_rate = trak->stdata[poffs] << 24 | trak->stdata[poffs+1] << 16 | 
			 			  trak->stdata[poffs+2] << 8 | trak->stdata[poffs+3] ;
		        mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: btrt atom (len = %d) = %d \n", atom_len, bit_rate);
			 sh->i_bps = 	bit_rate / 8;
		      }	      
		      break;
		//	  
		    case 0:
		      break;
		    default:
	      	      mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Found unknown movie atom %c%c%c%c (%d)!\n",
	      		  trak->stdata[pos+4],trak->stdata[pos+5],trak->stdata[pos+6],trak->stdata[pos+7],
	      		  atom_len);
		   }
		   if(atom_len<8) break;
		   pos+=atom_len;
//		   printf("pos=%d max=%d\n",pos,trak->stdata_len);
		  }
		}
#if 1	// Raymond 2007/09/13
//		printf("size = %d, timescale = %d, duration = %d\n", trak->samples_size , trak->timescale , trak->length);
		if(trak->length != 0)
//			sh->fps= (float)(trak->samples_size * trak->timescale) / (float)trak->length;	// may be overflow
			sh->fps= (float)(trak->samples_size) * ( (float)(trak->timescale) / (float)(trak->length) );	// Raymond 2007/11/02
		else
#endif		
		sh->fps=trak->timescale/
		    ((trak->durmap_size>=1)?(float)trak->durmap[0].dur:1);
		sh->frametime=1.0f/sh->fps;
		mov_keyframe_size = trak->keyframes_size;	//Barry 2007-11-22

		if ( trak->fourcc == mmioFOURCC('m','p','4','v') )	// Raymond
		{	
		  sh->disp_w = trak->width;
  		  sh->disp_h = trak->height;
		}
		else
		{
		  sh->disp_w=trak->stdata[25]|(trak->stdata[24]<<8);
  		  sh->disp_h=trak->stdata[27]|(trak->stdata[26]<<8);
		}

		// Raymond 2008/08/27
		if ( trak->fourcc == mmioFOURCC('h','2','6','3') )	
		{	
		  	sh->disp_w = trak->stdata[25]|(trak->stdata[24]<<8);
  		  	sh->disp_h = trak->stdata[27]|(trak->stdata[26]<<8);
		  
		  	if( sh->disp_w > 176 )
		  	{
		  		sh->disp_w = 352;
  		  		sh->disp_h = 288;
		  	}
		}
		
		// if image size is zero, fallback to display size
		if(!sh->disp_w && !sh->disp_h) {
		  sh->disp_w=trak->tkdata[77]|(trak->tkdata[76]<<8);
		  sh->disp_h=trak->tkdata[81]|(trak->tkdata[80]<<8);
		}
		else if(sh->disp_w!=(trak->tkdata[77]|(trak->tkdata[76]<<8)))
		{
		  // codec and display width differ... use display one for aspect
		  sh->aspect=trak->tkdata[77]|(trak->tkdata[76]<<8);
		  sh->aspect/=trak->tkdata[81]|(trak->tkdata[80]<<8);
		}
		
		if(depth>32+8) printf("*** depth = 0x%X\n",depth);

		// palettized?
		gray = 0;
		if (depth > 32) { depth&=31; gray = 1; } // depth > 32 means grayscale
		if ((depth == 2) || (depth == 4) || (depth == 8))
		  palette_count = (1 << depth);
		else
		  palette_count = 0;

		// emulate BITMAPINFOHEADER:
		if (palette_count)
		{
		  //sh->bih=malloc(sizeof(BITMAPINFOHEADER) + palette_count * 4);
		  sh->bih=salloc(sizeof(BITMAPINFOHEADER) + palette_count * 4);	//Barry 2008-06-27
		  memset(sh->bih,0,sizeof(BITMAPINFOHEADER) + palette_count * 4);
		  sh->bih->biSize=40 + palette_count * 4;
//		   fprintf(stderr,"Clearing BIH\n");

		  // fetch the relevant fields
		  flag = BE_16(&trak->stdata[hdr_ptr]);
		  hdr_ptr += 2;
		  start = BE_32(&trak->stdata[hdr_ptr]);
		  hdr_ptr += 4;
		  count_flag = BE_16(&trak->stdata[hdr_ptr]);
		  hdr_ptr += 2;
		  end = BE_16(&trak->stdata[hdr_ptr]);
		  hdr_ptr += 2;
		  palette_map = (unsigned char *)sh->bih + 40;
		  mp_msg(MSGT_DEMUX, MSGL_INFO, "Allocated %d entries for palette\n",
		    palette_count);
		  mp_msg(MSGT_DEMUX, MSGL_DBG2, "QT palette: start: %x, end: %x, count flag: %d, flags: %x\n",
		    start, end, count_flag, flag);

		  /* XXX: problems with sample (statunit6.mov) with flag&0x4 set! - alex*/

		  // load default palette
		  if (flag & 0x08)
		  {
		    if (gray)
		    {
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "Using default QT grayscale palette\n");
		      if (palette_count == 16)
		        memcpy(palette_map, qt_default_grayscale_palette_16, 16 * 4);
		      else if (palette_count == 256) {
		        memcpy(palette_map, qt_default_grayscale_palette_256, 256 * 4);
		        if (trak->fourcc == mmioFOURCC('c','v','i','d')) {
		          int i;
		          // Hack for grayscale CVID, negative palette
		          // If you have samples where this is not required contact me (rxt)
		          mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: greyscale cvid with default palette,"
		            " enabling negative palette hack.\n");
		          for (i = 0; i < 256 * 4; i++)
		            palette_map[i] = palette_map[i] ^ 0xff;
		        }
		      }
		    }
		    else
		    {
		      mp_msg(MSGT_DEMUX, MSGL_INFO, "Using default QT colour palette\n");
		      if (palette_count == 4)
		        memcpy(palette_map, qt_default_palette_4, 4 * 4);
		      else if (palette_count == 16)
		        memcpy(palette_map, qt_default_palette_16, 16 * 4);
		      else if (palette_count == 256)
		        memcpy(palette_map, qt_default_palette_256, 256 * 4);
		    }
		  }
		  // load palette from file
		  else
		  {
		    mp_msg(MSGT_DEMUX, MSGL_INFO, "Loading palette from file\n");
		    for (i = start; i <= end; i++)
		    {
		      entry = BE_16(&trak->stdata[hdr_ptr]);
		      hdr_ptr += 2;
		      // apparently, if count_flag is set, entry is same as i
		      if (count_flag & 0x8000)
		        entry = i;
		      // only care about top 8 bits of 16-bit R, G, or B value
		      if (entry <= palette_count && entry >= 0)
		      {
		        palette_map[entry * 4 + 2] = trak->stdata[hdr_ptr + 0];
		        palette_map[entry * 4 + 1] = trak->stdata[hdr_ptr + 2];
		        palette_map[entry * 4 + 0] = trak->stdata[hdr_ptr + 4];
		        mp_dbg(MSGT_DEMUX, MSGL_DBG2, "QT palette: added entry: %d of %d (colors: R:%x G:%x B:%x)\n",
			    entry, palette_count,
			    palette_map[entry * 4 + 2], 
			    palette_map[entry * 4 + 1],
			    palette_map[entry * 4 + 0]);
		      }
		      else
		        mp_msg(MSGT_DEMUX, MSGL_V, "QT palette: skipped entry (out of count): %d of %d\n",
			    entry, palette_count);
		      hdr_ptr += 6;
		    }
		  }
		}
		else
		{
		 if (trak->fourcc == mmioFOURCC('a','v','c','1')) {
#ifdef __ARC_MPLAYER__
		  sh->bih=salloc(sizeof(BITMAPINFOHEADER) + trak->stream_header_len);
#else
		  sh->bih=malloc(sizeof(BITMAPINFOHEADER) + trak->stream_header_len);
#endif
		  memset(sh->bih,0,sizeof(BITMAPINFOHEADER) + trak->stream_header_len);
		  sh->bih->biSize=40  + trak->stream_header_len;
		  memcpy(((unsigned char *)sh->bih)+40,  trak->stream_header, trak->stream_header_len);
		  sfree (trak->stream_header);
		  trak->stream_header_len = 0;
		  trak->stream_header = NULL;
		 } else {
		  //sh->bih=malloc(sizeof(BITMAPINFOHEADER));
		  sh->bih=salloc(sizeof(BITMAPINFOHEADER));		//Barry 2008-06-27
		  memset(sh->bih,0,sizeof(BITMAPINFOHEADER));
		  sh->bih->biSize=40;
		 }
		}
		sh->bih->biWidth=sh->disp_w;
		sh->bih->biHeight=sh->disp_h;
		sh->bih->biPlanes=0;
		sh->bih->biBitCount=depth;
		sh->bih->biCompression=trak->fourcc;
		sh->bih->biSizeImage=sh->bih->biWidth*sh->bih->biHeight;

		mp_msg(MSGT_DEMUX, MSGL_INFO, "Image size: %d x %d (%d bpp)\n",sh->disp_w,sh->disp_h,sh->bih->biBitCount);
		if(trak->tkdata_len>81)
		mp_msg(MSGT_DEMUX, MSGL_INFO, "Display size: %d x %d\n",
		    trak->tkdata[77]|(trak->tkdata[76]<<8),
		    trak->tkdata[81]|(trak->tkdata[80]<<8));
		mp_msg(MSGT_DEMUX, MSGL_INFO, "Fourcc: %.4s  Codec: '%.*s'\n",&trak->fourcc,trak->stdata[42]&31,trak->stdata+43);

#ifdef BUILD_SK2851	// Raymond 2008/08/20
		// Raymond 2008/08/27
		if ( trak->fourcc == mmioFOURCC('h','2','6','3') )	
		{
			disp_x = trak->tkdata[77]|(trak->tkdata[76]<<8); 
			disp_y = trak->tkdata[81]|(trak->tkdata[80]<<8);
			//if( disp_x < sh->disp_w || disp_y < sh->disp_h )
			if( ((disp_x > 0) && (disp_x < sh->disp_w)) || ((disp_y > 0) && (disp_y < sh->disp_h)) )	//Barry 2008-08-26
			{
				set_real_display = 1;
			}
		}
#endif

//		if(demuxer->video->id==-1 || demuxer->video->id==priv->track_db){
//		    // (auto)selected video track:
//		    demuxer->video->id=priv->track_db;
//		    demuxer->video->sh=sh; sh->ds=demuxer->video;
//		}
		break;
	    }
	    case MOV_TRAK_GENERIC:
		mp_msg(MSGT_DEMUX, MSGL_INFO, "Generic track - not completely understood! (id: %d)\n", trak->id);
		/* XXX: Also this contains the FLASH data */

#if 0
	    {
		int pos = stream_tell(demuxer->stream);
		int i;
		int fd;
		char name[20];
	
		for (i=0; i<trak->samples_size; i++)
		{
		    char buf[trak->samples[i].size];
		    stream_seek(demuxer->stream, trak->samples[i].pos);
		    snprintf((char *)&name[0], 20, "samp%d", i);
		    fd = open((char *)&name[0], O_CREAT|O_WRONLY);
		    stream_read(demuxer->stream, &buf[0], trak->samples[i].size);
		    write(fd, &buf[0], trak->samples[i].size);
		    close(fd);
		 }
		for (i=0; i<trak->chunks_size; i++)
		{
		    char buf[trak->length];
		    stream_seek(demuxer->stream, trak->chunks[i].pos);
		    snprintf((char *)&name[0], 20, "chunk%d", i);
		    fd = open((char *)&name[0], O_CREAT|O_WRONLY);
		    stream_read(demuxer->stream, &buf[0], trak->length);
		    write(fd, &buf[0], trak->length);
		    close(fd);
		 }
		 if (trak->samplesize > 0)
		 {
		    char *buf;
		    
		    buf = malloc(trak->samplesize);
		    stream_seek(demuxer->stream, trak->chunks[0].pos);
		    snprintf((char *)&name[0], 20, "trak%d", trak->id);
		    fd = open((char *)&name[0], O_CREAT|O_WRONLY);
		    stream_read(demuxer->stream, buf, trak->samplesize);
		    write(fd, buf, trak->samplesize);
		    close(fd);
		 }
		 stream_seek(demuxer->stream, pos);
	    }		
#endif
		break;
	    default:
		mp_msg(MSGT_DEMUX, MSGL_INFO, "Unknown track type found (type: %d)\n", trak->type);
#if 1	// Raymond 2008/03/06	
	if(trak->keyframes_size > 1)	sfree(trak->keyframes);

	if(audio_cache == 0)
	{
		if(trak->chunks_size > 1)	sfree(trak->chunks);
		if(trak->samples != NULL)	sfree(trak->samples);
	}
	else
	{
		if(trak->chunks_size > 1)	free(trak->chunks);
		if(trak->samples != NULL)	free(trak->samples);
	}
#endif		
		break;
	    }
	    mp_msg(MSGT_DEMUX, MSGL_INFO, "--------------\n");
	    priv->track_db++;
	    trak=NULL;
	    break;
	}
#ifndef HAVE_ZLIB
	case MOV_FOURCC('c','m','o','v'): {
	    mp_msg(MSGT_DEMUX,MSGL_ERR,MSGTR_MOVcomprhdr);
	    return;
	}
#else
	case MOV_FOURCC('m','o','o','v'):
	case MOV_FOURCC('c','m','o','v'): {
//	    mp_msg(MSGT_DEMUX,MSGL_ERR,MSGTR_MOVcomprhdr);
	    lschunks(demuxer,level+1,pos+len,NULL);
	    break;
	}
	case MOV_FOURCC('d','c','o','m'): {
//	    int temp=stream_read_dword(demuxer->stream);
	    unsigned int algo=stream_read_dword(demuxer->stream);
	    algo=be2me_32(algo);
	    mp_msg(MSGT_DEMUX, MSGL_INFO, "Compressed header uses %.4s algo!\n",&algo);
	    break;
	}
	case MOV_FOURCC('c','m','v','d'): {
//	    int temp=stream_read_dword(demuxer->stream);
	    unsigned int moov_sz=stream_read_dword(demuxer->stream);
	    unsigned int cmov_sz=len-4;
	    unsigned char* cmov_buf=malloc(cmov_sz);
	    unsigned char* moov_buf=malloc(moov_sz+16);
	    int zret;
	    z_stream zstrm;
	    stream_t* backup;

	    mp_msg(MSGT_DEMUX, MSGL_INFO, "Compressed header size: %d / %d\n",cmov_sz,moov_sz);

	    stream_read(demuxer->stream,cmov_buf,cmov_sz);

	      zstrm.zalloc          = (alloc_func)0;
	      zstrm.zfree           = (free_func)0;
	      zstrm.opaque          = (voidpf)0;
	      zstrm.next_in         = cmov_buf;
	      zstrm.avail_in        = cmov_sz;
	      zstrm.next_out        = moov_buf;
	      zstrm.avail_out       = moov_sz;
	    
	      zret = inflateInit(&zstrm);
	      if (zret != Z_OK)
		{ mp_msg(MSGT_DEMUX, MSGL_ERR, "QT cmov: inflateInit err %d\n",zret);
		return;
		}
	      zret = inflate(&zstrm, Z_NO_FLUSH);
	      if ((zret != Z_OK) && (zret != Z_STREAM_END))
		{ mp_msg(MSGT_DEMUX, MSGL_ERR, "QT cmov inflate: ERR %d\n",zret);
		return;
		}
#if 0
	      else {
		FILE *DecOut;
		DecOut = fopen("Out.bin", "w");
		fwrite(moov_buf, 1, moov_sz, DecOut);
		fclose(DecOut);
	      }
#endif
	      if(moov_sz != zstrm.total_out)
	        mp_msg(MSGT_DEMUX, MSGL_WARN, "Warning! moov size differs cmov: %d  zlib: %d\n",moov_sz,zstrm.total_out);
	      zret = inflateEnd(&zstrm);
	      
	      backup=demuxer->stream;
	       demuxer->stream=new_memory_stream(moov_buf,moov_sz);
	       stream_skip(demuxer->stream,8);
	       lschunks(demuxer,level+1,moov_sz,NULL); // parse uncompr. 'moov'
	       //free_stream(demuxer->stream);
	      demuxer->stream=backup;
	      free(cmov_buf);
	      free(moov_buf);	    
	      break;
	}
#endif
	case MOV_FOURCC('u','d','t','a'):
	{
	    unsigned int udta_id;
	    off_t udta_len;
	    off_t udta_size = len;
	
	    mp_msg(MSGT_DEMUX, MSGL_DBG2, "mov: user data record found\n");
	    mp_msg(MSGT_DEMUX, MSGL_V, "Quicktime Clip Info:\n");

	    while((len > 8) && (udta_size > 8))
	    {
		udta_len = stream_read_dword(demuxer->stream);
		udta_id = stream_read_dword(demuxer->stream);
		udta_size -= 8;
		mp_msg(MSGT_DEMUX, MSGL_DBG2, "udta_id: %.4s (len: %d)\n", &udta_id, udta_len);
		switch (udta_id)
		{
		    case MOV_FOURCC(0xa9,'c','p','y'):
		    case MOV_FOURCC(0xa9,'d','a','y'):
		    case MOV_FOURCC(0xa9,'d','i','r'):
		    /* 0xa9,'e','d','1' - '9' : edit timestamps */
		    case MOV_FOURCC(0xa9,'f','m','t'):
		    case MOV_FOURCC(0xa9,'i','n','f'):
		    case MOV_FOURCC(0xa9,'p','r','d'):
		    case MOV_FOURCC(0xa9,'p','r','f'):
		    case MOV_FOURCC(0xa9,'r','e','q'):
		    case MOV_FOURCC(0xa9,'s','r','c'):
		    case MOV_FOURCC('n','a','m','e'):
		    case MOV_FOURCC(0xa9,'n','a','m'):
		    case MOV_FOURCC(0xa9,'A','R','T'):
		    case MOV_FOURCC(0xa9,'c','m','t'):
		    case MOV_FOURCC(0xa9,'a','u','t'):
		    case MOV_FOURCC(0xa9,'s','w','r'):
		    {
			off_t text_len = stream_read_word(demuxer->stream);
			char text[text_len+2+1];
			stream_read(demuxer->stream, (char *)&text, text_len+2);
			text[text_len+2] = 0x0;
			switch(udta_id)
			{
			    case MOV_FOURCC(0xa9,'a','u','t'):
				demux_info_add(demuxer, "author", &text[2]);
				mp_msg(MSGT_DEMUX, MSGL_V, " Author: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'c','p','y'):
				demux_info_add(demuxer, "copyright", &text[2]);
				mp_msg(MSGT_DEMUX, MSGL_V, " Copyright: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'i','n','f'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Info: %s\n", &text[2]);
				break;
			    case MOV_FOURCC('n','a','m','e'):
			    case MOV_FOURCC(0xa9,'n','a','m'):
				demux_info_add(demuxer, "name", &text[2]);
				mp_msg(MSGT_DEMUX, MSGL_V, " Name: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'A','R','T'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Artist: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'d','i','r'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Director: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'c','m','t'):
				demux_info_add(demuxer, "comments", &text[2]);
				mp_msg(MSGT_DEMUX, MSGL_V, " Comment: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'r','e','q'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Requirements: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'s','w','r'):
				demux_info_add(demuxer, "encoder", &text[2]);
				mp_msg(MSGT_DEMUX, MSGL_V, " Software: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'d','a','y'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Creation timestamp: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'f','m','t'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Format: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'p','r','d'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Producer: %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'p','r','f'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Performer(s): %s\n", &text[2]);
				break;
			    case MOV_FOURCC(0xa9,'s','r','c'):
				mp_msg(MSGT_DEMUX, MSGL_V, " Source providers: %s\n", &text[2]);
				break;
			}
			udta_size -= 4+text_len;
			break;
		    }
		    /* some other shits:    WLOC - window location,
					    LOOP - looping style,
					    SelO - play only selected frames
					    AllF - play all frames
		    */
		    case MOV_FOURCC('W','L','O','C'):
		    case MOV_FOURCC('L','O','O','P'):
		    case MOV_FOURCC('S','e','l','O'):
		    case MOV_FOURCC('A','l','l','F'):
		    default:
		    {
			if( udta_len>udta_size)
				udta_len=udta_size;
			{
			char dump[udta_len-4];
			stream_read(demuxer->stream, (char *)&dump, udta_len-4-4);
			udta_size -= udta_len;
			}
		    }
		}
	    }
	    break;
	} /* eof udta */
	default:
	  id = be2me_32(id);
	  mp_msg(MSGT_DEMUX,MSGL_V,"MOV: unknown chunk: %.4s %d\n",&id,(int)len);
	} /* endof switch */
	} /* endof else */

	pos+=len+8;
	if(pos>=endpos) break;
	if(!stream_seek(demuxer->stream,pos)) break;
    }
}

int mov_read_header(demuxer_t* demuxer){
    mov_priv_t* priv=demuxer->priv;
    int t_no;
    int best_a_id=-1, best_a_len=0;
    int best_v_id=-1, best_v_len=0;
    // Raymond 2008/03/10
    unsigned int atrak_start = 0, atrack_end = 0, atrak_size = 0;
    unsigned int vtrak_start = 0, vtrack_end = 0, vtrak_size = 0;
    
    mp_msg(MSGT_DEMUX, MSGL_DBG3, "mov_read_header!\n");

    // Parse header:    
    stream_reset(demuxer->stream);
    if(!stream_seek(demuxer->stream,priv->moov_start))
    {
	mp_msg(MSGT_DEMUX,MSGL_ERR,"MOV: Cannot seek to the beginning of the Movie header (0x%x)\n",
	    priv->moov_start);
	return 0;
    }
    lschunks(demuxer, 0, priv->moov_end, NULL);
    // just in case we have hit eof while parsing...
    demuxer->stream->eof = 0;
//    mp_msg(MSGT_DEMUX, MSGL_INFO, "--------------\n");

    // find the best (longest) streams:
    for(t_no=0;t_no<priv->track_db;t_no++){
        mov_track_t* trak=priv->tracks[t_no];
	int len=(trak->samplesize) ? trak->chunks_size : trak->samples_size;
	if(demuxer->a_streams[t_no]) // need audio
	{
	    	if(len>best_a_len){	best_a_len=len; best_a_id=t_no; }

		// Raymond 2008/06/03		
           	if( trak->samplesize == 0 )		// trak->samplesize = 0, trak->samples_size > 0 -> trak->samples
           	{
              	// Raymond 2008/03/10
              	atrak_start = trak->samples[0].pos;
			atrack_end = trak->samples[len-1].pos;
			atrak_size = trak->total_sample_size;

			// Raymond 2008/06/02
			if(audio_cache == 0)
				sfree(trak->chunks);
			else
				free(trak->chunks);
           	}
	    	else		// trak->samplesize = 1, trak->samples_size == 0 ->  trak->chunks
	    	{
	       	atrak_start = trak->chunks[0].pos;
			atrack_end = trak->chunks[len-1].pos;
			atrak_size = trak->total_sample_size;

#if 1	// Raymond 2008/06/03
			int mm = 0;
			if( trak->samplesize == 1 && audio_cache == 0 && get_preview == 0 && atrak_size < (4<<20) )
			{
				for( mm = 0 ; mm < len ; mm++ )
				{
					trak->chunks[mm].buf = (unsigned char *)salloc(trak->chunks[mm].size);
					stream_seek(demuxer->stream, trak->chunks[mm].pos);
					stream_read(demuxer->stream, trak->chunks[mm].buf, trak->chunks[mm].size);

					mp_msg(MSGT_DEMUX,MSGL_V,"A trak: [%d] read %d bytes at %d\n", 
						mm, trak->chunks[mm].size, trak->chunks[mm].pos);
				}

				mp_msg(MSGT_DEMUX,MSGL_V,"\nA trak: read %d samples first\n\n", len);
			}
			else
			{
				for( mm = 0 ; mm < len ; mm++ )
					trak->chunks[mm].buf = NULL;
			}
#endif
	    	}	

		mp_msg(MSGT_DEMUX,MSGL_V,"A trak->samplesize = %d\n", trak->samplesize);
		mp_msg(MSGT_DEMUX,MSGL_V,"A trak->samples_size = %d\n", trak->samples_size);
	    	mp_msg(MSGT_DEMUX,MSGL_V,"A trak: start = %d\n", atrak_start);
           	mp_msg(MSGT_DEMUX,MSGL_V,"A trak: end = %d\n", atrack_end);
	    	mp_msg(MSGT_DEMUX,MSGL_V,"A trak: total size = %d\n\n", atrak_size);	
		
	}
	if(demuxer->v_streams[t_no]) // need video
	{
	    	if(len>best_v_len){	best_v_len=len; best_v_id=t_no; }

		// Raymond 2008/03/10
              vtrak_start = trak->samples[0].pos;
		vtrack_end = trak->samples[len-1].pos;
		vtrak_size = trak->total_sample_size;
		
	    	// Raymond 2008/03/07		
	    	mp_msg(MSGT_DEMUX,MSGL_V,"V trak: start = %d\n", trak->samples[0].pos);
           	mp_msg(MSGT_DEMUX,MSGL_V,"V trak: end = %d\n", trak->samples[len-1].pos);
           	mp_msg(MSGT_DEMUX,MSGL_V,"V trak: total size = %d\n\n", trak->total_sample_size);

		// Raymond 2008/03/10
		if(trak->samplesize == 0)
		{
			if(audio_cache == 0)
				sfree(trak->chunks);
			else
				free(trak->chunks);
		}   

		// Raymond 2008/03/21
		if ( trak->fourcc == mmioFOURCC('j','p','e','g') || trak->fourcc == mmioFOURCC('A','V','D','J')  )	
		{
			is_mjpeg = 1;
		}
		
	}
#if 1	// Raymond 2008/03/06
	if(trak->tkdata_len > 1)		sfree(trak->tkdata);
//	if(trak->stdata_len > 1)		sfree(trak->stdata);	// Raymond 2008/03/20
	if(trak->chunkmap_size > 1)	sfree(trak->chunkmap);
	if(trak->durmap_size > 1)		sfree(trak->durmap);		
#endif
    }
    // Raymond 2008/03/10	
    mp_msg(MSGT_DEMUX,MSGL_V,"File size = %d\n", demuxer->stream->end_pos);
    mp_msg(MSGT_DEMUX,MSGL_V,"A trak: start -> end = %d\n\n", demuxer->stream->end_pos -atrak_start );
	
    mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: longest streams: A: #%d (%d samples)  V: #%d (%d samples)\n",
	best_a_id,best_a_len,best_v_id,best_v_len);
    if(demuxer->audio->id==-1 && best_a_id>=0) demuxer->audio->id=best_a_id;
    if(demuxer->video->id==-1 && best_v_id>=0) demuxer->video->id=best_v_id;

#ifdef MOV_READ_PHY	// Raymond 2008/03/10

#ifndef BUILD_SK2851		// Raymond 2008/08/06
	if( get_preview == 0 && audio_cache == 0 && is_mjpeg == 0 && demuxer->stream->type != STREAMTYPE_STREAM )		// Raymond 2008/03/21
	{
#else
	if( get_preview == 0 && audio_cache == 0 && demuxer->stream->type != STREAMTYPE_STREAM )
	{
		MOV_VIDEO_BUF_SIZE >>= 1;
//		MOV_AUDIO_BUF_SIZE = 44100;
		MOV_AUDIO_BUF_SIZE = 65536;	// Raymond 2008/08/13
#endif
		mov_readphy = 1;	// Raymond 2008/03/19
	
		if( best_a_id >= 0 )
		{
			if( atrak_size < (5<<20) && ( atrak_start > vtrack_end ||vtrak_start > atrack_end ) )
			{
//				priv->mov_abuf = salloc(atrak_size);
				// Raymond 2008/08/21
				priv->mov_abuf = salloc(atrak_size+512);
				priv->mov_abuf += (512 -((int)(priv->mov_abuf) & 511));

				stream_seek(demuxer->stream, atrak_start);				
				stream_read(demuxer->stream, priv->mov_abuf, atrak_size);
				priv->mov_abuf_pos = atrak_start;
				priv->mov_abuf_len = atrak_size;
				
				mp_msg(MSGT_DEMUX,MSGL_V,"A[0]: Read all audio data in buffer - %d bytes at %d\n", atrak_size, atrak_start);
			}
			else
			{
//				priv->mov_abuf = salloc(MOV_VIDEO_BUF_SIZE);
				// Raymond 2008/06/02				
				priv->mov_abuf = salloc(MOV_VIDEO_BUF_SIZE+512);
				priv->mov_abuf += (512 -((int)(priv->mov_abuf) & 511));

				priv->mov_abuf_pos = atrak_start - (atrak_start & 511);				
				priv->mov_abuf_len = read_mov_buffer(demuxer->stream, priv->mov_abuf, priv->mov_abuf_pos, MOV_VIDEO_BUF_SIZE);
				mp_msg(MSGT_DEMUX,MSGL_V,"A[0]: fill at %d\n", priv->mov_abuf_pos);
			}
		}

//		priv->mov_vbuf = salloc(MOV_VIDEO_BUF_SIZE);
		// Raymond 2008/06/02
		priv->mov_vbuf = salloc(MOV_VIDEO_BUF_SIZE+512);
		priv->mov_vbuf += (512 -((int)(priv->mov_vbuf) & 511));

		priv->mov_vbuf_pos = vtrak_start - (vtrak_start & 511);		
		priv->mov_vbuf_len = read_mov_buffer(demuxer->stream, priv->mov_vbuf, priv->mov_vbuf_pos, MOV_VIDEO_BUF_SIZE);
		mp_msg(MSGT_DEMUX,MSGL_V,"V[0]: fill at %d\n", priv->mov_vbuf_pos);
		
	}
#endif
    
    // setup sh pointers:
    if(demuxer->audio->id>=0){
	sh_audio_t* sh=demuxer->a_streams[demuxer->audio->id];
	if(sh){
	    demuxer->audio->sh=sh; sh->ds=demuxer->audio;
		sh->seek_audio_frame = -1;	// Raymond 2007/06/06
	} else {
	    mp_msg(MSGT_DEMUX, MSGL_ERR, "MOV: selected audio stream (%d) does not exists\n",demuxer->audio->id);
	    demuxer->audio->id=-2;
	}
    }
    if(demuxer->video->id>=0){
	sh_video_t* sh=demuxer->v_streams[demuxer->video->id];
	if(sh){
	    demuxer->video->sh=sh; sh->ds=demuxer->video;
		// Raymond 2008/06/04
		if(best_v_len && sh->i_bps == 0)
			sh->i_bps = vtrak_size * sh->fps / best_v_len;
	} else {
	    mp_msg(MSGT_DEMUX, MSGL_ERR, "MOV: selected video stream (%d) does not exists\n",demuxer->video->id);
	    demuxer->video->id=-2;
	}
    }

	// Raymond 2008/06/04 - fill first video sample to save 50 ms time
	if( is_mjpeg == 1 )
		demux_mov_fill_buffer(demuxer, demuxer->video);

#if 1
    if(verbose>2){
	for(t_no=0;t_no<priv->track_db;t_no++){
	    mov_track_t* trak=priv->tracks[t_no];
	    if(trak->type==MOV_TRAK_GENERIC){
		int i;
		int fd;
		char name[20];
		mp_msg(MSGT_DEMUX, MSGL_INFO, "MOV: Track #%d: Extracting %d data chunks to files\n",t_no,trak->samples_size);
		for (i=0; i<trak->samples_size; i++)
		{
		    int len=trak->samples[i].size;
		    char buf[len];
		    stream_seek(demuxer->stream, trak->samples[i].pos);
		    snprintf(name, 20, "t%02d-s%03d.%s", t_no,i,
			(trak->media_handler==MOV_FOURCC('f','l','s','h')) ?
			    "swf":"dump");
		    fd = open(name, O_CREAT|O_WRONLY);
//		    { int j;
//			for(j=0;j<trak->stdata_len-3; j++)
//			    printf("stdata[%d]=0x%X ize=0x%X\n",j,char2int(trak->stdata,j),MOV_FOURCC('z','l','i','b'));
//		    }
		    if( //trak->media_handler==MOV_FOURCC('s','p','r','t') &&
			trak->stdata_len>=16 &&
			char2int(trak->stdata,12)==MOV_FOURCC('z','l','i','b') 
		    ){
			int newlen=stream_read_dword(demuxer->stream);
#ifdef HAVE_ZLIB
			// unzip:
			z_stream zstrm;
			int zret;
			char buf2[newlen];

			len-=4;
			stream_read(demuxer->stream, buf, len);

	                zstrm.zalloc          = (alloc_func)0;
	                zstrm.zfree           = (free_func)0;
	                zstrm.opaque          = (voidpf)0;
	                zstrm.next_in         = buf;
	                zstrm.avail_in        = len;
	                zstrm.next_out        = buf2;
	                zstrm.avail_out       = newlen;

	                zret = inflateInit(&zstrm);
			zret = inflate(&zstrm, Z_NO_FLUSH);
			if(newlen != zstrm.total_out)
	    		    mp_msg(MSGT_DEMUX, MSGL_WARN, "Warning! unzipped frame size differs hdr: %d  zlib: %d\n",newlen,zstrm.total_out);
			
			write(fd, buf2, newlen);
		    } else {
#else
			len-=4;
			printf("******* ZLIB COMPRESSED SAMPLE!!!!! (%d->%d bytes) *******\n",len,newlen);
		    }
		    {
#endif
			stream_read(demuxer->stream, buf, len);
			write(fd, buf, len);
		    }
		    close(fd);
		}
	    }
	}
    }
    demuxer->stream->eof = 0;
#endif

    return 1;
}

/**
 * \brief return the mov track that belongs to a demuxer stream
 * \param ds the demuxer stream, may be NULL
 * \return the mov track info structure belonging to the stream,
 *          NULL if not found
 */
static mov_track_t *stream_track(mov_priv_t *priv, demux_stream_t *ds) {
  if (ds && (ds->id >= 0) && (ds->id < priv->track_db))
    return priv->tracks[ds->id];
  return NULL;
}

// Raymond 2008/06/03
void mov_get_audio_packet(demux_stream_t *ds,unsigned char * tbuffer,int len,float pts,off_t pos,int flags)
{
  	demux_packet_t* dp=new_demux_packet_phy(len);
  	dp->buffer = tbuffer;
  	dp->pts=pts; //(float)pts/90000.0f;
  	dp->pos=pos;
  	dp->flags=flags;
  	// append packet to DS stream:
  	ds_add_packet(ds,dp);
}

// return value:
//     0 = EOF or no stream found
//     1 = successfully read a packet
int demux_mov_fill_buffer(demuxer_t *demuxer,demux_stream_t* ds){
    mov_priv_t* priv=demuxer->priv;
    mov_track_t* trak=NULL;
    float pts;
    int x;
    off_t pos;
    int i, frame;	//Barry 2007-10-22
    

    trak = stream_track(priv, ds);
    if (!trak) return 0;
    //fprintf(stderr,"**demux_mov_fill_buffer trak->pos is %d**\n",trak->pos);

if(trak->samplesize){
    // read chunk:
    if(trak->pos>=trak->chunks_size) return 0; // EOF
#ifdef MOV_READ_PHY  		// Raymond 2008/03/19  
    if( mov_readphy == 1 )
    {
    }
    else	
#endif		
    stream_seek(demuxer->stream,trak->chunks[trak->pos].pos);
    pts=(float)(trak->chunks[trak->pos].sample*trak->duration)/(float)trak->timescale;
    if(trak->samplesize!=1)
    {
	mp_msg(MSGT_DEMUX, MSGL_DBG2, "WARNING! Samplesize(%d) != 1\n",
	    trak->samplesize);
#if 1	// Raymond 2007/06/05
	if((trak->fourcc != MOV_FOURCC('t','w','o','s')) && (trak->fourcc != MOV_FOURCC('s','o','w','t')))
		x=trak->chunks[trak->pos].size*trak->samplesize;				
	else
       	x=trak->chunks[trak->pos].size;
#else
	x=trak->chunks[trak->pos].size*trak->samplesize;
#endif
    }
    else
	x=trak->chunks[trak->pos].size;
//    printf("X = %d\n", x);
    /* the following stuff is audio related */
    if (trak->type == MOV_TRAK_AUDIO){
      if(trak->stdata_len>=44 && trak->stdata[9]>=1 && char2int(trak->stdata,28)>0){
        // stsd version 1 - we have audio compression ratio info:
	x/=char2int(trak->stdata,28); // samples/packet
//	x*=char2int(trak->stdata,32); // bytes/packet
	x*=char2int(trak->stdata,36); // bytes/frame
      } else {
	if(ds->ss_div && ds->ss_mul){
	    // workaround for buggy files like 7up-high-traffic-areas.mov,
	    // with missing stsd v1 header containing compression rate
	    x/=ds->ss_div; x*=ds->ss_mul; // compression ratio fix  ! HACK !
	} else {
	    x*=trak->nchannels;
	    x*=trak->samplebytes;
	}
      }
      mp_msg(MSGT_DEMUX, MSGL_DBG2, "Audio sample %d bytes pts %5.3f\n",trak->chunks[trak->pos].size*trak->samplesize,pts);
      
    } /* MOV_TRAK_AUDIO */
    pos=trak->chunks[trak->pos].pos;
#if 1	// Raymond 2008/06/04
	// Raymond 2008/06/03
	if( (trak->chunks[trak->pos].buf != NULL) && (x == trak->chunks[trak->pos].size) )
	{
		mp_msg(MSGT_DEMUX,MSGL_V,"A: x = %d, pos = %d\n", x, pos);

		mov_get_audio_packet(ds, trak->chunks[trak->pos].buf, x, pts, pos, 0);	// Raymond 2008/06/03
		++trak->pos;
		if(trak->pos == trak->chunks_size)
		{
			ds->eof = 1;
			return 0;
		}

		return 1;
	}
	else
		mp_msg(MSGT_DEMUX,MSGL_V,"A: x = %d, size = %d\n", x, trak->chunks[trak->pos].size);
#endif

#ifdef MOV_READ_PHY		// Raymond 2008/03/20  
    if( mov_readphy == 1 )
    {
//		printf("pos = %d\n", pos );
//		printf("x = %d\n", x );
		
		if( (pos >= priv->mov_vbuf_pos) && (pos + x <= priv->mov_vbuf_pos + priv->mov_vbuf_len) )
		{
		}
		else if( (pos >= priv->mov_abuf_pos) && (pos + x <= priv->mov_abuf_pos + priv->mov_abuf_len) ) 
		{			
//			printf("\t[%d] Audio in mov_vbuf: pos = %d\n", frame, pos);
		}
		else
		{			
			priv->mov_abuf_pos = pos - (pos & 511);
			priv->mov_abuf_len = read_mov_buffer(demuxer->stream, priv->mov_abuf, priv->mov_abuf_pos, MOV_AUDIO_BUF_SIZE);	// 32 KB
			mp_msg(MSGT_DEMUX,MSGL_V,"\t\t\t\tA(1): refill at %d\n", priv->mov_abuf_pos);
		}
    }	
#endif
} else {
    //Barry 2007-10-22
    //int frame=trak->pos;
    frame=trak->pos;
	
    // editlist support:
    if(trak->type == MOV_TRAK_VIDEO && trak->editlist_size>=1){
	// find the right editlist entry:
	if(frame<trak->editlist[trak->editlist_pos].start_frame)
	    trak->editlist_pos=0;
	while(trak->editlist_pos<trak->editlist_size-1 &&
	    frame>=trak->editlist[trak->editlist_pos+1].start_frame)
		++trak->editlist_pos;
	if(frame>=trak->editlist[trak->editlist_pos].start_frame+
	    trak->editlist[trak->editlist_pos].frames) return 0; // EOF
	// calc real frame index:
	frame-=trak->editlist[trak->editlist_pos].start_frame;
	frame+=trak->editlist[trak->editlist_pos].start_sample;
	// calc pts:
	pts=(float)(trak->samples[frame].pts+
	    trak->editlist[trak->editlist_pos].pts_offset)/(float)trak->timescale;
    } else {
	if(frame>=trak->samples_size) return 0; // EOF
	pts=(float)trak->samples[frame].pts/(float)trak->timescale;
    }
    // Raymond 2007/05/15
   if( get_preview == 1 && trak->keyframes_size > 3 )
    {	
	frame = trak->keyframes[frame];
//    	printf("MOV: key frame %d\n", frame);
    }

  //Barry 2007-10-22	3GP-only one keyframe
  if (speed_mult > 0 && trak->keyframes_size == 1)
  	Only_One_I = 1;
  if (speed_mult < 0 && trak->keyframes_size == 1)
  	ds->eof = 1;

#if 1	//Barry 2007-10-22
   //	forward
//   if (speed_mult >= 1 && trak->keyframes_size > 3 && is_mjpeg==0){
      if (speed_mult >= 1 && is_mjpeg==0){	// Raymond 2008/04/10
   	if (forward_keyframe_idx >= trak->keyframes_size-1)
		ds->eof = 1;
	
   	if (!forward_keyframe_idx){
		for(i=0;i<trak->keyframes_size;i++){
			if (trak->keyframes[i] >= frame){
				frame = trak->keyframes[i];
				forward_keyframe_idx = i+1;
				break;
			}
		}
	}else{
		frame = trak->keyframes[forward_keyframe_idx];
		forward_keyframe_idx++;
	}
    	//printf("MOV(forward): key frame %d\n", frame);
   }
   else
   	forward_keyframe_idx=0;

   //backward
//   if (speed_mult < 0 && trak->keyframes_size > 3 && is_mjpeg==0){
   if (speed_mult < 0 && is_mjpeg==0){		// Raymond 2008/04/10
   	if (backward_keyframe_idx == 0)
		ds->eof = 1;
	
	if (backward_keyframe_idx == -1){
		for(i=(trak->keyframes_size-1);i>=0;i--){
			if (trak->keyframes[i] <= frame){
				frame = trak->keyframes[i];
				backward_keyframe_idx = i-1;
				break;
			}
		}
	}else{
		frame = trak->keyframes[backward_keyframe_idx];
		backward_keyframe_idx--;
	}
	//printf("MOV(backward): key frame %d\n", frame);
   }
   else
   	backward_keyframe_idx=-1;
#endif

    x=trak->samples[frame].size;
    pos=trak->samples[frame].pos;

#ifdef MOV_READ_PHY		// Raymond 2008/03/11
	if( mov_readphy == 1 )
	{
		//Barry 2008-08-28
		if (trak->samples[frame].pos >= demuxer->stream->end_pos)
		{
			ds->eof = 1;
			return 0;
		}
			
		if( trak->type == MOV_TRAK_VIDEO )
		{		
	//		printf("[%d] pos = %d\n", frame, trak->samples[frame].pos);
		
			if( (pos < priv->mov_vbuf_pos) || (pos > priv->mov_vbuf_pos + priv->mov_vbuf_len) )
			{				
				priv->mov_vbuf_pos = pos - (pos & 511);
				priv->mov_vbuf_len = read_mov_buffer(demuxer->stream, priv->mov_vbuf, priv->mov_vbuf_pos, MOV_VIDEO_BUF_SIZE);
				mp_msg(MSGT_DEMUX,MSGL_V,"V(1): refill at %d\n", priv->mov_vbuf_pos);
			}
		}
		else if( trak->type == MOV_TRAK_AUDIO )
		{
			if( priv->mov_vbuf && (pos >= priv->mov_vbuf_pos) && (pos + x <= priv->mov_vbuf_pos + priv->mov_vbuf_len) )
			{
	//			printf("\t\t\t[%d] Audio in mov_vbuf: pos = %d\n", frame, pos);
			}
			else if( priv->mov_abuf && (pos >= priv->mov_abuf_pos) && (pos + x <= priv->mov_abuf_pos + priv->mov_abuf_len) ) 
			{			
	//			printf("\t\t\t[%d] Audio in mov_abuf: pos = %d\n", frame, pos);
			}
			else
			{
				priv->mov_abuf_pos = pos - (pos & 511);
				priv->mov_abuf_len = read_mov_buffer(demuxer->stream, priv->mov_abuf, priv->mov_abuf_pos, 32768);	// 32 KB
				mp_msg(MSGT_DEMUX,MSGL_V,"\t\t\t\tA(2): refill at %d\n", priv->mov_abuf_pos);
			}
		}
	}	
	else
	{
		stream_seek(demuxer->stream,trak->samples[frame].pos);
//		printf("[%d] stream_seek pos = %d\n", frame, pos);
	}
#else
	stream_seek(demuxer->stream,trak->samples[frame].pos);
#endif

    //Barry 2007-10-22
    if (speed_mult != 0 && is_mjpeg==0 && Only_One_I == 0)
		pts = trak->samples[frame].pts;

//	printf("[%3d] pos = %ld, len = %d\n", frame, pos, trak->samples[frame].size);

}
if(trak->pos==0 && trak->stream_header_len>0){
//printf("trak->stream_header_len = %d\n", trak->stream_header_len);
	
    // we have to prepend the stream header...
    demux_packet_t* dp=new_demux_packet(x+trak->stream_header_len);
    memcpy(dp->buffer,trak->stream_header,trak->stream_header_len);
#ifdef MOV_READ_PHY  		// Raymond 2008/03/19  
    if( mov_readphy == 1 )
    {
		if( trak->type == MOV_TRAK_VIDEO )
		{
			memcpy(dp->buffer + trak->stream_header_len, priv->mov_vbuf + pos - priv->mov_vbuf_pos, x);
		}
		else if(  trak->type == MOV_TRAK_AUDIO )
		{	
			memcpy(dp->buffer + trak->stream_header_len, priv->mov_abuf + pos - priv->mov_abuf_pos, x);
		}
    }
    else	
#endif			
    stream_read(demuxer->stream,dp->buffer+trak->stream_header_len,x);
    sfree(trak->stream_header);
    trak->stream_header = NULL;
    trak->stream_header_len = 0;
    dp->pts=pts;
    dp->flags=0;
    dp->pos=pos; // FIXME?
    ds_add_packet(ds,dp);
} 
else
{
#ifdef MOV_READ_PHY 	// Raymond 2008/03/19  
    	if( mov_readphy == 1 )
    	{
		if( trak->type == MOV_TRAK_VIDEO )
		{
			// Raymond 2008/08/06
			int skip_packet = 0;
			packet_cnt++;
			if( speed_mult != 0 && is_mjpeg == 1 )
			{
				if( packet_cnt % mjpeg_skip2 < mjpeg_skip1 )
					skip_packet = 0;
				else
					skip_packet = 1;					
			}	

			if( skip_packet == 0 )	// Raymond 2008/08/06
			{
			demux_packet_t* dp=new_demux_packet(x);

			int plen = priv->mov_vbuf_pos + MOV_VIDEO_BUF_SIZE - pos;

			if( plen >= x )
				memcpy(dp->buffer, priv->mov_vbuf + pos - priv->mov_vbuf_pos, x);
			else
				memcpy(dp->buffer, priv->mov_vbuf + pos - priv->mov_vbuf_pos, plen);

	//		printf("dp->buffer = %X %X %X %X\n", dp->buffer[0], dp->buffer[1], dp->buffer[2], dp->buffer[3]);
						
			if( plen < x )
			{	
				priv->mov_vbuf_pos += MOV_VIDEO_BUF_SIZE;
				priv->mov_vbuf_len = read_mov_buffer(demuxer->stream, priv->mov_vbuf, priv->mov_vbuf_pos, MOV_VIDEO_BUF_SIZE);
				mp_msg(MSGT_DEMUX,MSGL_V,"V(2): refill at %d\n", priv->mov_vbuf_pos);
				
				memcpy(dp->buffer + plen, priv->mov_vbuf, x - plen);
			}
			
			dp->pts=pts;
	    		dp->flags=0;
	    		dp->pos=pos; // FIXME?
	    		ds_add_packet(ds,dp);
			}
		}
		else if(  trak->type == MOV_TRAK_AUDIO )
		{
			// get from video buffer
			if ( priv->mov_vbuf && (pos >= priv->mov_vbuf_pos) && (pos +x < priv->mov_vbuf_pos + priv->mov_vbuf_len) )
			{
				ds_read_audio_packet(ds,priv->mov_vbuf + pos - priv->mov_vbuf_pos, x, pts, pos, 0);
			}
			else if( priv->mov_abuf != NULL )	// get from audio buffer
			{
				ds_read_audio_packet(ds,priv->mov_abuf + pos - priv->mov_abuf_pos, x, pts, pos, 0);
			}
			else
				ds_read_packet(ds,demuxer->stream,x,pts,pos,0);
		}
    	}	
	else	
#endif		
	 // Raymond 2007/05/02
	if( (avsync_mjpeg == 1 || (speed_mult != 0 && is_mjpeg == 1)) && trak->type == MOV_TRAK_VIDEO )	//Barry 2007-11-14
	{
    		packet_cnt++;	
		
//	    	if( packet_cnt % mjpeg_skip != (mjpeg_skip - 1) )
		if( packet_cnt % mjpeg_skip2 < mjpeg_skip1 )	// Raymond 2007/05/03 - skip mjpeg_skip1 per mjpeg_skip2 frame
		{
		    	ds_read_packet(ds,demuxer->stream,x,pts,pos,0);
//			printf("(%d) READ[V] %5.3f  (%d) , len = %d\n",packet_cnt, pts, pos,x);
	    	}
    	}
	else		
    		ds_read_packet(ds,demuxer->stream,x,pts,pos,0);
	
	//Barry 2007-10-22
	if (speed_mult != 0 && Only_One_I == 0)
		ds->pts = pts=(float)trak->samples[frame].pts/(float)trak->timescale;
}

	// Raymond 2007/04/13
	if(ds->eof == 1)
		return 0;
	
//  fprintf(stderr,"**Leaving demux_mov_fill_buffer trak at 0x%.8x, trak->pos is %d, header len is %d **\n",trak, trak->pos,trak->stream_header_len);

    //Barry 2007-10-22
    //++trak->pos;
    if (speed_mult != 0 && is_mjpeg==0 && Only_One_I == 0)
		trak->pos = frame;
    else if (speed_mult < 0 && is_mjpeg==1){
		--trak->pos;
		if (trak->pos < 0){
			ds->eof = 1;
			return 0;
		}
    }
    else
		++trak->pos;

    return 1;    
}

static float mov_seek_track(mov_track_t* trak,float pts,int flags){

  //    printf("MOV track seek called  %5.3f  \n",pts);
    if(flags&2) pts*=trak->length; else pts*=(float)trak->timescale;

if(trak->samplesize){
    int sample=pts/trak->duration;
    //    printf("MOV track seek - chunk: %d  (pts: %5.3f  dur=%d)  \n",sample,pts,trak->duration);
    if(!(flags&1)) sample+=trak->chunks[trak->pos].sample; // relative
    trak->pos=0;
    while(trak->pos<trak->chunks_size && trak->chunks[trak->pos].sample<sample) ++trak->pos;
    pts=(float)(trak->chunks[trak->pos].sample*trak->duration)/(float)trak->timescale;
} else {
    unsigned int ipts;
    if(!(flags&1)) pts+=trak->samples[trak->pos].pts;
    if(pts<0) pts=0;
    ipts=pts;
    //    printf("MOV track seek - sample: %d  \n",ipts);
    for(trak->pos=0;trak->pos<trak->samples_size;++trak->pos){
	if(trak->samples[trak->pos].pts>=ipts) break; // found it!
    }
    if(trak->keyframes_size){
	// find nearest keyframe
	int i;
	for(i=0;i<trak->keyframes_size;i++){
	    if(trak->keyframes[i]>=trak->pos) break;
	}
#if 1	// Raymond 2007/05/04
	if( i == trak->keyframes_size )
		i--;
	else
#endif
	if(i>0 && (trak->keyframes[i]-trak->pos) > (trak->pos-trak->keyframes[i-1]))
	  --i;
	if (trak->keyframes_size > 1)	//Barry 2007-10-22
	trak->pos=trak->keyframes[i];
	//	printf("nearest keyframe: %d  \n",trak->pos);
    }
    pts=(float)trak->samples[trak->pos].pts/(float)trak->timescale;
}

//    printf("MOV track seek done:  %5.3f  \n",pts);

return pts;
}

void demux_seek_mov(demuxer_t *demuxer,float pts,int flags){
    mov_priv_t* priv=demuxer->priv;
    demux_stream_t* ds;
    mov_track_t* trak;

    //    printf("MOV seek called  %5.3f  flag=%d  \n",pts,flags);
    
    ds=demuxer->video;
    trak = stream_track(priv, ds);
    if (trak) {
	//if(flags&2) pts*=(float)trak->length/(float)trak->timescale;
	//if(!(flags&1)) pts+=ds->pts;
	pts=ds->pts=mov_seek_track(trak,pts,flags);
	flags=1; // absolute seconds
    }

    ds=demuxer->audio;
    trak = stream_track(priv, ds);
    if (trak) {
  sh_audio_t* sh_audio = demuxer->audio->sh;
	//if(flags&2) pts*=(float)trak->length/(float)trak->timescale;
	//if(!(flags&1)) pts+=ds->pts;
	ds->pts=mov_seek_track(trak,pts,flags);
	if (demuxer->video->id < 0)
	  ((sh_audio_t*)ds->sh)->delay = ds->pts;
	// PS - force the codec to resync
	sh_audio->seek_audio_frame = trak->pos;	// Raymond 2007/06/06
#ifdef __ARC_MPLAYER__
	resync_audio_stream(sh_audio);
#endif
	// end PS
    }

}

int demux_mov_control(demuxer_t *demuxer, int cmd, void *arg){
  mov_track_t* track;

  // try the video track
  track = stream_track(demuxer->priv, demuxer->video);
  if (!track || !track->length)
    // otherwise try to get the info from the audio track
    track = stream_track(demuxer->priv, demuxer->audio);

  if (!track || !track->length)
    return DEMUXER_CTRL_DONTKNOW;

  switch(cmd) {
    case DEMUXER_CTRL_GET_TIME_LENGTH:
      if (!track->timescale)
        return DEMUXER_CTRL_DONTKNOW;
      *((unsigned long *)arg) = track->length / track->timescale;
      return DEMUXER_CTRL_OK;

    case DEMUXER_CTRL_GET_PERCENT_POS:
      {
        off_t pos = track->pos;
        if (track->durmap_size >= 1)
          pos *= track->durmap[0].dur;
        *((int *)arg) = (int)(100 * pos / track->length);
        return DEMUXER_CTRL_OK;
      }
  }
  return DEMUXER_CTRL_NOTIMPL;
}

