#ifdef BUILD_SK2851		//Barry 2008-05-09

/*
 * arc_pgu.h - Register definitions for the ARC pixel graphics unit.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * THIS	 SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 * WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.	 IN
 * NO EVENT  SHALL   THE AUTHOR	 BE  LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright 2005 (C) ARC International (www.arc.com). 
 * 
 */

#ifndef _ASM_ARC_PGU_H
#define _ASM_ARC_PGU_H

#include <linux/fb.h>

/*
 *	Scalar information.
 *
 */
enum scalar_state
{
	SCALAR_Y	= 0,
	SCALAR_U	= 1,
	SCALAR_V	= 2,
	SCALAR_IDLE	= 3,
	SCALAR_STOP = 4,
};

struct _buf_info
{
	unsigned int dec_idx, scale_idx;
	unsigned int dec_w, dec_h;
	unsigned int scale_w, scale_h;
	unsigned int dec_rw, dec_rh;
	unsigned int scale_rw, scale_rh;
};

struct _disp_scalar_info
{
	unsigned short en;
	unsigned short width;
	unsigned short height;	
	unsigned short src_x;
	unsigned short src_y;
	unsigned short src_x_size;
	unsigned short src_y_size;
	unsigned short dest_x;
	unsigned short dest_y;
	unsigned short dest_x_size;
	unsigned short dest_y_size;
	unsigned long src_y_offset;
	unsigned long src_uv_offset;	
};

/*----------------------------------------------------------------------------*/
/* YUV buffer layout - all sizes and offsets in bytes. Note that base addresses tend to be 
   passed around as unsigned longs, in which case you'll need to typecast before applying 
   any offsets. */

struct yuv_info {
  unsigned int displayed_buffer;
  unsigned long phys_base;
  unsigned long virt_base;
  unsigned long yuv_size;
  unsigned long alignment;
  unsigned long num_buffers;
  unsigned long start_offset;
  unsigned long width;
  unsigned long height;
  unsigned long u_offset;
  unsigned long v_offset;
  unsigned long y_stride;
  unsigned long u_stride;
  unsigned long v_stride;
};


/*--------------------------------------------------------------------------------------------*/
#define ARCPGUFB_GETYUV					0xffff0001	// Send the fb_info to the user space.
#define ARCPGUFB_SWITCH_YUV				0xffff0004
#define ARCPGUFB_BKLGHT_ON				0xffff0006
#define ARCPGUFB_START_DISPLAY			0xffff0007
#define ARCPGUFB_STOP_DISPLAY			0xffff0008
#define ARCPGUFB_GET_MAIN_MODE			0xffff000b
#define ARCPGUFB_SET_MAIN_MODE			0xffff000c
#define ARCPGUFB_SET_RGB_BUF			0xffff000d
#define ARCPGUFB_GET_YUV_RES			0xffff0012
#define ARCPGUFB_SET_YUV_RES			0xffff0013

#define ARCPGUFB_GET_ORIGIN				0xffff001e
#define ARCPGUFB_SET_ORIGIN				0xffff001f

#define ARCPGUFB_ALLOC					0xffff0020
#define ARCPGUFB_FREE					0xffff0021
#define ARCPGUFB_SEND					0xffff0022
#define ARCPGUFB_FRAMERATE_SET			0xffff0023
#define ARCPGUFB_QUEUE_LEN_GET			0xffff0024
#define ARCPGUFB_FLUSH					0xffff0025
#define ARCPGUFB_FRAME_DROP				0xffff0026
#define ARCPGUFB_PAUSE					0xffff0027
#define ARCPGUFB_RESUME					0xffff0028
#define ARCPGUFB_RESET_SYNC				0xffff0029
#define ARCPGUFB_NUM_FREE_BUFS			0xffff002a
#define ARCPGUFB_PAUSE_MS				0xffff002b
#define ARCPGUFB_GET_YUV_BASE			0xffff002c /* return address of yuv buffer n */
#define ARCPGUFB_SET_VIDEO_YUV_RES		0xffff002d	// set yuv buffer res. in video mode.

//Flowbee
#define ARCPGUFB_GET_DISP_INDEX 		0xffff0032	// get the display mode (NTSC, PAL, LCD)
#define ARCPGUFB_SET_DISP_INDEX 		0xffff0033	// set the display mode (NTSC, PAL, LCD)
#define ARCPGUFB_GET_TV_INTERFACE		0xffff0034	// get tv output interface
#define ARCPGUFB_SET_TV_INTERFACE		0xffff0035	// set tv output interface
#define ARCPGUFB_GET_DEFAULT_DISP_INDEX	0xffff003a	// get display index of startup stage

//Flowbee
#define ARCPGUFB_GET_DEFAULT_YUV		0xffff0040	// get the default yuv width and height.
#define ARCPGUFB_SET_DEC_YUV_RES		0xffff0041	// set decoder buffer resolution.
#define ARCPGUFB_GET_DEC_YUV			0xffff0042	// get decoder buffer info.
#define ARCPGUFB_GET_DEC_YUV_BASE		0xffff0043	// get decoder buffer base address.
#define ARCPGUFB_DEC_ALLOC				0xffff0044	// allocate decoder buffer.
#define ARCPGUFB_DEC_FREE				0xffff0045	// free decoder buffer.
#define ARCPGUFB_SCALE_ALLOC			0xffff0046	// allocate display buffer.
#define ARCPGUFB_SCALE_FREE				0xffff0047	// free display buffer.
#define ARCPGUFB_SCALE_SEND				0xffff0048	// send display buffer.
#define ARCPGUFB_NUM_FREE_DEC_BUFS		0xffff0049	// get the num of free decoder buffers.
#define ARCPGUFB_NUM_FREE_SCALE_BUFS	0xffff004A	// get the num of free display buffers.
#define ARCPGUFB_GET_FB_SIZE			0xffff004C	// get frame buffer size.

//== Flowbee 2007-11-12 Net config
#define ARCPGUFB_GET_NET_CONFIG_INFO	0xffff0070
#define ARCPGUFB_SET_NET_CONFIG_INFO	0xffff0071


#if 1	//Barry 2008-06-18
//== Flowbee 2008-04-28 Osd function
#define ARCPGUFB_ENABLE_OSD				0xffff00a0
#define ARCPGUFB_SELECT_OSD				0xffff00a1
#define ARCPGUFB_SET_OSD_ALPHA			0xffff00a2
#define ARCPGUFB_SET_OSD_START_POSITION	0xffff00a3
#define ARCPGUFB_SET_OSD_END_POSITION	0xffff00a4
#define ARCPGUFB_SET_OSD_ADDR			0xffff00a5
#define ARCPGUFB_SET_OSD_SIZE				0xffff00a6
#endif

#define ARCPGUFB_GET_SCALAR_STATE		0xffff00f0
#define ARCPGUFB_START_SCALAR			0xffff00f1
#define ARCPGUFB_SET_SCALAR_INFO		0xffff00f3
//-- Flowbee 2008-03-13
#define ARCPGUFB_SET_DISP_RES			0xffff0100

#define ARCPGUFB_TEST_MODE				0xffff0fff	// test mode for user specified func.

#if 0	//Barry 2008-08-18
#define ARCPGUFB_SETYUV					0xffff0002	// Set fb_info, not used.
#define ARCPGUFB_FILLYUV				0xffff0003	// not used
#define ARCPGUFB_BKLGHT_OFF				0xffff0005
#define ARCPGUFB_GET_PGU_STATE			0xffff0009
#define ARCPGUFB_SET_PGU_STATE			0xffff000a
#define ARCPGUFB_GET_RGB_BUF			0xffff000e
#define ARCPGUFB_GET_FB_MAIN			0xffff000f
#define ARCPGUFB_CHECKPOINT				0xffff0010
#define ARCPGUFB_RESTORE				0xffff0011
#define ARCPGUFB_RESET					0xffff0012
#define ARCPGUFB_GET_FB_INFO			0xffff0014
#define ARCPGUFB_SET_OL_MODE			0xffff0015
#define ARCPGUFB_GET_OL_MODE			0xffff0016
#define ARCPGUFB_GET_OL_COLKEY			0xffff0017
#define ARCPGUFB_SET_OL_COLKEY			0xffff0018
#define ARCPGUFB_GET_OL_START			0xffff0019
#define ARCPGUFB_SET_OL_START			0xffff001a
#define ARCPGUFB_GET_OL_END				0xffff001b
#define ARCPGUFB_SET_OL_END				0xffff001c
#define ARCPGUFB_SET_OL_OFFSET			0xffff001d

#define ARCPGUFB_GET_IRVALUE			0xffff0030
#define ARCPGUFB_SET_IRVALUE			0xffff0031

#define ARCPGUFB_GET_MPLAYER_PLAY_TIME	0xffff0036	// get video play time
#define ARCPGUFB_SET_MPLAYER_PLAY_TIME	0xffff0037	// set video play time
#define ARCPGUFB_GET_MPLAYER_DURATION	0xffff0038	// get video play time
#define ARCPGUFB_SET_MPLAYER_DURATION	0xffff0039	// set video play time
#define ARCPGUFB_GET_MPLAYER_STATUS		0xffff003b	// get mplayer status
#define ARCPGUFB_SET_MPLAYER_STATUS		0xffff003c	// set mplayer status

#define ARCPGUFB_SWITCH_USER_DEFINED_YUV	0xffff004B	// set user defined YUV address
#define ARCPGUFB_SET_SCALE_YUV_RES		0xffff004D
// Robert
#define ARCPGUFB_WAIT_BLANK				0xffff004E
#define ARCPGUFB_CHECK_BLANK			0xffff004F
#define ARCPGUFB_SWITCH_YUV_WITH_INTERRUPT		0xffff0050

//== Flowbee 2007-11-09 Advertisment
#define ARCPGUFB_MPLAYER_ADVERTISMENT	0xffff0060
#define ARCPGUFB_ADVERTISMENT_UPDATE	0xffff0061
#define ARCPGUFB_GET_ADVER_VIDEO_INFO	0xffff0062
#define ARCPGUFB_SET_ADVER_VIDEO_INFO	0xffff0063
#define ARCPGUFB_GET_ADVER_JPEG_INFO	0xffff0064
#define ARCPGUFB_SET_ADVER_JPEG_INFO	0xffff0065

//== Flowbee 2008-02-21 Terawins scale ratio
#define ARCPGUFB_SET_TCON_SCALE_TABLE	0xffff00c0

//== Flowbee 2008-02-15 OSD function
#define ARCPGUFB_TCON_SET_COMMAND		0xffff00d0
#define ARCPGUFB_TCON_SET_OSD_CMD		0xffff00d1
#define ARCPGUFB_TCON_SET_OSD_RAMADDR	0xffff00d2
#define ARCPGUFB_TCON_SET_OSD_RAMDATA	0xffff00d3
#define ARCPGUFB_TCON_SET_OSD_LINE_DATA	0xffff00d4

#define ARCPGUFB_SET_OSD_SUBTITLE_INFO	0xffff00dc
#define ARCPGUFB_SET_OSD_SPEED_INFO		0xffff00dd
#define ARCPGUFB_INIT_TCON_SCALE_RATIO	0xffff00de
#define ARCPGUFB_SET_TCON_SCALE_RATIO	0xffff00df

//== Flowbee 2007-12-20 OSD function
#define ARCPGUFB_SHOW_MUTE_VOL			0xffff00e0
#define ARCPGUFB_SHOW_ICON				0xffff00e1
#define ARCPGUFB_SHOW_VIDEO_SUBTITLE	0xffff00e2
#define ARCPGUFB_SHOW_VIDEO_TIME		0xffff00e3
#define ARCPGUFB_SHOW_VIDEO_SPEED		0xffff00e4
#define ARCPGUFB_CLEAR_MUTE_VOL			0xffff00e8
#define ARCPGUFB_CLEAR_ICON				0xffff00e9
#define ARCPGUFB_CLEAR_VIDEO_SUBTITLE	0xffff00ea
#define ARCPGUFB_CLEAR_VIDEO_TIME		0xffff00eb
#define ARCPGUFB_CLEAR_VIDEO_SPEED		0xffff00ec
#define ARCPGUFB_GET_VIDEO_TIME_ONOFF 	0xffff00ed
#define ARCPGUFB_SET_VIDEO_TIME_ONOFF 	0xffff00ee
#define ARCPGUFB_CLEAR_LCD_OSD			0xffff00ef

// Flowbee
#define ARCPGUFB_STOP_SCALAR			0xffff00f2
#define ARCPGUFB_SET_CONTRAST			0xffff00f4
#define ARCPGUFB_SET_PWM				0xffff00f5
#define ARCPGUFB_DISABLE_MUTE_VOL_OSD	0xffff00f6
#define ARCPGUFB_ENABLE_TV_OSD			0xffff00f7
#define ARCPGUFB_DISABLE_TV_OSD			0xffff00f8
#define ARCPGUFB_UPDATE_OSD				0xffff00f9
#define ARCPGUFB_ENABLE_OL				0xffff00fa
#define ARCPGUFB_DISABLE_OL				0xffff00fb
#define ARCPGUFB_GET_OL_ALPHA			0xffff00fc
#define ARCPGUFB_SET_OL_ALPHA			0xffff00fd
#define ARCPGUFB_GET_STATCTRL			0xffff00fe
#define ARCPGUFB_SET_STATCTRL			0xffff00ff
#endif

/*----------------------------------------------------------------------------*/

/*	TV encoder Control Registers  */

/*
 *	Base+0x00	TVE_FORMAT_REG:
 *					TV encoder output format control register.
 *					Controlling the output format of TV signal.
 *					(NTSC, PAL etc...)
 */
#define TVE_FORMAT_REG_OFFSET			0x00000000					

/*		
 *			[0-1] TVF:			TV standard format select(should select correct 
 *								Black0IRE)
 *				00: NTSC-M,J (M:Black level=7.5 IRE, J:Black level=0 IRE)
 *				01: PAL-M (BL=7.5 IRE)(Mono, 480I/60Hz)
 *				10: PAL-B,D,G,H,I,N (N:BL=7.5 IRE, others:0 IRE)
 *				11: PAL-Nc (BL=0 IRE)(No color, 576I/50Hz)
 */
#define TVE_FORMAT_REG_TVF_MASK			0x00000007
#define TVE_NTSC_M						0x00000000
#define TVE_NTSC_J						0x00000004
#define TVE_PAL_M						0x00000001
#define TVE_PAL_N						0x00000002
#define TVE_PAL_B_D_G_H_I				0x00000006
#define TVE_PAL_NC						0x00000007

/*		
 *			[2] BLACK0IRE:		Black level select
 *				0: BL is 7.5 IRE above blanking level.
 *				1: BL is 0 IRE (Black level is the same as blanking level)
 */
#define TVE_FORMAT_REG_BLACK0IRE_MASK	0x00000004
#define BLACK0IRE_7_5_IRE				0x00000000
#define BLACK0IRE_0_IRE					0x00000004

/*		
 *			[3] VIN_RANGE:		The range of the video pixel input range.
 *				0: 16~235 for Y, 16~240 for Cb and Cr
 *				1: 1~254 for Y, Cb and Cr
 */
#define TVE_FORMAT_REG_VIN_RANGE_MASK	0x00000008
#define VIN_RANGE_16_235				0x00000000
#define VIN_RANGE_1_254					0x00000008

/*
 *	Base+0x04	TVE_INPUT_REG:
 *					TV encoder input control register.
 *					Controlling the input format of signal.
 *					(BT601, BT656, Color Bar etc...)
 */
#define TVE_INPUT_REG_OFFSET			0x00000004		

/*		
 *			[1-2] VINF:			Video input mode selection.
 *				00: BT601 slave mode
 *				01: BT656 mode
 *				10: Reserved
 *				11: internal 100% color bar pattern (Field-mode should be = 0)
 */
#define TVE_INPUT_REG_VINF_MASK			0x00000006
#define VINF_BT601						0x00000000
#define VINF_BT656						0x00000002
#define VINF_COLOR_BAR					0x00000006

/*
 *	Base+0x08	TVE_OUTPUT_REG:
 *					TV encoder output control register.
 *					Controlling the output format of signal.
 *					(CVBS and S-video, component YUV etc...)
 */
#define TVE_OUTPUT_REG_OFFSET			0x00000008

/*		
 *			[3] NO_C:			Disable color signal
 *				0: Normal video output
 *				1: Color signal is disable on CVBS output. S-video or component
 *				   output is normal.
 */
#define TVE_OUTPUT_REG_NO_C_MASK		0x00000008
#define NO_C_NORMAL						0x00000000
#define NO_C_DISABLE					0x00000008

/*		
 *			[4] BLACK:			Enable black output
 *				0: Normal video output
 *				1: Black output in all output modes
 */
#define TVE_OUTPUT_REG_BLACK_MASK		0x00000010
#define BLACK_NORMAL					0x00000000
#define BLACK_OUTPUT					0x00000010

/*		
 *			[5] BLUE:			Enable blue output.	It is used to display blue
 *								screen while there is no video input signal.
 *				0: Normal video output
 *				1: Blue output in all output modes
 */
#define TVE_OUTPUT_REG_BLUE_MASK		0x00000020
#define BLUE_NORMAL						0x00000000
#define BLUE_OUTPUT						0x00000020

/*		
 *			[6] VOF:			Output format
 *				0: CVBS and S-video output simultaneously
 *				1: Component YUV output
 */
#define TVE_OUTPUT_REG_VOF_MASK			0x00000040
#define VOF_CVBS_SVIDEO					0x00000000
#define VOF_COMPONENT_YUV				0x00000040

/*
 *	Base+0x0C	TVE_PWDN_REG:
 *					Power down control register.
 */
#define TVE_PWDN_REG_OFFSET				0x0000000C

/*		
 *			[0] SRESET:			Software reset. It will reset all registers 
 *								to default reset values.
 *				0: No reset
 *				1: Software reset. Writing '1' to this bit enables the software
 *				   reset function and this bit will automatically reset to '0'. 
 *				   It is not necessary to write '0' to disable "sreset".
 */
#define TVE_PWDN_REG_DAC_MASK			0x00000007 
#define TVE_PWDN_REG_CVBS_DAC			0x00000004
#define TVE_PWDN_REG_Y_DAC				0x00000002
#define TVE_PWDN_REG_C_DAC				0x00000001

/*
 *	Base+0x10	TVE_SWRST_REG:
 *					Software reset. It will reset all registers to default 
 *					reset values.
 */
#define TVE_SWRST_REG_OFFSET			0x00000010

/*		
 *			[0] SRESET:			Software reset. It will reset all registers 
 *								to default reset values.
 *				0: No reset
 *				1: Software reset. Writing '1' to this bit enables the software
 *				   reset function and this bit will automatically reset to '0'. 
 *				   It is not necessary to write '0' to disable "sreset".
 */
#define TVE_SWRST_REG_SRESET_MASK		0x00000001
#define SRESET_DISABLE					0x00000000
#define SRESET_ENABLE					0x00000001

/*----------------------------------------------------------------------------*/

/*	TV Encoder Control Registers  */

typedef volatile struct	 {
	volatile unsigned long fmt_ctrl;
	volatile unsigned long input_ctrl;
	volatile unsigned long output_ctrl;
	volatile unsigned long power;
	volatile unsigned long reset;
} sky_tv;

/*******************************************************************************
					SK2851 Defines
*******************************************************************************/
/*----------------------------------------------------------------------------*/
//== New structure for FB.
struct known_displays{
	unsigned long xres;						// x resolution
	unsigned long yres;						// y resolution
	unsigned long bpp;						// bits per pixel
	unsigned long ctrl_reg;					// control register
	unsigned long ctrl_flag;					// control flag
	unsigned long pll_clk;					// display clock
	unsigned long pol;						// polarity 
	unsigned long ht1;						// horizontal timing ctrl 1
	unsigned long ht2;						// horizontal timing ctrl 2
	unsigned long vt1;						// vertical timing ctrl 1
	unsigned long vt2;						// vertical timing ctrl 2
	char display_name[64];				// name of display mode
};


/*----------------------------------------------------------------------------*/
//== Macro define.
#define ENCODE_RES(x, y)			(((x & 0xFFF) << 16) | (y & 0xFFF))
#define BITS_MASK(x, y)				(x << y)

/*----------------------------------------------------------------------------*/
//== Disp regs form 0xC0FC1300 ~ 0xC0FC13FC
typedef volatile struct	 {
	volatile unsigned long disp_ctrl;
	volatile unsigned long y_addr1;
	volatile unsigned long cb_addr1;
	volatile unsigned long cr_addr1;
	volatile unsigned long y_addr2;
	volatile unsigned long cb_addr2;
	volatile unsigned long cr_addr2;
	volatile unsigned long y_addr3;
	volatile unsigned long cb_addr3;
	volatile unsigned long cr_addr3;
	volatile unsigned long y_addr4;
	volatile unsigned long cb_addr4;
	volatile unsigned long cr_addr4;

	volatile unsigned long src_res;
	volatile unsigned long disp_res;
	volatile unsigned long line_stride;
	volatile unsigned long black_tb;
	volatile unsigned long black_lr;
	
	volatile unsigned long fifo_th;
	volatile unsigned long osd_addr;
	volatile unsigned long osd1_start;
	volatile unsigned long osd1_end;

	volatile unsigned long osd_red;
	volatile unsigned long osd_green1;
	volatile unsigned long osd_green2;
	volatile unsigned long osd_blue;

	volatile unsigned long sharpness;
	volatile unsigned long scalar;
	volatile unsigned long img_enhance;
	
	volatile unsigned long gamma_red1;
	volatile unsigned long gamma_red2;
	volatile unsigned long gamma_red3;
	volatile unsigned long gamma_red4;

	volatile unsigned long ccir_coef_y;
	volatile unsigned long ccir_coef_cb;
	volatile unsigned long ccir_coef_cr;

	volatile unsigned long tcon_sth_ld_time;
	volatile unsigned long tcon_ckv_stv_blank;
	volatile unsigned long tcon_pol_oev_blank;
	volatile unsigned long tcon_sth_ld_ckv_sig;
	volatile unsigned long tcon_stv_pol_time;
	volatile unsigned long tcon_oev_ap_sig;

	volatile unsigned long osd_size;
	volatile unsigned long osd2_start;
	volatile unsigned long osd2_end;

	volatile unsigned long dpwm_period_ctrl;
	volatile unsigned long dpwm_gap;
	volatile unsigned long dpwm_burst1;
	volatile unsigned long dpwm_burst2;
	volatile unsigned long dpwm_dim1;
	volatile unsigned long dpwm_dim2;
	volatile unsigned long dpwm0_duty;
	volatile unsigned long dpwm1_duty;
	volatile unsigned long dpwm2_duty;
	volatile unsigned long dpwm3_duty;

	volatile unsigned long int_src_mask;
	volatile unsigned long int_sts_clr;

	volatile unsigned long osd3_start;
	volatile unsigned long osd3_end;
	volatile unsigned long osd4_start;
	volatile unsigned long osd4_end;
	volatile unsigned long osd5_start;
	volatile unsigned long osd5_end;
} sk8851_disp1;

//== Disp reg form 0xC0FC1900 ~ 0xC0FC19xx
typedef volatile struct	 {
	volatile unsigned long lcd_pol;
	volatile unsigned long lcd_htime1;
	volatile unsigned long lcd_htime2;
	volatile unsigned long lcd_vtime1;
	volatile unsigned long lcd_vtime2;
	
	volatile unsigned long gamma_green1;
	volatile unsigned long gamma_green2;
	volatile unsigned long gamma_green3;
	volatile unsigned long gamma_green4;
	volatile unsigned long gamma_blue1;
	volatile unsigned long gamma_blue2;
	volatile unsigned long gamma_blue3;
	volatile unsigned long gamma_blue4;
} sk8851_disp2;


//-- Bit defines of control register. (0xC0FC1300)
// DISP_EN
#define DISP_EN_SHIFT				31
#define DISP_EN_MASK				(0x01 << DISP_EN_SHIFT)
#define DISP_DIS					(0x00 << DISP_EN_SHIFT)
#define DISP_EN						(0x01 << DISP_EN_SHIFT)
// OL_BG
#define OL_BG_SHIFT					29
#define OL_BG_MASK					(0x03 << OL_BG_SHIFT)
#define OL_BG_DISABLE				(0x00 << OL_BG_SHIFT)
#define BG_EN_ONLY					(0x01 << OL_BG_SHIFT)
#define OL_EN_ONLY					(0x02 << OL_BG_SHIFT)
#define OL_BG_EN					(0x03 << OL_BG_SHIFT)
// PAT_GEN
#define PAT_GEN_SHIFT				28
#define PAT_GEN_MASK				(0x01 << PAT_GEN_SHIFT)
#define PAT_GEN_DIS					(0x00 << PAT_GEN_SHIFT)
#define PAT_GEN_EN					(0x01 << PAT_GEN_SHIFT)
// SCALAR_SHARPNESS
#define SCALAR_SHARPNESS_SHIFT		26
#define SCALAR_SHARPNESS_MASK		(0x03 << SCALAR_SHARPNESS_SHIFT)
#define SCALAR_SHARPNESS_DISABLE	(0x00 << SCALAR_SHARPNESS_SHIFT)
#define EDGE_EN_ONLY				(0x01 << SCALAR_SHARPNESS_SHIFT)
#define SHARPNESS_EN_ONLY			(0x02 << SCALAR_SHARPNESS_SHIFT)
#define SCALAR_SHARPNESS_EN			(0x03 << SCALAR_SHARPNESS_SHIFT)
// IMG_ENHANCE
#define IMG_ENHANCE_SHIFT			25
#define IMG_ENHANCE_MASK			(0x01 << IMG_ENHANCE_SHIFT)
#define IMG_ENHANCE_DIS				(0x00 << IMG_ENHANCE_SHIFT)
#define IMG_ENHANCE_EN				(0x01 << IMG_ENHANCE_SHIFT)
// LCD_TCON
#define LCD_TCON_SHIFT				24
#define LCD_TCON_MASK				(0x01 << LCD_TCON_SHIFT)
#define LCD_TCON_DIS				(0x00 << LCD_TCON_SHIFT)
#define LCD_TCON_EN					(0x01 << LCD_TCON_SHIFT)
// INTERLACE_MODE
#define INTERLACE_MODE_SHIFT		23
#define INTERLACE_MODE_MASK			(0x01 << INTERLACE_MODE_SHIFT)
#define INTERLACE_MODE_DIS			(0x00 << INTERLACE_MODE_SHIFT)
#define INTERLACE_MODE_EN			(0x01 << INTERLACE_MODE_SHIFT)
// DOUBLE_READ
#define DOUBLE_READ_SHIFT			22
#define DOUBLE_READ_MASK			(0x01 << DOUBLE_READ_SHIFT)
#define DOUBLE_READ_DIS				(0x00 << DOUBLE_READ_SHIFT)
#define DOUBLE_READ_EN				(0x01 << DOUBLE_READ_SHIFT)
// INV_CLK
#define INV_CLK_SHIFT				21
#define INV_CLK_MASK				(0x01 << INV_CLK_SHIFT)
#define INV_CLK_DIS					(0x00 << INV_CLK_SHIFT)
#define INV_CLK_EN					(0x01 << INV_CLK_SHIFT)
// ADDR_MODE
#define ADDR_MODE_SHIFT				20
#define ADDR_MODE_MASK				(0x01 << ADDR_MODE_SHIFT)
#define ADDR_MODE_FIX				(0x00 << ADDR_MODE_SHIFT)
#define ADDR_MODE_SEQ				(0x01 << ADDR_MODE_SHIFT)
// SYNC_MODE
#define SYNC_MODE_SHIFT				19
#define SYNC_MODE_MASK				(0x01 << SYNC_MODE_SHIFT)
#define SYNC_MODE_Y					(0x00 << SYNC_MODE_SHIFT)
#define SYNC_MODE_ALL				(0x01 << SYNC_MODE_SHIFT)
// OSD_SEL
#define OSD_SEL_SHIFT				12
#define OSD_SEL_MASK				(0x1F << OSD_SEL_SHIFT)
#define SEL_OSD_NONE				(0x00 << OSD_SEL_SHIFT)
#define SEL_OSD_1					(0x01 << OSD_SEL_SHIFT)
#define SEL_OSD_2					(0x02 << OSD_SEL_SHIFT)
#define SEL_OSD_3					(0x04 << OSD_SEL_SHIFT)
#define SEL_OSD_4					(0x08 << OSD_SEL_SHIFT)
#define SEL_OSD_5					(0x10 << OSD_SEL_SHIFT)
#define SEL_OSD_ALL					(0x1F << OSD_SEL_SHIFT)
// VDAC_SWITCH
#define VDAC_SWITCH_SHIFT			10
#define VDAC_SWITCH_MASK			(0x03 << VDAC_SWITCH_SHIFT)
#define VDAC_SWITCH_YPBPR			(0x00 << VDAC_SWITCH_SHIFT)
#define VDAC_SWITCH_CVBS			(0x01 << VDAC_SWITCH_SHIFT)
#define VDAC_SWITCH_RGB888			(0x02 << VDAC_SWITCH_SHIFT)
#define VDAC_SWITCH_RESERVE			(0x03 << VDAC_SWITCH_SHIFT)
// INPUT_FORMAT
#define INPUT_FORMAT_SHIFT			8
#define INPUT_FORMAT_MASK			(0x03 << INPUT_FORMAT_SHIFT)
#define INPUT_FORMAT_422			(0x00 << INPUT_FORMAT_SHIFT)
#define INPUT_FORMAT_420			(0x01 << INPUT_FORMAT_SHIFT)
#define INPUT_FORMAT_RGB888			(0x02 << INPUT_FORMAT_SHIFT)
// ACTIVE_MODE
#define ACTIVE_MODE_SHIFT			4
#define ACTIVE_MODE_MASK			(0x0F << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_480I			(0x00 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_W480I			(0x01 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_576I			(0x02 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_W576I			(0x03 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_1080I30FPS		(0x04 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_1080I25FPS		(0x05 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_480P			(0x08 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_W480P			(0x09 << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_576P			(0x0A << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_W576P			(0x0B << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_1080P			(0x0C << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_720P			(0x0D << ACTIVE_MODE_SHIFT)
#define ACTIVE_MODE_LCD				(0x0F << ACTIVE_MODE_SHIFT)
// PATGEN_RANGE
#define PATGEN_RANGE_SHIFT			3
#define PATGEN_RANGE_MASK			(0x01 << PATGEN_RANGE_SHIFT)
#define PATGEN_RANGE_100			(0x00 << PATGEN_RANGE_SHIFT)
#define PATGEN_RANGE_75				(0x01 << PATGEN_RANGE_SHIFT)
// PATGEN_MODE
#define PATGEN_MODE_SHIFT			0
#define PATGEN_MODE_MASK			(0x07 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_VERTICAL		(0x00 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_HORIZONTAL		(0x01 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_GRAY			(0x02 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_BLACK			(0x03 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_WHITE			(0x04 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_RED				(0x05 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_GREEN			(0x06 << PATGEN_RANGE_SHIFT)
#define PATGEN_MODE_BLUE			(0x07 << PATGEN_RANGE_SHIFT)


//-- Polarity and Htiming control register. (0xC0FC1900)
// CLK_POL
#define CLK_POL_SHIFT				27
#define CLK_POL_MASK				(0x01 << CLK_POL_SHIFT)
#define CLK_POL_NEGATIVE			(0x00 << CLK_POL_SHIFT)
#define CLK_POL_POSITIVE			(0x01 << CLK_POL_SHIFT)
// VS_POL
#define VS_POL_SHIFT				26
#define VS_POL_MASK					(0x01 << VS_POL_SHIFT)
#define VS_POL_ACTIVE_LO			(0x00 << VS_POL_SHIFT)
#define VS_POL_ACTIVE_HI			(0x01 << VS_POL_SHIFT)
// HS_POL
#define HS_POL_SHIFT				25
#define HS_POL_MASK					(0x01 << HS_POL_SHIFT)
#define HS_POL_ACTIVE_LO			(0x00 << HS_POL_SHIFT)
#define HS_POL_ACTIVE_HI			(0x01 << HS_POL_SHIFT)
// DE_POL
#define DE_POL_SHIFT				24
#define DE_POL_MASK					(0x01 << DE_POL_SHIFT)
#define DE_POL_ACTIVE_LO			(0x00 << DE_POL_SHIFT)
#define DE_POL_ACTIVE_HI			(0x01 << DE_POL_SHIFT)
// HS_PULSE
#define HS_PULSE_SHIFT				16
#define HS_PULSE_MASK				(0xFF << HS_PULSE_SHIFT)
// HS_BPORCH1
#define HS_BPORCH1_SHIFT			8
#define HS_BPORCH1_MASK				(0xFF << HS_BPORCH1_SHIFT)
// HS_BPORCH0
#define HS_BPORCH0_SHIFT			0
#define HS_BPORCH0_MASK				(0xFF << HS_BPORCH0_SHIFT)

//-- Htimming control register 1 (0xC0FC1904)
// HS_WIDTH1
#define HS_WIDTH1_SHIFT				16
#define HS_WIDTH1_MASK				(0x7FF << HS_WIDTH1_SHIFT)
// HS_WIDTH2
#define HS_WIDTH2_SHIFT				0
#define HS_WIDTH2_MASK				(0x7FF << HS_WIDTH2_SHIFT)

//-- Htimming control register 2 (0xC0FC1908)
// HS_WIDTH
#define HS_WIDTH_SHIFT				16
#define HS_WIDTH_MASK				(0x7FF << HS_WIDTH_SHIFT)
// HS_FPORCH
#define HS_FPORCH_SHIFT				0
#define HS_FPORCH_MASK				(0xFF << HS_FPORCH_SHIFT)

//-- Vtimming control register 1 (0xC0FC190C)
// VS_PULSE
#define VS_PULSE_SHIFT				28
#define VS_PULSE_MAKS				(0x07 << VS_PULSE_SHIFT)
// VS_BPORCH1
#define VS_BPORCH1_SHIFT			24
#define VS_BPORCH1_MAKS				(0x07 << VS_BPORCH1_SHIFT)
// VS_BPORCH0
#define VS_BPORCH0_SHIFT			16
#define VS_BPORCH0_MAKS				(0xFF << VS_BPORCH0_SHIFT)
// VS_HEIGHT
#define VS_HEIGHT_SHIFT				0
#define VS_HEIGHT_MAKS				(0x7FF << VS_HEIGHT_SHIFT)

//-- Vtimming control register 2 (0xC0FC1910)
// COEF_R
#define COEF_R_SHIFT				28
#define COEF_R_MAKS					(0x0F << COEF_R_SHIFT)
// COEF_G
#define COEF_G_SHIFT				24
#define COEF_G_MAKS					(0x0F << COEF_G_SHIFT)
// COEF_B
#define COEF_B_SHIFT				20
#define COEF_B_MAKS					(0x0F << COEF_B_SHIFT)
// VS_FPORCH
#define VS_FPORCH_SHIFT				0
#define VS_FPORCH_MAKS				(0xFF << VS_FPORCH_SHIFT)

/*----------------------------------------------------------------------------*/
enum DISP_MODE{
	//-- Interlace
	DISP_480I,
	DISP_W480I,
	DISP_576I,
	DISP_W576I,
	DISP_1080I_30FPS,
	DISP_1080I_25FPS,

	//-- Progress
	DISP_480P,
	DISP_W480P,
	DISP_576P,
	DISP_W576P,
	DISP_1080P,
	DISP_720P,

	//-- LCD
	DISP_640x480_LCD,
	DISP_800x480_LCD,
	DISP_800x600_LCD,
	DISP_1024x600_LCD,
	DISP_1366x768_LCD,
};


//-- Bit defines of control flag
#define CONTROL_FLAG_NONE			0x00000000
#define YPBPR_PLL_DIV2				0x00000001	// bit0

/*----------------------------------------------------------------------------*/
//-- 480i(13.5MHz)
#define D480I_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_EN | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_480I)			

#define D480I_CTRL_FLAG				(YPBPR_PLL_DIV2)
#define D480I_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- w480i(18MHz)
#define DW480I_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_EN | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_W480I)			

#define DW480I_CTRL_FLAG			(YPBPR_PLL_DIV2)
#define DW480I_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- 480p(27MHz)
#define D480P_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_480P)			

#define D480P_CTRL_FLAG				0x00000000
#define D480P_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- w480p(36MHz)
#define DW480P_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_W480P)			

#define DW480P_CTRL_FLAG			0x00000000
#define DW480P_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- 576i(13.5MHz)
#define D576I_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_EN | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_576I)			

#define D576I_CTRL_FLAG				(YPBPR_PLL_DIV2)
#define D576I_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- w576i(18MHz)
#define DW576I_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_EN | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_W576I)			

#define DW576I_CTRL_FLAG			(YPBPR_PLL_DIV2)
#define DW576I_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- 576p(27MHz)
#define D576P_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_576P)			

#define D576P_CTRL_FLAG				0x00000000
#define D576P_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- w576p(36MHz)
#define DW576P_CTRL_REG				(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_YPBPR | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_W576P)			

#define DW576P_CTRL_FLAG			0x00000000
#define DW576P_PLL_CLK				0x00000000

/*----------------------------------------------------------------------------*/
//-- D-sub640x480(60Hz)
#define DSUB640x480_60HZ_CTRL_REG	(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_RGB888 | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_LCD)			

#define DSUB640x480_60HZ_CTRL_FLAG	0x00000000
#define DSUB640x480_60HZ_PLL_CLK	0x00000000

/*----------------------------------------------------------------------------*/
//-- D-sub800x600(56Hz)
#define DSUB800x600_56HZ_CTRL_REG	(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_RGB888 | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_LCD)			

#define DSUB800x600_56HZ_CTRL_FLAG	0x00000000
#define DSUB800x600_56HZ_PLL_CLK	0x00000000


/*----------------------------------------------------------------------------*/
//-- LCD 800x600(No T-CON)
#define DLCD800x600_CTRL_REG		(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_DIS |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_RESERVE | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_LCD)			

#define DLCD800x600_CTRL_FLAG		0x00000000
#define DLCD800x600_PLL_CLK			0x00000000


/*----------------------------------------------------------------------------*/
//-- LCD 800x600(T-CON)
#define DLCD800x600_TCON_CTRL_REG	(DISP_DIS |					\
									BG_EN_ONLY | 				\
									PAT_GEN_DIS |				\
									SCALAR_SHARPNESS_DISABLE |	\
									IMG_ENHANCE_DIS |			\
									LCD_TCON_EN |				\
									INTERLACE_MODE_DIS | 		\
									DOUBLE_READ_DIS |			\
									INV_CLK_DIS |				\
									ADDR_MODE_FIX |				\
									SYNC_MODE_Y |				\
									SEL_OSD_NONE |				\
									VDAC_SWITCH_RESERVE | 		\
									INPUT_FORMAT_420 | 			\
									ACTIVE_MODE_LCD)			

#define DLCD800x600_TCON_CTRL_FLAG	0x00000000
#define DLCD800x600_TCON_PLL_CLK	0x00000000


/*----------------------------------------------------------------------------*/
#define BIT(x)						(1 << (x))
#define CHECK_BIT(val, mask)		((val & mask) == mask)
#define SET_BIT(val, mask)			(val |= mask)
#define CLEAR_BIT(val, mask)		(val &= (~mask))

//-- 0xC0fC13DC  Interrupt source and mask control register.
#define MEM_ADDR_SET_SHIFT			30
#define MEM_ADDR_SET_MASK			(0x03 << MEM_ADDR_SET_SHIFT)
#define FRAME_BODY_LOCK_SHIFT		29
#define FRAME_BODY_LOCK_MASK		(0x01 << FRAME_BODY_LOCK_SHIFT)
#define FRAME_ADV_SYNC_SHIFT		19
#define FRAME_ADV_SYNC_MASK			(0x01 << FRAME_ADV_SYNC_SHIFT)
#define FRAME_BODY_UPDATE_SHIFT		16
#define FRAME_BODY_UPDATE_MASK		(0x01 << FRAME_BODY_UPDATE_SHIFT)
#define FRAME_ADV_SYNC_INT_SHIFT	3
#define FRAME_ADV_SYNC_INT_MASK		(0x01 << FRAME_ADV_SYNC_INT_SHIFT)
#define FRAME_ADV_SYNC_INT_ENABLE	(0x00 << FRAME_ADV_SYNC_INT_SHIFT)
#define FRAME_ADV_SYNC_INT_DISABLE	(0x01 << FRAME_ADV_SYNC_INT_SHIFT)
#define FRAME_BODY_UPDATE_INT_SHIFT	0
#define FRAME_BODY_UPDATE_INT_MASK	(0x01 << FRAME_BODY_UPDATE_INT_SHIFT)
#define FRAME_BODY_UPDATE_INT_ENABLE	(0x00 << FRAME_ADV_SYNC_INT_SHIFT)
#define FRAME_BODY_UPDATE_INT_DISABLE	(0x01 << FRAME_ADV_SYNC_INT_SHIFT)

//-- 0xC0fC13E0  Interrupt status and clear control register.
#define FRAME_ADV_SYNC_STS_SHIFT	19
#define FRAME_ADV_SYNC_STS_MASK		(0x01 << FRAME_ADV_SYNC_STS_SHIFT)
#define FRAME_BODY_UPDATE_STS_SHIFT	16
#define FRAME_BODY_UPDATE_STS_MASK	(0x01 << FRAME_BODY_UPDATE_STS_SHIFT)
#define FRAME_ADV_SYNC_CLR_SHIFT	3
#define FRAME_ADV_SYNC_CLR_MASK		(0x01 << FRAME_ADV_SYNC_CLR_SHIFT)
#define FRAME_BODY_UPDATE_CLR_SHIFT	0
#define FRAME_BODY_UPDATE_CLR_MASK	(0x01 << FRAME_BODY_UPDATE_CLR_SHIFT)

/*----------------------------------------------------------------------------*/
//-- OSD
#define OSD_ON						0x80
#define OSD_OFF						0x00
#define OSD_ON_MASK					0x80
#define OSD_COLOR_MASK				0x7F

#define OSD_BLOCK_W					30
#define OSD_BLOCK_H					120
#define OSD_BLOCK_SIZE				(OSD_BLOCK_W * OSD_BLOCK_H)

#define OSD_ALPHA_SHIFT				28
#define OSD_ALPHA_MASK				(0x0F << OSD_ALPHA_SHIFT)
#define OSD_START_X_SHIFT			16
#define OSD_START_X_MASK			(0x7FF << OSD_START_X_SHIFT)
#define OSD_START_Y_SHIFT			0
#define OSD_START_Y_MASK			(0x7FF << OSD_START_X_SHIFT)
#define OSD_END_X_SHIFT				16
#define OSD_END_X_MASK				(0x7FF << OSD_START_X_SHIFT)
#define OSD_END_Y_SHIFT				0
#define OSD_END_Y_MASK				(0x7FF << OSD_START_X_SHIFT)

#define OSD1_START					((0x01 << OSD_ALPHA_SHIFT) | (0x10 ))

/*----------------------------------------------------------------------------*/
//-- DPWM
#define DPWM_LAMP_PERIOD_SHIFT		16
#define DPWM_LAMP_PERIOD_MASK		(0x3FFF << DPWM_LAMP_PERIOD_SHIFT)

#define DPWM_BURST_ON_SHIFT			4
#define DPWM_BURST_ON_MASK			(0x0F << DPWM_BURST_ON_SHIFT)
#define DPWM0_BURST_ON				(0x01 << DPWM_BURST_ON_SHIFT)
#define DPWM1_BURST_ON				(0x02 << DPWM_BURST_ON_SHIFT)
#define DPWM2_BURST_ON				(0x04 << DPWM_BURST_ON_SHIFT)
#define DPWM3_BURST_ON				(0x08 << DPWM_BURST_ON_SHIFT)
#define DPWM_ALL_BURST_ON			(0x0F << DPWM_BURST_ON_SHIFT)

#define DPWM_MODE_SHIFT				0
#define DPWM_MODE_MASK				(0x0F << DPWM_MODE_SHIFT)
#define DPWM0_ENABLE				(0x01 << DPWM_MODE_SHIFT)
#define DPWM1_ENABLE				(0x02 << DPWM_MODE_SHIFT)
#define DPWM2_ENABLE				(0x04 << DPWM_MODE_SHIFT)
#define DPWM3_ENABLE				(0x08 << DPWM_MODE_SHIFT)
#define DPWM_ALL_ENABLE				(0x0F << DPWM_MODE_SHIFT)

#define DPWM_0_2_BURST_PERIOD_SHIFT 0
#define DPWM_0_2_BURST_PERIOD_MASK 	(0x3FF << DPWM_0_2_BURST_PERIOD_SHIFT)
#define DPWM_1_3_BURST_PERIOD_SHIFT 16
#define DPWM_1_3_BURST_PERIOD_MASK 	(0x3FF << DPWM_1_3_BURST_PERIOD_SHIFT)

#define DPWM_0_2_DIM_PERIOD_SHIFT 	0
#define DPWM_0_2_DIM_PERIOD_MASK 	(0x3FF << DPWM_0_2_DIM_PERIOD_SHIFT)
#define DPWM_1_3_DIM_PERIOD_SHIFT 	16
#define DPWM_1_3_DIM_PERIOD_MASK 	(0x3FF << DPWM_1_3_DIM_PERIOD_SHIFT)

#define DPWM_01_02_DC_SHIFT			0
#define DPWM_01_02_DC_MASK 			(0x0F << DPWM_01_02_DC_SHIFT)	
#define DPWM_03_04_DC_SHIFT			4
#define DPWM_03_04_DC_MASK 			(0x0F << DPWM_03_04_DC_SHIFT)	
#define DPWM_05_06_DC_SHIFT			8
#define DPWM_05_06_DC_MASK 			(0x0F << DPWM_05_06_DC_SHIFT)	
#define DPWM_07_08_DC_SHIFT			12
#define DPWM_07_08_DC_MASK 			(0x0F << DPWM_07_08_DC_SHIFT)	
#define DPWM_09_10_DC_SHIFT			16
#define DPWM_09_10_DC_MASK 			(0x0F << DPWM_09_10_DC_SHIFT)	
#define DPWM_11_12_DC_SHIFT			20
#define DPWM_11_12_DC_MASK 			(0x0F << DPWM_11_12_DC_SHIFT)	
#define DPWM_13_14_DC_SHIFT			24
#define DPWM_13_14_DC_MASK 			(0x0F << DPWM_13_14_DC_SHIFT)	
#define DPWM_15_16_DC_SHIFT			28
#define DPWM_15_16_DC_MASK 			(0x0F << DPWM_15_16_DC_SHIFT)	

enum{
	SKY_DPWM_0,
	SKY_DPWM_1,
	SKY_DPWM_2,
	SKY_DPWM_3,
	SKY_DPWM_ALL,
};

#endif	/* _ASM_ARC_PGU_H */




#else	//SK2850

/*
 * arc_pgu.h - Register definitions for the ARC pixel graphics unit.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * THIS	 SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 * WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.	 IN
 * NO EVENT  SHALL   THE AUTHOR	 BE  LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright 2005 (C) ARC International (www.arc.com). 
 * 
 */

#ifndef _ASM_ARC_PGU_H
#define _ASM_ARC_PGU_H

#include <linux/fb.h>

/*--------------------------------------------------------------------------------------------*/
#if 1	//Barry 2007-08-20	AVI with subtitle
/*
	Display index.
*/
enum DisplayIndex 
{
	DISP_NTSC = 0,				
	DISP_PAL,				
	DISP_HDTV,				
	DISP_INNOLUX_1024600_TFT,
	DISP_AU_800480_TFT,
	DISP_INNOLUX_800600_TFT,	
	DISP_HIMAX_480240_TFT,	
	DISP_HIMAX_NTSC, 			
	DISP_HIMAX_PAL, 					
	DISP_MAX,
};
#endif

/* Ioctl selectors	      FIXME - make sure these are unique within the fb subsystem */
#define ARCPGUFB_GETYUV		0xffff0001
#define ARCPGUFB_SETYUV		0xffff0002
#define ARCPGUFB_FILLYUV	0xffff0003
#define ARCPGUFB_SWITCH_YUV	0xffff0004
#define ARCPGUFB_BKLGHT_OFF	0xffff0005
#define ARCPGUFB_BKLGHT_ON	0xffff0006
#define ARCPGUFB_START_DISPLAY	0xffff0007
#define ARCPGUFB_STOP_DISPLAY	0xffff0008
#define ARCPGUFB_GET_PGU_STATE	0xffff0009
#define ARCPGUFB_SET_PGU_STATE	0xffff000a
#define ARCPGUFB_GET_MAIN_MODE	0xffff000b
#define ARCPGUFB_SET_MAIN_MODE	0xffff000c
#define ARCPGUFB_SET_RGB_BUF	0xffff000d
#define ARCPGUFB_GET_RGB_BUF	0xffff000e
#define ARCPGUFB_GET_FB_MAIN	0xffff000f
#define ARCPGUFB_CHECKPOINT	0xffff0010
#define ARCPGUFB_RESTORE	0xffff0011
#define ARCPGUFB_RESET		0xffff0012
#define ARCPGUFB_SET_YUV_RES	0xffff0013
#define ARCPGUFB_GET_FB_INFO	0xffff0014
#define ARCPGUFB_SET_OL_MODE	0xffff0015
#define ARCPGUFB_GET_OL_MODE	0xffff0016
#define ARCPGUFB_GET_OL_COLKEY	0xffff0017
#define ARCPGUFB_SET_OL_COLKEY	0xffff0018
#define ARCPGUFB_GET_OL_START	0xffff0019
#define ARCPGUFB_SET_OL_START	0xffff001a
#define ARCPGUFB_GET_OL_END	0xffff001b
#define ARCPGUFB_SET_OL_END	0xffff001c
#define ARCPGUFB_SET_OL_OFFSET	0xffff001d
#define ARCPGUFB_GET_YUV_BASE	0xffff002c /* return address of yuv buffer n */

#ifdef CONFIG_ENABLE_CENTERING
#define ARCPGUFB_GET_ORIGIN	0xffff001e
#define ARCPGUFB_SET_ORIGIN	0xffff001f
#endif

#define ARCPGUFB_ALLOC		0xffff0020
#define ARCPGUFB_FREE		0xffff0021
#define ARCPGUFB_SEND		0xffff0022
#define ARCPGUFB_FRAMERATE_SET	0xffff0023
#define ARCPGUFB_QUEUE_LEN_GET	0xffff0024
#define ARCPGUFB_FLUSH		0xffff0025
#define ARCPGUFB_FRAME_DROP	0xffff0026
#define ARCPGUFB_PAUSE		0xffff0027
#define ARCPGUFB_RESUME		0xffff0028
#define ARCPGUFB_RESET_SYNC	0xffff0029
#define ARCPGUFB_NUM_FREE_BUFS	0xffff002a
#define ARCPGUFB_PAUSE_MS	0xffff002b
//Barry 2008-03-06
//#define ARCPGUFB_GET_AVAIL_BUFS	0xffff002d
#define ARCPGUFB_SET_VIDEO_YUV_RES		0xffff002d	// set yuv buffer res. in video mode.

#define ARCPGUFB_GET_IRVALUE	0xffff0030
#define ARCPGUFB_SET_IRVALUE	0xffff0031

//Barry 2007-08-20
#define ARCPGUFB_GET_DISP_INDEX 		0xffff0032	// get the display mode (NTSC, PAL, LCD)
#define ARCPGUFB_SET_DISP_INDEX 		0xffff0033	// set the display mode (NTSC, PAL, LCD)		//Barry 2007-11-27
#define ARCPGUFB_SHOW_MPLAYER_SUBTITLE	0xffff00e2
#define ARCPGUFB_SHOW_VIDEO_SPEED		0xffff00e4	//Barry 2007-12-24
#define ARCPGUFB_CLEAR_MPLAYER_SUBTITLE	0xffff00ea
#define ARCPGUFB_CLEAR_VIDEO_SPEED		0xffff00ec	//Barry 2007-12-24
#define ARCPGUFB_GET_VIDEO_TIME_ONOFF 	0xffff00ed	//Barry 2008-01-07
#define ARCPGUFB_ENABLE_TV_OSD			0xffff00f7
#define ARCPGUFB_DISABLE_TV_OSD			0xffff00f8

#define ARCPGUFB_GET_MPLAYER_PLAY_TIME	0xffff0036	// get video play time 
#define ARCPGUFB_SET_MPLAYER_PLAY_TIME	0xffff0037	// set video play time
#define ARCPGUFB_GET_MPLAYER_DURATION   0xffff0038      // get video play time
#define ARCPGUFB_SET_MPLAYER_DURATION   0xffff0039      // set video play time
// Raymond 2008/05/20
#define ARCPGUFB_GET_DEFAULT_DISP_INDEX	0xffff003a	// get display index of startup stage
#define ARCPGUFB_GET_MPLAYER_STATUS		0xffff003b	// get mplayer status
#define ARCPGUFB_SET_MPLAYER_STATUS		0xffff003c	// set mplayer status

//== Flowbee 2007-11-23 Advertisment
#define ARCPGUFB_MPLAYER_ADVERTISMENT	0xffff0060

#define ARCPGUFB_GET_STATCTRL	0xffff00fe
#define ARCPGUFB_SET_STATCTRL	0xffff00ff

// Raymond 2004/04/25
#define ARCPGUFB_ENABLE_OL			0xffff00fa
#define ARCPGUFB_DISABLE_OL			0xffff00fb

//Flowbee
#define ARCPGUFB_GET_DEFAULT_YUV	0xffff0040

// Raymond 2006/12/26
#define ARCPGUFB_SET_DEC_YUV_RES		0xffff0041	// set decoder buffer resolution.
#define ARCPGUFB_GET_DEC_YUV			0xffff0042	// get decoder buffer info.
#define ARCPGUFB_GET_DEC_YUV_BASE		0xffff0043	// get decoder buffer base address.
#define ARCPGUFB_DEC_ALLOC				0xffff0044	// allocate decoder buffer.
#define ARCPGUFB_DEC_FREE				0xffff0045	// free decoder buffer.
#define ARCPGUFB_SCALE_ALLOC			0xffff0046	// allocate display buffer.
#define ARCPGUFB_SCALE_FREE				0xffff0047	// free display buffer.
#define ARCPGUFB_SCALE_SEND				0xffff0048	// send display buffer.
#define ARCPGUFB_NUM_FREE_DEC_BUFS		0xffff0049	// get the num of free decoder buffers.
#define ARCPGUFB_NUM_FREE_SCALE_BUFS	0xffff004A	// get the num of free display buffers.

// Raymond 2007/04/30
enum scalar_state
{
	SCALAR_Y	= 0,
	SCALAR_U,
	SCALAR_V,
	SCALAR_IDLE,	
};

// Raymond 2007/04/26
typedef struct 
{
	unsigned int DecIndex, ScaleIndex;
	unsigned int YIn, UIn, VIn; 
	unsigned int YOut, UOut, VOut;
	
	// Y
	unsigned int Y_Width, Y_Height;
	unsigned int Y_xdiv, Y_ydiv, Y_dispx, Y_dispy, Y_xblank, Y_yblank;
	unsigned int Y_wait_time;

	// UV
	unsigned int UV_Width, UV_Height;
	unsigned int UV_xdiv, UV_ydiv, UV_dispx, UV_dispy, UV_xblank, UV_yblank;
	unsigned int UV_wait_time;
	
}ScaleInfo;

// Raymond 2007/05/21
#define ARCPGUFB_GET_SCALAR_STATE	0xffff00f0
#define ARCPGUFB_START_SCALAR			0xffff00f1
#define ARCPGUFB_STOP_SCALAR			0xffff00f2
#define ARCPGUFB_SET_SCALAR_INFO		0xffff00f3

// end Raymond 2007/04/26 

/*--------------------------------------------------------------------------------------------*/

/* YUV buffer layout - all sizes and offsets in bytes. Note that base addresses tend to be 
   passed around as unsigned longs, in which case you'll need to typecast before applying 
   any offsets. */

struct yuv_info {
  unsigned int displayed_buffer;
  unsigned long phys_base;
  unsigned long virt_base;
  unsigned long yuv_size;
  unsigned long alignment;
  unsigned long num_buffers;
  unsigned long start_offset;
  unsigned long width;
  unsigned long height;
  unsigned long u_offset;
  unsigned long v_offset;
  unsigned long y_stride;
  unsigned long u_stride;
  unsigned long v_stride;
#ifdef CONFIG_ARC_PGU_CROPPING
  unsigned long cropped_width;
  unsigned long cropped_height;
  unsigned long x_offset;
  unsigned long y_offset;
#endif
};

/*--------------------------------------------------------------------------------------------*/

/* Register definitions */

/*	Base+0x00    PGU_MODULE_ID   Register containing read only module ID and
 *				     Revision numbers */
#define PGU_MODULE_ID_OFFSET 0x0
#define PGU_DISPLAY_TYPE_MASK 0x1000000
#define PGU_VERSION_MASK 0xff0000
#define PGU_VERSION_SHIFT 16

/*	 Base+0x08    CLK_CFG_REG     Clock Configuration register. It is used to
 *				      Define the clock divider value, and the
 *				      pulse width (high cycle) in clock cycles:
 *				      [ 7: 0] CLK_DIV  : clock divider value - 1
 *				      [23:16] CLK_HIGH : High pulse width - 1 */
#define CLK_CFG_REG_OFFSET 0x8
#define CLK_CFG_REG_CLK_DIV_MASK 0xff
#define CLK_CFG_REG_CLK_HIGH_MASK 0xff0000
#define GET_CLK_CFG_REG_CLK_HIGH_VAL(x) (x & CLK_CFG_REG_CLK_HIGH_MASK) >> 16
#define SET_CLK_CFG_REG_CLK_HIGH_VAL(x, y) (x & !CLK_CFG_REG_CLK_HIGH_MASK) | ((y << 16) & CLK_CFG_REG_CLK_HIGH_MASK)

/*	Base+0x10    DISP_DIM_REG    Display Dimensions register. This specify
 *				     the dimensions of the actual display.
 *				     [10: 0] PGU_X_RES : X dimension - 1
 *				     [26:16] PGU_Y_RES : Y dimension - 1 */

#define DISP_DIM_REG_OFFSET 0x10
#define DISP_DIM_REG_PGU_X_RES_MASK 0x3ff
#define DISP_DIM_REG_PGU_Y_RES_MASK 0x3ff0000
#define DISP_DIM_Y_SHIFT 16
#define ENCODE_PGU_DIMS(x,y) ((x & DISP_DIM_REG_PGU_X_RES_MASK) | ((y &	 DISP_DIM_REG_PGU_X_RES_MASK) * 0x10000))

/*	Base+0x18    HSYNC_CFG_REG   Hsync Configuration register. This is used
 *				     to specify various hsync related timings in
 *				     clock cycles:
 *				     [11: 0] HSYNC_CYC	 : Hsync period - 1
 *				     [23:16] H_BCK_PORTH : Back porth width - 1
 *				     [31:23] HSYNC_WIDTH : Hsync low width - 1 */

#define PGU_SYNC_REGS_CYC_MASK 0xfff
#define PGU_SYNC_REGS_PORCH_MASK 0xff0000
#define PGU_SYNC_REGS_WIDTH_MASK 0xff000000
#define PGU_SYNC_REGS_PORCH_OFFSET 16
#define PGU_SYNC_REGS_WIDTH_OFFSET 24

#define HSYNC_CFG_REG_OFFSET 0x18

/*	Base+0x20    VSYNC_CFG_REG   Vsync Configuration register. This is used
 *				     to specify various Vsync related timings in
 *				     Hsync cycles:
 *				     [11: 0] VSYNC_CYC	 : Vsync period - 1
 *				     [23:16] V_BCK_PORTH : Back porth width - 1
 *				     [31:23] VSYNC_WIDTH : Hsync low width */

#define VSYNC_CFG_REG_OFFSET 0x20

/*	Base+0x28   FRMSTART_REG     Frame Start Address. This 32 bit address
 *				     specifies the start of the display frame
 *				     in memory. Must be at least 2 byte aligned
 *				     for 16 bit per pixel format. */

#define FRMSTART_REG_OFFSET 0x28

/*	Base+0x30   STRIDE_REG	     Line Stride Address. This 32 bit address
 *				     specifies the different in address between
 *				     the first pixels of adjecent display lines
 *				     in memory. Must be at least 2 byte aligned
 *				     for 16 bit per pixel format. */

#define STRIDE_REG_OFFSET 0x30

/*	Base+0x38   FRM_DIM_REG	     Frame Dimension register. This register
 *				     defines the frame size of the display in
 *				     memory and can be smaller than the display.
 *				     [10:0] FRM_WTH  : frame line width in
 *						       pixels. Must be <=
 *						       PGU_X_RES without x2 mode
 *						       or <= PGU_X_RES/2 in x2 
 *						       mode.
 *				     [26:16] FRM_LNS : frame number of lines
 *						       Must be <= PGU_X_RES 
 *						       without x2 mode or <= 
 *						       PGU_X_RES/2 in x2 mode. */

#define FRM_DIM_REG_OFFSET 0x38
#define FRM_DIM_REG_FRM_WTH_MASK 0x7ff
#define FRM_DIM_REG_FRM_LNS_MASK 0x7ff
						   
/*
 *	Base+0x40   STATCTRL_REG     Status and Control register. This register
 *				     is used to control the display module and
 *				     monitor its status: */
#define STATCTRL_REG_OFFSET 0x40
/*				     [0] DISP_EN   : Display enable bit. When
 *						     set to high, starts the
 *						     display. It then clears 
 *						     after a few cycles, if
 *						     the CONT bit is not set,
 *						     hence allowing the display
 *						     to do the frame once. */
#define STATCTRL_REG_DISP_EN_MASK 1
/*				     [1] INTR_EN   : Interrupt Enable. Active
 *						     high. */
#define STATCTRL_INTR_EN_MASK 2
/*				     [2] INTR_CLR  : Interrupt Clear. Set to 
 *						     high to clear the interrupt
 *						     flag/request. Always reads
 *						     as zeros */
#define STATCTRL_INTR_CLR_MASK 4
/*				     [3] CONT	   : Continuous mode. When set,
 *						     will not allow the DISP_EN
 *						     bit to be cleared by itself
 *						     and as long as DISP_EN is
 *						     not cleared manually, the
 *						     Dispaly will drive the PGU
 *						     continuously, frame after
 *						     frame. */
#define STATCTRL_CONT_B_MASK 8
/*				     [5:4] PIX_FMT   Pixel Format. Select the
 *						     pixel format:
 *						     "00" - RGB555
 *						     "01" - YUV420 (aka YV12)
 *						     "10" - reserved
 *						     "11" - reserved */
#define STATCTRL_PIX_FMT_MASK 0x30
#define STATCTRL_PIX_FMT_OFFSET 4
#define STATCTRL_PIX_FMT_RGB555 0
#define STATCTRL_PIX_FMT_YUV420 0x10
/*				     [6] BACKLIGHT   Active high backlight 
 *						     enable. When set to high,
 *						     sets the back light control
 *						     signal to high to turn on
 *						     the backlight of the PGU. */
#define STATCTRL_BACKLIGHT_MASK 0x40
/*				     [7] X2	     Zoom 2 mode. when set, 
 *						     every pixel in the frame is
 *						     display twice in a line &
 *						     the line is displayed twice
 *						     therefore enlarging the
 *						     picture. */
#define STATCTRL_X2_MASK 0x80
/*				     [8] REV_LN_SCAN Reverse line scan. When set
 *						     the display displays the
 *						     last line first, back 
 *						     towards the first */
#define STATCTRL_REV_LN_SCAN_MASK 0x100
/*				     [9] HPOL	     HSync signal polarity */
#define STATCTRL_HPOL_MASK 0x200
/*				     [10] VPOL	     VSync signal polarity */
#define STATCTRL_VPOL_MASK 0x400
/*				     [11] DPOL	     Data enable signal polarity */
#define STATCTRL_DPOL_MASK 0x800
/*				     [12] OL_EN	     Overlay enable */
#define STATCTRL_OL_EN_MASK 0x1000
/*				     [13-15] OL_FMT  Overlay format:
 *						     "00" - RGBA4444
 *						     "01" - RGBA5551
 *						     "10" - RGB555 with colour keying */
#define STATCTRL_OL_FMT_MASK 0x6000
#define STATCTRL_OL_FMT_OFFSET 13
#define STATCTRL_OL_FMT_RGBA4444 0
#define STATCTRL_OL_FMT_RGBA5551 0x2000
#define STATCTRL_OL_FMT_RGB555 0x4000
/*				     [16] DISP_BUSY  Display Busy status. Active
 *						     High. When set indicates 
 *						     that the display is 
 *						     currently scanning out 
 *						     pixel data, or generating 
 *						     syncs for the current 
 *						     display */
#define STATCTRL_DISP_BUSY_MASK 0x10000
/*				     [17] BU_BUSY    Burst Unit busy. Active 
 *						     high. When set indicates
 *						     that the burst unit is
 *						     busy fetching pixel data. */
#define STATCTRL_BU_BUSY_MASK 0x20000
/*				     [18] INTR_FLG   Interrupt flag. Refelects
 *						     the interrupt signal.
 *						     Active low level interrupt. */
#define STATCTRL_INTR_FLAG 0x40000
/*				     [19-20] IL_MD   Interface mode select
 *						     "00" - Non-interlaced
 *						     "01" - Frame mode interlace
 *						     "10" - Even field interlace
 *						     "11" - Odd field interlace */
#define STATCTRL_IL_MODE_MASK 0x180000
#define STATCTRL_IL_NONE 0
#define STATCTRL_IL_FRAME 0x80000
#define STATCTRL_IL_EVEN 0x100000
#define STATCTRL_IL_ODD 0x180000

/*--------------------------------------------------------------------------------------------*/

typedef volatile struct	 {
  volatile unsigned long pgu_id;	 /* Identity register */
  volatile unsigned long padding1;
  volatile unsigned long clk_cfg;	 /* Clock control */
  volatile unsigned long padding2;
  volatile unsigned long disp_dim;	 /* Display dimension */
  volatile unsigned long padding3;
  volatile unsigned long hsync_cfg;	 /* HSYNC */
  volatile unsigned long padding4;
  volatile unsigned long vsync_cfg;	 /* VSYNC */
  volatile unsigned long padding5;
  volatile unsigned long frm_start;	 /* Frame start */
  volatile unsigned long padding6;
  volatile unsigned long stride;	 /* Frame stride */
  volatile unsigned long padding7;
  volatile unsigned long frm_dim;	 /* Frame dimension */
  volatile unsigned long padding8;
  volatile unsigned long statctrl;	 /* Status and control */
  volatile unsigned long padding9;
  volatile unsigned long cru_frm_st;	 /* Chroma U frame start */
  volatile unsigned long padding10;
  volatile unsigned long cru_stride;	 /* Chroma U stride */
  volatile unsigned long padding11;
  volatile unsigned long crv_frm_st;	 /* Chroma V frame start */
  volatile unsigned long padding12;
  volatile unsigned long crv_stride;	 /* Chroma V stride */

#ifdef CONFIG_ENABLE_ARCPGU_OVERLAY
  volatile unsigned long padding13;
  volatile unsigned long ol_frm_st;	 /* Overlay frame start */
  volatile unsigned long padding14;
  volatile unsigned long ol_stride;	 /* Overlay stride */
  volatile unsigned long padding15;
  volatile unsigned long ol_scn_line;	 /* Overlay start and end line numbers */
  volatile unsigned long padding16;
  volatile unsigned long ol_col_key;	 /* Overlay transparency colour key - RGB 555 */
#endif

#ifdef CONFIG_ENABLE_CENTERING
  volatile unsigned long padding17;
  volatile unsigned long centreoffsets;	 /* X, Y offsets for centering */
#endif
} arc_pgu;

/*--------------------------------------------------------------------------------------------*/

#endif	/* _ASM_ARC_PGU_H */

#endif	//SK2850
