/*
 * vo_arcpgufb.c - MPlayer video out controller for the ARC pixel graphics unit.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY	DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the	 GNU General Public License along
 * with this program; if not, write	 to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright 2005 (C) ARC International. www.arc.com
 *
 * Ripped off with thanks from tdfxfb.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <linux/fb.h>

#include "config.h"
#include "fastmemcpy.h"
#include "video_out.h"
#include "video_out_internal.h"
#include "arc_pgu.h"
#include "aspect.h"
#include "sub.h"
#include "vbuf.h"

#ifdef ARC_MPLAYER_CONTROLS
#include <defns.h>
#include <osd_types.h>
#include <osd_state.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#endif

#ifdef ARC_OWN_OSD
#include <font_load.h>
#endif

#ifdef HAVE_AURORA
// This is only required if the code at REF ACM-20050909-B
// is included.
#include "../libarcavcodec/arc/aurora_asm.h"
#endif

static vo_info_t info =
{
	"ARC PGU",
	"arcpgufb",
	"ARC International <support@arc.com>",
	""
};

LIBVO_EXTERN(arcpgufb)

#define DEFAULT_FB "/dev/fb0"
#define OSD_OFFSET 70

static int fd = -1;
static struct fb_fix_screeninfo fb_finfo;
static struct fb_var_screeninfo fb_vinfo;
static uint32_t in_width, in_height, in_format, in_depth,
	screenwidth, screenheight, screendepth, vidwidth, vidheight, vidx, vidy,
	r_width, r_height;
volatile void *memBase0 = NULL;
int n_yuv_buffers; /* kludge used by H264 */

static void (*alpha_func)(), (*alpha_func_double)();
struct yuv_info yuv;
#ifndef BUILD_SK2851		//Barry 2008-05-09
static arc_pgu safe_pgu;
#endif
static int active_buffer;
static int verbose = 0;
static int can_doublebuffer = 1;
static int direct_rendering = 0;
static int frame_count = 0;
static int osd_buf = 0;

#ifdef ARC_OWN_OSD

typedef struct {
	unsigned long osd_offset;
	mp_osd_obj_t* obj;
	int x0;
	int y0;
	int w;
	int h;
	unsigned char* src;
	unsigned char *srca;
	int stride;
	int validold;
	unsigned char * oldbit;
	unsigned char *olda;
	int xold;
	int oldw;
} render_desc;

#define OSD_TEXT_LEN 63

static char buf1_contents[OSD_TEXT_LEN];
static char buf2_contents[OSD_TEXT_LEN];
static char extra1_buf1_contents[OSD_TEXT_LEN];
static char extra1_buf2_contents[OSD_TEXT_LEN];
static char extra2_buf1_contents[OSD_TEXT_LEN];
static char extra2_buf2_contents[OSD_TEXT_LEN];
int lastbuf1;
int lastbuf2;

int chars_rendered;

extern font_desc_t* vo_font;
extern mp_osd_obj_t* vo_osd_list;
extern int use_arcdemo;
#endif

#ifdef ARC_MPLAYER_CONTROLS
static int shm_id;
static int sem_id;
osd_state* osd_ctrl = 0;
struct shmid_ds shm_ctrl;
#endif

int render_static_info = 0;

typedef struct {
	volatile void** Y;
	volatile void** U;
	volatile void** V;
} YUV_planes;

static YUV_planes planes = {0,0,0};

// Raymond
static YUV_planes dec_planes = {0,0,0};	
struct yuv_info dec_yuv;

#ifdef BUILD_SK2851		//Barry 2008-05-09
int HW_Scalar = 2;
int relative = 0;
extern int set_dest, destX, destY;
#else
// Raymond -for HW scalar 2006/12/20
int HW_Scalar = 1;
#endif

int DispX = 0;
int DispY = 0;
int DispX_def = 0;
int OutX = 0;
int OutY = 0;	
//

int use_DecBuffer = 0;	// Raymond 2006/12/28

int debug_pgu = 0;		// Raymond 2007/03/15

extern int real_w;
extern int real_h;

//for scalar Barry 2007-05-11
extern int display_frame;

#define NUM_BUFFERS	5	//Barry 2007-05-28

extern int set_bufs;	//Barry 2007-07-11

// Raymond 2007/09/11

int start_display_pgu = 0;

// Raymond 2008/05/20
int mplayer_status = -100;

unsigned char *preview_buf = NULL;		// Raymond 2008/06/23

#ifndef BUILD_MPLAYER_LITE
extern int is_flv1;		//Barry 2008-08-05
#endif

#ifdef BUILD_SK2851	// Raymond 2008/06/24

#include "../libmpdemux/uncached_buf.h"
#include "../skysyscalls.h"	//Barry 2008-08-18

// Raymond 2008/08/20
int set_real_display = 0;
int disp_x = 0;
int disp_y = 0;

#endif

// Raymond 2008/06/26
static int fb_addr_to_index(void *vaddr)
{
    	int i;

	if( use_DecBuffer == 1 )
	{
		for (i=0; dec_planes.Y[i] != (unsigned)vaddr; i++);
	}
	else
	{
    		for (i=0; planes.Y[i] != (unsigned)vaddr; i++);
	}

    	return i;
}

void start_display(void)
{       	
	if(start_display_pgu == 0)
		start_display_pgu = 1;
	else
		return;	
	
	if (debug_pgu) 
		printf("start_display\n");
	
	if (ioctl(fd, ARCPGUFB_START_DISPLAY) == -1)
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_START_DISPLAY ioctl failed due to %s\n", strerror(errno));
	}

	return;
}

// Set decoder buffer resolution - Raymond 2006/12/27

int get_dec_yuv_buffers()
{
	int i;
	if (ioctl(fd, ARCPGUFB_GET_DEC_YUV, &dec_yuv) == -1) 
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_GET_DEC_YUV ioctl failed due to %s\n", strerror(errno));
		return -1;
	}

	n_yuv_buffers = dec_yuv.num_buffers;	// Raymond 2007/03/16

	if(debug_pgu)
		printf("get_dec_yuv_buffers: width = %d, height = %d\n",	dec_yuv.width, dec_yuv.height );
	
	if (debug_pgu) 
		fprintf(stderr,"arcpgufb: yuv size 0x%x no. buffers %d y stride 0x%x, u stride 0x%x, v stride 0x%x top of buffer 0x%x\n",
			dec_yuv.yuv_size, dec_yuv.num_buffers, dec_yuv.y_stride, dec_yuv.u_stride, dec_yuv.v_stride, memBase0+fb_finfo.smem_len);

	if ((! (dec_planes.Y = malloc(sizeof(void*) * dec_yuv.num_buffers)))
			|| (! (dec_planes.U = malloc(sizeof(void*) * dec_yuv.num_buffers)))
			|| (! (dec_planes.V = malloc(sizeof(void*) * dec_yuv.num_buffers))))
		return -1;

	for (i = 0; i < dec_yuv.num_buffers; i++)		// Raymond 2007/03/05 : yuv -> dec_yuv
	{
    		int offset = ioctl(fd, ARCPGUFB_GET_DEC_YUV_BASE, i);
	    
		dec_planes.Y[i] = memBase0 + offset;
		dec_planes.U[i] = dec_planes.Y[i] + (dec_yuv.u_offset);
		dec_planes.V[i] = dec_planes.Y[i] + (dec_yuv.v_offset);
		if (verbose) 
			fprintf(stderr,"arcpgufb: y%d at 0x%x, u%d at 0x%x, v%d at 0x%x limit 0x%x\n",
				 i, dec_planes.Y[i], i, dec_planes.U[i], i, dec_planes.V[i], dec_planes.V[i] + ((dec_yuv.width/2) * (dec_yuv.height/2)));
	}
	return 0;
}

int SetDecBufferResolution(int w, int h)
{
	dec_yuv.width  = w;
	dec_yuv.height = h;
	dec_yuv.num_buffers = set_bufs;	//Barry 2007-07-11

	if(debug_pgu)
		printf("\narcpgufb: SetDecoderResolution() : %d x %d\n", w, h);
	
	if (ioctl(fd, ARCPGUFB_SET_DEC_YUV_RES, &dec_yuv) == -1) 
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_SET_DEC_YUV_RES ioctl failed due to %s\n", strerror(errno));
		return -1;
	}

	usleep(100);

	// Raymond 2006/12/27
	if (get_dec_yuv_buffers()) 
		return 0;

	//Barry 2007-05-28
	if (dec_yuv.num_buffers < NUM_BUFFERS)
	{
		fprintf(stderr,"dec_yuv.num_buffers %d < 5\n", dec_yuv.num_buffers);
		return 0;
	}
#if 0	// Raymond 2008/04/02 disable
	// Raymond 2008/03/31 - for H.264
	if(ioctl(fd, ARCPGUFB_PAUSE, NULL) == -1)
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_SET_DEC_YUV_RES ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
#endif	
	return 1;

}

// Raymond 2007/04/26
unsigned char * get_bufN_Y(int n)
{
	return (unsigned char *) (planes.Y[n]) ;
}

unsigned char * get_bufN_U(int n)
{
	return (unsigned char *) (planes.U[n]) ;
}

unsigned char * get_bufN_V(int n)
{
	return (unsigned char *) (planes.V[n]) ;
}

unsigned char * get_dec_yuv_bufN(int n)
{
	return (unsigned char *) (dec_planes.Y[n]) ;
}

//Barry 2008-01-08
unsigned char * get_dec_bufN_U(int n)
{
	return (unsigned char *) (dec_planes.U[n]) ;
}

unsigned char * get_dec_bufN_V(int n)
{
	return (unsigned char *) (dec_planes.V[n]) ;
}

// Raymond 2007/05/21, Barry 2007-05-11
void vbuf_scalar_start(int n)
{
	//start_display();	// Raymond 2007/09/11	//Barry 2007-11-23
	
	int res = ioctl(fd, ARCPGUFB_START_SCALAR, &n);

	if (debug_pgu)
		printf("vbuf_start_scalar() : %d\n", n);
	
	if (res)
	{
		perror("ARCPGUFB_START_SCALAR");
	}
}
//

// Raymond 2007/04/30
int vbuf_scalar_status(void)
{
    int i = 0;
    int res = ioctl(fd, ARCPGUFB_GET_SCALAR_STATE, &i);

	if (debug_pgu) 
		printf("vbuf_scalar_status() : %d\n", i);
	
    if (res) {
	perror("ARCPGUFB_GET_SCALAR_STATE");
	exit(1);
    }

    return i;
}	


int vbuf_dec_alloc(void)
{
    int i = 0;
    int res = ioctl(fd, ARCPGUFB_DEC_ALLOC, &i);

	if (debug_pgu) 
		printf("vbuf_dec_alloc() : %d\n", i);
	
    if (res) {
	perror("ARCPGUFB_DEC_ALLOC");
	exit(1);
    }

    return i;
}

void vbuf_dec_free(int v)
{
	if (debug_pgu) 
		printf("vbuf_dec_free( %d )\n", v);

    int res = ioctl(fd, ARCPGUFB_DEC_FREE, &v);

    if (res){
	perror("ARCPGUFB_DEC_FREE");
    }
}

int vbuf_scale_alloc(void)
{
    int i = 0;
    int res = ioctl(fd, ARCPGUFB_SCALE_ALLOC, &i);

	if (debug_pgu) 
		printf("vbuf_scale_alloc() : %d\n", i);

    if (res) {
	perror("ARCPGUFB_SCALE_ALLOC");
	exit(1);
    }

    return i;
}

void vbuf_scale_free(int v)
{
	if (debug_pgu) 
		printf("vbuf_scale_free( %d )\n", v);

    int res = ioctl(fd, ARCPGUFB_SCALE_FREE, &v);

    if (res){
	perror("ARCPGUFB_SCALE_FREE");
    }
}

void vbuf_scale_send(int v)
{
	if (debug_pgu) 
		printf("vbuf_scale_send( %d )\n", v);

    if ( get_preview == 0 )		// Raymond
    {
    int res = ioctl(fd, ARCPGUFB_SCALE_SEND, &v);

    if (res)
    {
	perror("ARCPGUFB_SCALE_SEND");
    }
    }	
}

//-------------------------------------------------------------------------------------------

int vbuf_alloc(void)
{
    int i = 0;
    int res = ioctl(fd, ARCPGUFB_ALLOC, &i);
	
	if (debug_pgu) 
		printf("vbuf_alloc() : %d\n", i);

    if (res) {
	perror("ARCPGUFB_ALLOC");
	exit(1);
    }

    return i;
}

void vbuf_free(int v)
{
	if (debug_pgu) 
		printf("vbuf_free( %d )\n", v);

    int res = ioctl(fd, ARCPGUFB_FREE, &v);

    if (res){
	perror("ARCPGUFB_FREE");
    }
}

void vbuf_send(int v)
{	
	if (debug_pgu) 
		printf("vbuf_send( %d )\n", v);

    if ( get_preview == 0 )		// Raymond
    {
    	//start_display();        // Raymond 2007/09/11	// Raymond 2007/10/04	//Barry 2007-11-23
    
    int res = ioctl(fd, ARCPGUFB_SEND, &v);

    if (res)
    {
	perror("ARCPGUFB_SEND");
    }
    }	
}

#if 0	//Barry 2008-03-06
unsigned vbuf_avail(void)
{
    unsigned a;
    
    int res = ioctl(fd, ARCPGUFB_GET_AVAIL_BUFS, &a);

    if (res)
	perror("ARCPGUFB_GET_AVAIL_BUFS");

    return a;
}
#endif

//--------------------------------------------------------------------------------

static void clear_screen();

//--------------------------------------------------------------------------------

static int get_yuv_buffers()
{
	int i;
	if (ioctl(fd, ARCPGUFB_GETYUV, &yuv) == -1) {
		fprintf(stderr,"arcpgufb: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
		return -1;
	}

	n_yuv_buffers = yuv.num_buffers;	// Raymond 2007/03/15

	if (debug_pgu) 
		printf("get_yuv_buffers: width = %d, height = %d\n",	yuv.width, yuv.height );

	if (debug_pgu) 
		fprintf(stderr,"arcpgufb: yuv size 0x%x no. buffers %d y stride 0x%x, u stride 0x%x, v stride 0x%x top of buffer 0x%x\n",
			yuv.yuv_size, yuv.num_buffers, yuv.y_stride, yuv.u_stride, yuv.v_stride, memBase0+fb_finfo.smem_len);

	if ((! (planes.Y = malloc(sizeof(void*) * yuv.num_buffers)))
			|| (! (planes.U = malloc(sizeof(void*) * yuv.num_buffers)))
			|| (! (planes.V = malloc(sizeof(void*) * yuv.num_buffers))))
		return -1;

	for (i = 0; i < yuv.num_buffers; i++)
	{
    		int offset = ioctl(fd, ARCPGUFB_GET_YUV_BASE, i);
	    
		planes.Y[i] = memBase0 + offset;
		planes.U[i] = planes.Y[i] + (yuv.u_offset);
		planes.V[i] = planes.Y[i] + (yuv.v_offset);
		if (verbose) 
			fprintf(stderr,"arcpgufb: y%d at 0x%x, u%d at 0x%x, v%d at 0x%x limit 0x%x\n",
				 i, planes.Y[i], i, planes.U[i], i, planes.V[i], planes.V[i] + ((yuv.width/2) * (yuv.height/2)));
	}
	return 0;
}

//--------------------------------------------------------------------------------

static uint32_t preinit(const char *arg)
{
	int i;
	int kfps = (int)(1000.0 * vo_fps);

	if (verbose) fprintf(stderr,"arcpgufb: uint32_t preinit(const char *arg)\n");

	if((fd = open(DEFAULT_FB, O_RDWR)) == -1) {
		fprintf(stderr,"arcpgufb: can't open %s: %s\n", DEFAULT_FB, strerror(errno));
		return -1;
	}

#ifndef BUILD_SK2851		//Barry 2008-05-09
	if (ioctl(fd,	 ARCPGUFB_GET_PGU_STATE, &safe_pgu) == -1) {
		perror("arcpgufb: ARCPGUFB_GETPGU_STATE ioctl failed");
		goto error;
	}
#endif

	if(ioctl(fd, FBIOGET_FSCREENINFO, &fb_finfo)) {
	    	perror("arcpgufb: problem with FBIOGET_FSCREENINFO ioctl");
		goto error;
	}

	if(ioctl(fd, FBIOGET_VSCREENINFO, &fb_vinfo)) {
		perror("arcpgufb: problem with FBIOGET_VSCREENINFO ioctl");
		goto error;
	}

	if(ioctl(fd, ARCPGUFB_RESET_SYNC, &fb_vinfo)) {
	    	perror("arcpgufb: problem with ARCPGUFB_RESET_SYNC ioctl");
		fprintf(stderr, "You are using an old version of the kernel which does not\n"
			"support buffering. Please upgrade to a newer kernel.\n");
		exit(1);
	}
#ifndef BUILD_MPLAYER_LITE
	if (!(is_flv1==1 && get_preview==1))	//Barry 2008-08-05
	{
		if (ioctl(fd, ARCPGUFB_FRAMERATE_SET, &kfps)) {
			perror("arcpgufb: problem setting framerate");
			goto error;
		}
	}
#endif
	memBase0 = mmap(0, fb_finfo.smem_len, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);

	if((long)memBase0 == -1) {
		perror("arcpgufb: Couldn't map memory areas");
		goto error;
	}

	if (verbose) fprintf(stderr,"Framebuffer mapped at 0x%08x\n",memBase0);

	if (ioctl(fd, ARCPGUFB_GETYUV, &yuv)) {
		perror("arcpgufb: problem getting yuv buffer info");
		goto error;
	}
	
#ifdef BUILD_SK2851	// Raymond 2008/06/23
	if(get_preview == 1)
		preview_buf = memBase0 + fb_finfo.smem_len - (512<<10);
#endif
//	
	// Raymond 2006/11/20
	struct yuv_info yuv1;
	if (ioctl(fd, ARCPGUFB_GET_DEFAULT_YUV, &yuv1)) 
	{
		perror("arcpgufb: problem getting default yuv resolution");
		goto error;
	}
	if(debug_pgu)
		printf("Get PGU default resolution : %d x %d\n", yuv1.width, yuv1.height);
	DispX = yuv1.width;
	DispY = yuv1.height;

	DispX_def = DispX;	// Raymond - 2006/12/19

#ifdef BUILD_SK2851		//Barry 2008-05-09
	if ( DispX & 0xf)
		DispX = ((DispX) & ~0xf);
#else
	// Raymond fix bug - 2006/12/18
	if ( DispX & 0x3f) 
		DispX = ((DispX) & ~0x3f);
#endif

	n_yuv_buffers = yuv.num_buffers;

#ifdef HAVE_AURORA
	// TODO: Would rather not do this here!
	// REF: ACM-20050909-B
	Frame_Buffer_Virtual_Address = memBase0;
	Frame_Buffer_Physical_Address = fb_finfo.smem_start;
#endif

	//	if (get_yuv_buffers()) return -1;

#ifdef ARC_MPLAYER_CONTROLS
	if (use_arcdemo) {
		shm_id = shmget(SHMKEY, sizeof(osd_state), 0);
		osd_ctrl = (osd_state*) shmat(shm_id,NULL,0);

		chars_rendered = 0;
		
		sem_id = semget(SEMKEY, MAX_RGB_BUFFER, IPC_CREAT|IPC_EXCL);
	}
#endif

	active_buffer = 0;

	return 0;
error:
	memBase0 = NULL;
	if (fd >= 0)
	    	close(fd);
	fd = -1;
	return -1;
}

//--------------------------------------------------------------------------------

static void uninit(void)
{
	if (verbose) fprintf(stderr,"arcpgufb: void uninit(void)\n");

//	usleep(3000);		//Barry 2008-06-05

//#ifdef ALLOW_PGU_CENTERING
#if 0   //Barry 2008-07-22
	{
		unsigned long xy_offset = 0;

		xy_offset = 0;

		if (ioctl(fd, ARCPGUFB_SET_ORIGIN, &xy_offset) == -1) {
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_ORIGIN ioctl failed due to %s\n", strerror(errno));
			return;
		}
	}
#endif

//	clear_screen();	// Raymond 2008/05/21 disable

#if 1
	if( get_preview == 0 )	// Raymond 2007/02/15
	{
	#if 0	//Barry 2007-11-23
		if (ioctl(fd, ARCPGUFB_STOP_DISPLAY) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_STOP_DISPLAY ioctl failed due to %s\n", strerror(errno));
		}

		usleep(100);
	#endif
	
		yuv.width = DispX_def;
		yuv.height = DispY;
		
		if (ioctl(fd, ARCPGUFB_SET_YUV_RES, &yuv) == -1) 
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_RESOLUTION ioctl failed due to %s\n", strerror(errno));
			return;
		}

		usleep(100);
	#if 0	//Barry 2007-11-23
		if (ioctl(fd, ARCPGUFB_START_DISPLAY) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_START_DISPLAY ioctl failed due to %s\n", strerror(errno));
		}
	#endif
//		fprintf(stderr,"uninit: ARCPGUFB_SET_RESOLUTION %dx%d\n", yuv.width, yuv.height);
	}
#endif

	if(memBase0) {
		munmap(memBase0, fb_finfo.smem_len);
		memBase0 = NULL;
	}

	if(fd != -1) {
		close(fd);
		fd = -1;
	}
#ifdef ARC_MPLAYER_CONTROLS
	if (use_arcdemo) shmdt(osd_ctrl);
#endif

}

//--------------------------------------------------------------------------------

static int setup_pgu()
{
//	arc_pgu temp_pgu;	//Barry 2008-05-09

	int i;
	unsigned long tmp;
	int tmp1;

	if (verbose) fprintf(stderr,"arcpgufb: void setup_pgu()\n");

	//for scalar Barry 2007-05-11
	in_width = real_w;
	in_height = real_h;
	
	yuv.width = in_width;
	yuv.height = in_height;
//	yuv.num_buffers = set_bufs;	//Barry 2007-07-18

#ifndef BUILD_SK2851		//Barry 2008-05-09
	#ifdef ARC_DISPLAY_CROPPING
	yuv.cropped_width = in_width;
	yuv.cropped_height = in_height;
	#endif
#endif
	//active_buffer = 0;
	active_buffer = 1;	//Barry 2007-11-23

//if( get_preview == 0 )
{
#if 0	//Barry 2007-11-23
	if (ioctl(fd, ARCPGUFB_STOP_DISPLAY) == -1)
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_STOP_DISPLAY ioctl failed due to %s\n", strerror(errno));
	}

	usleep(500);
#endif
#if 0	//Barry 2008-01-02
	tmp = STATCTRL_PIX_FMT_YUV420;
	if (ioctl(fd, ARCPGUFB_SET_MAIN_MODE, &tmp) == -1) {
		fprintf(stderr,"ARCPGUFB_SET_MAIN_MODE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}

	usleep(100);
#endif
#if 0	//for scalar Barry 2007-05-11
	if (ioctl(fd, ARCPGUFB_SET_YUV_RES, &yuv) == -1) {
		fprintf(stderr,"arcpgufb: ARCPGUFB_SET_RESOLUTION ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
#else

#ifndef BUILD_SK2851		//Barry 2008-05-23
	// Raymond 2008/05/20
	if (ioctl(fd, ARCPGUFB_GET_MPLAYER_STATUS, &mplayer_status) == -1) 		//Barry 2008-03-06
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_GET_MPLAYER_STATUS ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	
	if (debug_pgu) 
		printf("mplayer_status = %d ", mplayer_status);
#endif

#ifdef BUILD_SK2851		//Barry 2008-05-09
	if (HW_Scalar == 1)	//scalar down
	{
		if (ioctl(fd, ARCPGUFB_SET_SCALAR_INFO, &yuv) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_SCALAR_INFO ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else
	{
		struct _disp_scalar_info scalar_yuv;
		if (HW_Scalar == 2)	//scalar up
		{
			scalar_yuv.en = 1;
			
			if (relative)
			{
				float w_relative, h_relative;
				w_relative = (float)DispX/in_width;
				h_relative = (float)DispY/in_height;
				if ((w_relative <= h_relative) && (w_relative < 4.0))
				{
					scalar_yuv.dest_x_size = DispX;
					scalar_yuv.dest_y_size = (int)(in_height * w_relative);
					
				}
				else if ((w_relative > h_relative) && (h_relative < 4.0))
				{
					scalar_yuv.dest_x_size = (int)(in_width * h_relative);
					scalar_yuv.dest_y_size = DispY;
				}
				else
				{
					scalar_yuv.dest_x_size = in_width << 2;
					scalar_yuv.dest_y_size = in_height << 2;
				}
			}
			else
			{
				if (((in_width << 2) >= DispX) && ((in_height << 2) >= DispY))
				{
					scalar_yuv.dest_x_size = DispX;
					scalar_yuv.dest_y_size = DispY;
				}
				else if (((in_width << 2) >= DispX) && ((in_height << 2) < DispY))
				{
					scalar_yuv.dest_x_size = DispX;
					scalar_yuv.dest_y_size = in_height << 2;
				}
				else if (((in_width << 2) < DispX) && ((in_height << 2) >= DispY))
				{
					scalar_yuv.dest_x_size = in_width << 2;
					scalar_yuv.dest_y_size = DispY;
				}
				else
				{
					scalar_yuv.dest_x_size = in_width << 2;
					scalar_yuv.dest_y_size = in_height << 2;
				}
			}
			scalar_yuv.dest_x = (DispX - scalar_yuv.dest_x_size) >> 1;
			scalar_yuv.dest_y = (DispY - scalar_yuv.dest_y_size) >> 1;			
		}
		else	//no scalar
		{
			scalar_yuv.en = HW_Scalar;
			scalar_yuv.dest_x_size = in_width;
			scalar_yuv.dest_y_size = in_height;

			if (set_dest)
			{
				scalar_yuv.dest_x = destX;
				scalar_yuv.dest_y = destY;
			}
			else
			{
				scalar_yuv.dest_x = (DispX - scalar_yuv.dest_x_size) >> 1;
				scalar_yuv.dest_y = (DispY - scalar_yuv.dest_y_size) >> 1;
			}
		}
		scalar_yuv.src_x = 0;
		scalar_yuv.src_y = 0;
		scalar_yuv.src_x_size = in_width;
		scalar_yuv.src_y_size = in_height;
		scalar_yuv.width = in_width;
		scalar_yuv.height = in_height;

		// Raymond 2008/08/20
		if( set_real_display == 1 )
		{
			scalar_yuv.src_x_size = disp_x;
			scalar_yuv.src_y_size = disp_y;
			printf("scalar_yuv.src = %3d x %3d\n", disp_x, disp_y);
		}

		//Set resolution to multiple of 2.
		scalar_yuv.src_x_size = (scalar_yuv.src_x_size & (~0x01));
		scalar_yuv.src_y_size = (scalar_yuv.src_y_size & (~0x01));
		scalar_yuv.dest_x = (scalar_yuv.dest_x & (~0x01));
		scalar_yuv.dest_y = (scalar_yuv.dest_y & (~0x01));
		scalar_yuv.dest_x_size = (scalar_yuv.dest_x_size & (~0x01));
		scalar_yuv.dest_y_size = (scalar_yuv.dest_y_size & (~0x01));

		//printf("\n\ndest_pos = %d x %d      dest_size=%d x %d\n\n", scalar_yuv.dest_x, scalar_yuv.dest_y, scalar_yuv.dest_x_size, scalar_yuv.dest_y_size);
		
		if (ioctl(fd, ARCPGUFB_SET_VIDEO_YUV_RES, &scalar_yuv) == -1) 		//Barry 2008-03-28
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_RESOLUTION ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	
#else	//SK2850

	//for scalar Barry 2007-05-11
	if (HW_Scalar == 1)
	{
		if (ioctl(fd, ARCPGUFB_SET_SCALAR_INFO, &yuv) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_SCALAR_INFO ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else
	{
		//if (ioctl(fd, ARCPGUFB_SET_YUV_RES, &yuv) == -1) 
		if (ioctl(fd, ARCPGUFB_SET_VIDEO_YUV_RES, &yuv) == -1) 		//Barry 2008-03-06
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_RESOLUTION ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
#endif	//SK2850

#ifndef BUILD_SK2851		//Barry 2008-05-23
	// Raymond 2008/05/20
	mplayer_status = 0;
	if (ioctl(fd, ARCPGUFB_SET_MPLAYER_STATUS, &mplayer_status) == -1)
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_SET_MPLAYER_STATUS ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	
	if (debug_pgu) 
		printf("=> %d\n", mplayer_status);
#endif

#endif
}

#if 0	//Barry 2007-11-23
if (HW_Scalar == 0){		//for scalar Barry 2007-05-11
#ifdef ALLOW_PGU_CENTERING
//	if( get_preview == 0 )	// Raymond 20060919
	{
		unsigned long x_offset, y_offset, xy_offset;

		x_offset = (fb_vinfo.xres - in_width) / 2;
		if (x_offset > fb_vinfo.xres) x_offset = 0;

#ifdef ARC_MPLAYER_CONTROLS
		if ((use_arcdemo) && (osd_ctrl))
		{
			if (in_height > osd_ctrl->ol_info.start_line)
			{
				y_offset = 0;             // Don't use offsets if there's any overlap
				x_offset = 0;
			}
			else
			{
				y_offset = ((fb_vinfo.yres - (fb_vinfo.yres - osd_ctrl->ol_info.start_line)) - in_height)/2;
				if (y_offset > fb_vinfo.yres) x_offset = 0;
			}
		}
		else
		{
			y_offset = (fb_vinfo.yres - in_height) / 2;
			if (y_offset > fb_vinfo.yres) x_offset = 0;
		}
#else
		y_offset = (fb_vinfo.yres - in_height) / 2;
		if (y_offset > fb_vinfo.yres) x_offset = 0;
#endif // ARC_MPLAYER_CONTROLS

		if (debug_pgu) 
			fprintf(stderr, "Setting offset to X %d, Y %d\n", x_offset, y_offset);

		xy_offset = ENCODE_PGU_DIMS(x_offset, y_offset);

		if (ioctl(fd, ARCPGUFB_SET_ORIGIN, &xy_offset) == -1) {
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_ORIGIN ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
#endif  // ALLOW_PGU_CENTERING
}
#endif

	if (planes.Y) { free(planes.Y); planes.Y = 0; }
	if (planes.U) { free(planes.U); planes.U = 0; }
	if (planes.V) { free(planes.V); planes.V = 0; }

	if (get_yuv_buffers()) return -1;

	//if (ioctl(fd, ARCPGUFB_SWITCH_YUV, &active_buffer) == -1)
	if (ioctl(fd, ARCPGUFB_SET_RGB_BUF, &active_buffer) == -1)	//Barry 2007-11-23
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_SWITCH ioctl failed due to %s\n", strerror(errno));
	}

	usleep(500);
#if 0	// Raymond 2007/09/11
	if (ioctl(fd, ARCPGUFB_START_DISPLAY) == -1)
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_START_DISPLAY ioctl failed due to %s\n", strerror(errno));
	}
#endif
	return 0;
}

//--------------------------------------------------------------------------------

static void clear_screen()
{
	int i;
	int tmp;
	
#ifdef ARC_MPLAYER_CONTROLS
	unsigned long osd_offset;
#endif

	if (verbose) fprintf(stderr,"arcpgufb: void clear_screen()\n");
	active_buffer = 0;
	
if( get_preview == 0 )	// Raymond
{
	// Raymond 2007/01/17
	if(planes.Y)	// Raymond 2007/11/30
	{
		for (i = 0; i < yuv.num_buffers; i++)
		{
			memset(planes.Y[i],0,(yuv.width*yuv.height));
			memset(planes.U[i],0x88,(yuv.width*yuv.height)/2);
		}
	}	
#if 0	// Raymond 2007/01/17
	usleep(500);

	if (ioctl(fd, ARCPGUFB_SWITCH_YUV, &active_buffer) == -1)
	{
		fprintf(stderr,"arcpgufb: ARCPGUFB_SWITCH ioctl failed due to %s\n", strerror(errno));
	}
#endif	
}

#ifdef ARC_MPLAYER_CONTROLS
	if (use_arcdemo) {
		osd_offset = ((osd_ctrl->areas[INFO_REGION].ymin * osd_ctrl->areas[INFO_REGION].width) + osd_ctrl->areas[INFO_REGION].xmin) * (fb_vinfo.bits_per_pixel/8);
		for (i = 0; i < osd_ctrl->num_avail_buffers; i++)									// FIXME merge control structures
	  {
			memset(memBase0+osd_ctrl->buffer_states[i].offset+osd_offset,0,(osd_ctrl->areas[INFO_REGION].width*osd_ctrl->areas[INFO_REGION].height*(fb_vinfo.bits_per_pixel/8)));
		}
	}
#endif
}

//--------------------------------------------------------------------------------

/* Setup output screen dimensions etc */
static void setup_screen(uint32_t full)
{
#if 0	// Raymond 2008/08/15
	if (verbose) fprintf(stderr,"arcpgufb: void setup_screen(uint32_t full)\n");
	aspect(&vidwidth, &vidheight, full ? A_ZOOM : A_NOZOOM);

	geometry(&vidx, &vidy, &vidwidth, &vidheight, screenwidth, screenheight);
	vo_fs = full;
#endif	
}

//--------------------------------------------------------------------------------

static uint32_t config(uint32_t width, uint32_t height, uint32_t d_width, uint32_t d_height,
		uint32_t flags, char *title, uint32_t format)
{
	int result = 0;

	if (verbose) fprintf(stderr,"arcpgufb: config\n");

	screenheight = fb_vinfo.yres;
	screendepth = 2; // Kludge
#if 0	// Raymond 2008/08/15	
	aspect_save_screenres(fb_vinfo.xres,fb_vinfo.yres);
#endif
	if (verbose) fprintf(stderr,"arcpgufb: void config(uint32_t width %d, uint32_t height %d, uint32_t d_width %d, uint32_t d_height %d, uint32_t flags %d, char *title %d, uint32_t format %d)\n", width, height, d_width, d_height, flags, *title, format);

	in_width = width;
	in_height = height;
	in_format = format;
#if 0	// Raymond 2008/08/15	
	aspect_save_orig(width,height);
#endif
	r_width = d_width;
	r_height = d_height;
#if 0	// Raymond 2008/08/15	
	aspect_save_prescale(d_width,d_height);
#endif
	alpha_func = 0;
	alpha_func_double = 0;

	/* Setup the screen for rendering to */

	/* Some defaults here */
	in_depth = 2;

	switch(in_format) {
		case IMGFMT_YV12:
		case IMGFMT_I420:
		case IMGFMT_IYUV:
			break;

		default:
			fprintf(stderr,"arcpgufb: Only YUV4:2:0 supported. Use fbdev for RGB.\n");
			return -1;
	}
	
if( get_preview == 0 )	// Raymond 2007/05/14
{
	//setup_screen(flags & VOFLAG_FULLSCREEN);	//Barry 2008-08-05

	result = setup_pgu();

	if (verbose) fprintf(stderr,"arcpgufb: screen is %dx%d at %d bpp, in is %dx%d at %d bpp, norm is %dx%d\n",
			screenwidth, screenheight, screendepth * 8,
			in_width, in_height, in_depth * 8,
			d_width, d_height);
}
	return result;
}

//--------------------------------------------------------------------------------

static void draw_alpha_rgba4444(int w,int h, unsigned char* src, unsigned char *srca, int srcstride, unsigned char* dstbase,int dststride)
{
	int y;
	register unsigned short *dst = (unsigned short*) dstbase;
	register int x;
	unsigned char grey;
	unsigned char alpha;

	for(y=0;y<h;y++)
	{
		dst = (unsigned short*) dstbase;
		for(x=0; x<w; x++)
		{
			if(srca[x])
			{
				//				alpha = (srca[x] & 0xf0) >> 4;
				alpha = 0x7;
				grey = (src[x] & 0xf0) >> 4;

				dst[x]=(grey<<12)|(grey<<8)|(grey<<4)|(alpha & 7);
			}
		}
		src+=srcstride;
		srca+=srcstride;
		dstbase+=dststride;
	}
	return;
}

//--------------------------------------------------------------------------------

/* Double-buffering draw_alpha - slow version */
// Needs tidying up.
static void draw_alpha_double(int x, int y, int w, int h, unsigned char *src,
		unsigned char *srca, int stride)
{
#ifdef ARC_MPLAYER_CONTROLS
	unsigned long osd_offset;
	unsigned long base_offset;
	int which;
	static int static_count = 0;
	static int extra1_count = 0;
	static int extra2_count = 0;

	if (verbose)
		fprintf(stderr,"arcpgufb:draw_alpha_double, type %d\n",x);

	// No control structure, no point continuing
	if (! osd_ctrl)
	{
		if (verbose) fprintf(stderr,"osd_ctrl is null\n");
		return;
	}

	// Do we recognise the field being passed in? (This is set in the modified version of libvo/sub.c)
	if ((x != OSDTYPE_OSD) && (x != OSDTYPE_OSDEXTRA1) && (x != OSDTYPE_OSDEXTRA2))
		return;
	// If we're being asked for static data, and it's disabled, give up.
	if (((x == OSDTYPE_OSDEXTRA1) || (x == OSDTYPE_OSDEXTRA2)) && (! osd_ctrl->show_static_info))
		return;

	// We don't need to keep regenerating the static info, so just ask the main loop for it once.
	if (osd_ctrl->show_static_info)
	{
		if (static_count++ == 0)
		{
			render_static_info = 1; // Info generation flag
			extra1_count = 0;				// Display counts
			extra2_count = 0;
		}
		else
		{
			render_static_info = 0;
		}
	}
	else
		static_count = 0;

	// No need to keep re-displaying static info, so don't. Since the static info currently overlays the controls, we can get away with this.
	// If the control progarm changes such that the player state can change while static data is being displayed, this bit will be broken.
	if (x == OSDTYPE_OSD) which = INFO_REGION;
	else if (x == OSDTYPE_OSDEXTRA1)
	{
		if (extra1_count++ > 2)
			return;
		which = STATIC1_REGION;
	}
	else if (x == OSDTYPE_OSDEXTRA2)
	{
		if (extra2_count++ > 2)
			return;
		which = STATIC2_REGION;
	}

	// OK, now we're at least likely to be about to display something. Find the offset into whichever buffer we'll be using.
	osd_offset = ((osd_ctrl->areas[which].ymin * osd_ctrl->areas[which].width) + osd_ctrl->areas[which].xmin) * (fb_vinfo.bits_per_pixel/8);

	// Find out which buffer we've been given to use.
	osd_buf ^= 1;
	if (osd_buf)
		which = osd_ctrl->current_buf1;
	else
		which = osd_ctrl->current_buf0;

	base_offset = osd_ctrl->buffer_states[which].offset;
	osd_offset += base_offset;

	if (verbose)
		fprintf(stderr,"Base offset 0x%x osd offset 0x%x will write to buffer %d (%d), final address 0x%x\n",base_offset,osd_offset,
						(osd_buf ? osd_ctrl->current_buf1 : osd_ctrl->current_buf0), osd_buf,memBase0+osd_offset);

	// Blank the dynamic part
	if (! osd_ctrl->buffer_states[which].blank)
		memset(memBase0+osd_offset,0,(osd_ctrl->areas[INFO_REGION].width*osd_ctrl->areas[INFO_REGION].height*(fb_vinfo.bits_per_pixel/8)));

	// Finally. Display the data. Being lazy here - if no basic info, then we won't show the static info.
	if (osd_ctrl->show_basic_info)
		draw_alpha_rgba4444(580 /* FIXME */ ,h,src,srca,stride, memBase0+osd_offset,(fb_vinfo.xres*2));

	// Only toggle the buffer if we're displaying dynamic (basic) data. Reset the buffer id if we're not going to toggle the display.
	if (x == OSDTYPE_OSD)
	{
		// Did we just leave the dynamic buffer blank?
		osd_ctrl->buffer_states[which].blank = (! osd_ctrl->show_basic_info);
		if (ioctl(fd,	 ARCPGUFB_SET_OL_OFFSET, &base_offset) == -1) {
			fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_OL_OFFSET ioctl failed due to %s\n", strerror(errno));
		}
	}
	else
		osd_buf ^= 1;

#endif
}

//--------------------------------------------------------------------------------

/* Single-buffering draw_alpha */
static void draw_alpha(int x, int y, int w, int h, unsigned char *src,
		unsigned char *srca, int stride)
{
	if (verbose)
		fprintf(stderr,"arcpgufb:draw_alpha. 0x%x %s\n", srca, srca);
}

//--------------------------------------------------------------------------------

#ifdef ARC_OWN_OSD

// The standard OSD functions are too slow for us, so here's an attempt to speed things up
// by directly rendering the OSD into the overlay framebuffer.
// We don't bother with subtitles, nor with the progress bar. We don't bother with bounding
// boxes, but use the structures for some data exchange.
// Only three lines of text can be displayed: the dynamic info line (OSDTYPE_OSD) containing
// pts, framecount and fps; the two static info lines (OSDTYPE_OSDEXTRA1 and OSDTYPE_OSDEXTRA2)
// containing fixed info about the current stream. The fixed lines are rendered only when
// absolutely necessary, and then in their entirity. The dynamic line is updated every frame.
// To keep this fast, the dynamic contents are retained as a string between calls to update the
// OSD, and only the characters that have changed are rerendered. Since we're double-buffering,
// we have to keep two strings. Note that it is occasinally necessary to force a complete
// regeneration of the dynamic display bitmap, e.g. when the buffers are switched due to the
// stream being paused. In this case memset our  slice of the framebuffer to 0, and render the
// OSD from scratch.The native MPlayer fonts make life unpleasant for us when rendering - each
// is made up of a bitmap and an underlying alpha map. A bit should be displayed only if it is in
// the correct state in both maps. Since the alpha map appears to be noisy, we apply a threshold
// to stave off jagged edges on the displayed text. The maps are noticably wider than the characters,
// and as a result it seems a negative inter character spacing is used - the blank edges of each bitmap
// intersect with those of the neighbouring characters. In order to drop a new character in place of
// an existing one, we therefore have to have access to both the bitmap and alpha map for both the
// new and old characters. This allows us to identify which pixels to set and which to clear
// without interfering with neighbouring characters.

#define ALPHA_CLIP 0x10
#define SET 0xafff
#define BLANK 0

// Render
static void dr_alpha_buf(render_desc* desc)
{
	int dststride = fb_vinfo.xres;
	int srcskip = desc->stride - desc->w;
	int i, j;
	unsigned short *dst = desc->osd_offset + (desc->x0 * 2); // FIXME bytes per pixel
	unsigned short *linedst;
	unsigned char *bs = desc->src;
	unsigned char *as = desc->srca;

	for (i = desc->h - 1; i; i--) {
		linedst = dst;
		for (j = desc->w; j; j--, bs++, as++)
		{
			if ((*bs) && (*as < ALPHA_CLIP)) *linedst = SET; // FIXME dirtydirtydirty, and assumes RGBA4444
			linedst++;
		}
		dst += dststride;
		bs+= srcskip;
		as+= srcskip;
	}
}

// Render and overwrite
static void dr_ow_alpha_buf(render_desc* desc)
{
	int which;
	int dststride = fb_vinfo.xres;
	int srcskip = desc->stride - desc->w;
	int oldskip = desc->stride - desc->oldw;
	int i, j;
	unsigned short *dst = desc->osd_offset + (desc->x0 * 2); // FIXME bytes per pixel
	unsigned short *old = desc->osd_offset + (desc->xold * 2);
	unsigned short *linedst;
	unsigned short *oldline;
	unsigned char *bs = desc->src;
	unsigned char *as = desc->srca;
	unsigned char *ob = desc->oldbit;
	unsigned char *oa = desc->olda;

	for (i = desc->h -1; i; i--) {
		linedst = dst;
		for (j = desc->w; j; j--, bs++, as++, oa++, ob++) {
			if ((*bs) && (*as < ALPHA_CLIP)) *linedst = SET; // FIXME as above
			else if ((*ob) && (*oa < ALPHA_CLIP)) *linedst = BLANK;
			linedst++;
		}
		dst += dststride;
		bs+= srcskip;
		as+= srcskip;
		oa+= oldskip;
		ob+= oldskip;
	}
}

// OSD write complete, toggle display pointer
static void flip_osd()
{
	unsigned long base_offset;
	int which;

	if (verbose)	fprintf(stderr,"arcpgufb:flip_osd.\n");

	which = osd_buf ? osd_ctrl->current_buf1 : osd_ctrl->current_buf0;
	base_offset = osd_ctrl->buffer_states[which].offset;

	//	osd_ctrl->buffer_states[which].blank = (! osd_ctrl->show_basic_info);
	if (ioctl(fd,	 ARCPGUFB_SET_OL_OFFSET, &base_offset) == -1) {
		fprintf(stderr,"Can\'t get PGU: ARCPGUFB_SET_OL_OFFSET ioctl failed due to %s\n", strerror(errno));
	}
	osd_buf ^= 1;
}

// Write text into the appointed buffer
static void arc_update_text_osd(mp_osd_obj_t* obj,int dxs,int dys)
{
	unsigned char *cp, *tmpcp;
	int x=20;
	static int h=0;
	int font;
	unsigned long osd_offset;
	unsigned long other_offset;
	unsigned long base_offset;
	int which, other;
	char *old, *op;
	int xold, cold;
	render_desc desc;
	int force_rerender = 0;

	if (verbose)	fprintf(stderr,"arcpgufb:arc_update_text_osd.\n");
	if (! osd_ctrl)
	{
		if (verbose) fprintf(stderr,"osd_ctrl is null - no idea where to render to\n");
		return;
	}

	cp = vo_osd_text;
	which = INFO_REGION;
	old = (osd_buf) ? buf2_contents : buf1_contents;

	if (obj->type == OSDTYPE_OSDEXTRA1)
	{
		cp = vo_osd_text_extra1;
		which = STATIC1_REGION;
		old = (osd_buf) ? extra1_buf2_contents : extra1_buf1_contents;
	}
	else
	{
		if (obj->type == OSDTYPE_OSDEXTRA2)
		{
			cp = vo_osd_text_extra2;
			which = STATIC2_REGION;
			old = (osd_buf) ? extra2_buf2_contents : extra2_buf1_contents;
		}
		else
		{
			//			printf("Status\n");
		}
	}
	tmpcp = cp;

	osd_offset = ((osd_ctrl->areas[which].ymin * osd_ctrl->areas[which].width) + osd_ctrl->areas[which].xmin) * (fb_vinfo.bits_per_pixel/8);

	// Find out which buffer we've been given to use.
	which = (osd_buf) ? osd_ctrl->current_buf1 : osd_ctrl->current_buf0;
	other = (osd_buf) ? osd_ctrl->current_buf0 : osd_ctrl->current_buf1;

	base_offset = osd_ctrl->buffer_states[which].offset;
	other_offset = osd_ctrl->buffer_states[other].offset;
	other_offset += osd_offset;
	osd_offset += base_offset;

	if (which != lastbuf2)
	{
		// Bugger
		//		printf("Re-rendering, since this buf is %d, last but 1 is %d amd last is %d\n",which, lastbuf2, lastbuf1);
		if (obj->type == OSDTYPE_OSD)
		{
			buf1_contents[0] = '\0';
			buf2_contents[0] = '\0';
			memset(memBase0+osd_offset,0,(osd_ctrl->areas[INFO_REGION].width*osd_ctrl->areas[INFO_REGION].height*(fb_vinfo.bits_per_pixel/8)));
			memset(memBase0+other_offset,0,(osd_ctrl->areas[INFO_REGION].width*osd_ctrl->areas[INFO_REGION].height*(fb_vinfo.bits_per_pixel/8)));
		}
		if (obj->type == OSDTYPE_OSDEXTRA1)
		{
			extra1_buf1_contents[0] = '\0';
			extra1_buf2_contents[0] = '\0';
			memset(memBase0+osd_offset,0,(osd_ctrl->areas[STATIC1_REGION].width*osd_ctrl->areas[STATIC1_REGION].height*(fb_vinfo.bits_per_pixel/8)));
			memset(memBase0+other_offset,0,(osd_ctrl->areas[STATIC1_REGION].width*osd_ctrl->areas[STATIC1_REGION].height*(fb_vinfo.bits_per_pixel/8)));
		}
		if (obj->type == OSDTYPE_OSDEXTRA2)
		{
			extra2_buf1_contents[0] = '\0';
			extra2_buf2_contents[0] = '\0';
			memset(memBase0+osd_offset,0,(osd_ctrl->areas[STATIC2_REGION].width*osd_ctrl->areas[STATIC2_REGION].height*(fb_vinfo.bits_per_pixel/8)));
			memset(memBase0+other_offset,0,(osd_ctrl->areas[STATIC2_REGION].width*osd_ctrl->areas[STATIC2_REGION].height*(fb_vinfo.bits_per_pixel/8)));
		}
	}

	if (obj->type == OSDTYPE_OSD)
	{
		lastbuf2 = lastbuf1;
		lastbuf1 = which;
	}

	if (verbose)
		fprintf(stderr,"Base offset 0x%x osd offset 0x%x will write to buffer %d (%d), final address 0x%x\n",base_offset,osd_offset,
						(osd_buf ? osd_ctrl->current_buf1 : osd_ctrl->current_buf0), osd_buf,memBase0+osd_offset);

	obj->bbox.x1=obj->x=x;
	obj->bbox.y1=obj->y=10;

	if (!h)
	{
		while (*cp){
			int c=*cp++;
			if ((font=vo_font->font[c])>=0)
				if (h < vo_font->pic_a[font]->h) h = vo_font->pic_a[font]->h;
		}
		cp = tmpcp;
	}

	x = obj->x;

	op = old;
	xold = x;
	desc.osd_offset =  memBase0+osd_offset;
	desc.obj = obj;
	desc.y0 = obj->y;
	desc.h = h;

	while (*cp){
		int c=*cp++;

		cold = (*op) ? *op++ : 0;

		if (((x != xold) || (c != cold)) && ((font=vo_font->font[c])>=0))
		{
			//			chars_rendered++;

			desc.x0 = x;
			desc.w = vo_font->width[c];
			desc.src = vo_font->pic_b[font]->bmp+vo_font->start[c];
			desc.srca = vo_font->pic_a[font]->bmp+vo_font->start[c];
			desc.stride = vo_font->pic_a[font]->w;
			if (desc.validold = cold)
			{
				desc.oldbit = vo_font->pic_b[font]->bmp+vo_font->start[cold];
				desc.olda = vo_font->pic_a[font]->bmp+vo_font->start[cold];
				desc.xold = xold;
				desc.oldw = vo_font->width[cold];

				if (x != xold) { force_rerender = 1; break; }

				dr_ow_alpha_buf(&desc);
			}
			else
				dr_alpha_buf(&desc);
		}
		x+=vo_font->width[c]+vo_font->charspace;
		xold+=vo_font->width[cold]+vo_font->charspace;
	}

	if (obj->type == OSDTYPE_OSD)
	{
		flip_osd();
	}

	if (force_rerender)
	{
		lastbuf1 = lastbuf2 = -1;
	}

	strncpy(old,tmpcp,OSD_TEXT_LEN);
}

// OSD update due - figure out what and where
int arc_update_osd(int dxs,int dys)
{
	mp_osd_obj_t* obj=vo_osd_list;
	int chg=0;
	static int static_count = 0;
	static int extra1_count = 0;
	static int extra2_count = 0;

	if (verbose)	fprintf(stderr,"arcpgufb:arc_update_osd.\n");
	// We don't need to keep regenerating the static info, so just ask the main loop for it once.
	if ((osd_ctrl) && (osd_ctrl->show_static_info))
	{
		if (static_count++ == 0)
		{
			render_static_info = 1; // Info generation flag
			extra1_count = 0;				// Display counts
			//extra2_count = 0;
		}
		else
		{
		  //			render_static_info = 0;
		}
	}
	else
		static_count = 0;

	while(obj) {
		if(dxs!=obj->dxs || dys!=obj->dys || obj->flags&OSDFLAG_FORCE_UPDATE){
			int vis=obj->flags&OSDFLAG_VISIBLE;
			obj->flags&=~OSDFLAG_BBOX;
			switch(obj->type){
				case OSDTYPE_OSD:
					if(vo_font && vo_osd_text && vo_osd_text[0]){
						arc_update_text_osd(obj,dxs,dys); // update bbox
						obj->flags|=OSDFLAG_VISIBLE|OSDFLAG_CHANGED;
					} else
						obj->flags&=~OSDFLAG_VISIBLE;
					break;
				case OSDTYPE_OSDEXTRA1:
					// If we're being asked for static data, and it's disabled, give up.
					if ((! osd_ctrl) || (! osd_ctrl->show_static_info))
						//						continue;
						goto next;
					else
					{
						if(vo_font && vo_osd_text_extra1 && vo_osd_text_extra1[0] && (++extra1_count <= 3)){
							arc_update_text_osd(obj,dxs,dys); // update bbox
							obj->flags|=OSDFLAG_VISIBLE|OSDFLAG_CHANGED;
						} else
							obj->flags&=~OSDFLAG_VISIBLE;
					}
					break;
				case OSDTYPE_OSDEXTRA2:
					// If we're being asked for static data, and it's disabled, give up.
				        if ((! osd_ctrl) || (! osd_ctrl->show_static_info))
						//						continue;
						goto next;
					else
					{
					  //						if(vo_font && vo_osd_text_extra2 && vo_osd_text_extra2[0] && (++extra2_count <= 2)){
						if(vo_font && vo_osd_text_extra2 && vo_osd_text_extra2[0]){
							arc_update_text_osd(obj,dxs,dys); // update bbox
							obj->flags|=OSDFLAG_VISIBLE|OSDFLAG_CHANGED;
						} else
							obj->flags&=~OSDFLAG_VISIBLE;
					}
					break;
				default:
					//					continue;
					goto next;
			}
			obj->flags&=~OSDFLAG_FORCE_UPDATE;
		}
		if(obj->flags&OSDFLAG_CHANGED){
			chg|=1<<obj->type;
		}
	next:
		obj=obj->next;
	}
	return chg;
}

// OSD display entry point
void arc_draw_text(int dxs,int dys)
{
	mp_osd_obj_t* obj=vo_osd_list;

	if (verbose)	fprintf(stderr,"arcpgufb:arc_draw_text.\n");

	arc_update_osd(dxs,dys);

	while(obj){
		if(obj->flags&OSDFLAG_VISIBLE){
			vo_osd_changed_flag=obj->flags&OSDFLAG_CHANGED; // temp hack
			obj->old_bbox=obj->bbox;
			obj->flags|=OSDFLAG_OLD_BBOX;
		}
		obj->flags&=~OSDFLAG_CHANGED;
		obj=obj->next;
	}
}

#endif

//--------------------------------------------------------------------------------

static void draw_osd(void)
{
	if (verbose)	fprintf(stderr,"arcpgufb:draw_osd.\n");

#ifdef ARC_MPLAYER_CONTROLS
	if (use_arcdemo) 
	{
		if (! osd_ctrl->show_basic_info)
		{
			lastbuf1 = lastbuf2 = -1;
			return;
		}
	}
#ifdef ARC_OWN_OSD
		if (use_arcdemo) arc_draw_text(in_width, in_height);
#else
		vo_draw_text(in_width, in_height, draw_alpha_double);
#endif
#endif
}

//--------------------------------------------------------------------------------

/* Render onto the screen */
static void flip_page(void)
{
#if 0
    	/* Commented out because flipping is now done by the kernel's timer interrupt */

	if (verbose)	fprintf(stderr,"arcpgufb:flip_page. active buffer %d\n", active_buffer);
	if ((! direct_rendering) && (can_doublebuffer))
	{
		if (ioctl(fd, ARCPGUFB_SWITCH_YUV, &active_buffer) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_SWITCH ioctl failed due to %s\n", strerror(errno));
		}
		active_buffer = active_buffer ? 0 : 1;
	}

	if (direct_rendering)
	{
		if (ioctl(fd, ARCPGUFB_SWITCH_YUV, &active_buffer) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_SWITCH ioctl (DR) failed due to %s\n", strerror(errno));
		}
	}
#endif
}

//--------------------------------------------------------------------------------

static uint32_t draw_frame(uint8_t *src[])
{
	if (verbose) fprintf(stderr,"arcpgufb:draw_frame.\n");
	// No RGB output with this driver right now.
	return 0;
}

//--------------------------------------------------------------------------------

static uint32_t draw_slice(uint8_t *src[], int stride[], int w, int h, int x, int y)
{
	int i, offset, step;
	void *dst;
	void *srcp;

	if (verbose) fprintf(stderr,"arcpgufb: draw_slice entered w %d h %d x %d y %d long offset %d short offset %d\n",w,h,x,y,y*320,y*640);

	dst = planes.Y[active_buffer];
	step = in_width;
	offset = (y * step) + x;
	srcp = src[0];
	dst += offset;
	for (i=0;i<h;i++) {
		memcpy(dst,srcp,w);
		dst += step;
		srcp += stride[0];
	}

	srcp = src[1];
	step /= 2;
	offset = ((y/2) * step) + (x/2);
	w /= 2;
	h /= 2;
	dst = planes.U[active_buffer];
	dst += offset;
	for (i=0;i<h;i++) {
		memcpy(dst,srcp,w);
		dst += step;
		srcp += stride[1];
	}

	srcp = src[2];
	dst = planes.V[active_buffer];
	dst += offset;
	for (i=0;i<h;i++) {
		memcpy(dst,srcp,w);
		dst += step;
		srcp += stride[2];
	}

	return 0;
}

//--------------------------------------------------------------------------------

/* Attempt to start doing DR */
static uint32_t get_image(mp_image_t *mpi)
{
	int buf;

	if (verbose)
	{
		fprintf(stderr,"arcpgufb:get_image.%d\n", mpi->w);

		if ( (mpi->flags & MP_IMGFLAG_PLANAR)) fprintf(stderr,"arcpgufb:get_image. Not planar\n");
		if ( (mpi->flags & MP_IMGFLAG_ACCEPT_ALIGNED_STRIDE)) fprintf(stderr,"arcpgufb:get_image. Aligned stride accepted\n");
		if ( (mpi->flags & MP_IMGFLAG_PREFER_ALIGNED_STRIDE)) fprintf(stderr,"arcpgufb:get_image. Aligned stride preferred\n");
		if ( (mpi->flags & MP_IMGFLAG_ACCEPT_STRIDE)) fprintf(stderr,"arcpgufb:get_image. Stride accepted\n");
		if ( (mpi->flags & MP_IMGFLAG_ACCEPT_WIDTH)) fprintf(stderr,"arcpgufb:get_image. Width accepted\n");
		if ( (mpi->flags & MP_IMGFLAG_COMMON_STRIDE)) fprintf(stderr,"arcpgufb:get_image. Common stride\n");
		if ( (mpi->flags & MP_IMGFLAG_COMMON_PLANE)) fprintf(stderr,"arcpgufb:get_image. Common plane\n");

		if ( (mpi->type == MP_IMGTYPE_EXPORT)) fprintf(stderr,"arcpgufb:get_image. Export\n");
		if ( (mpi->type == MP_IMGTYPE_STATIC)) fprintf(stderr,"arcpgufb:get_image. Static\n");
		if ( (mpi->type == MP_IMGTYPE_TEMP)) fprintf(stderr,"arcpgufb:get_image. Temp\n");
		if ( (mpi->type == MP_IMGTYPE_IP)) fprintf(stderr,"arcpgufb:get_image. IP\n");
		if ( (mpi->type == MP_IMGTYPE_IPB)) fprintf(stderr,"arcpgufb:get_image. IPB\n");

		if(mpi->flags & MP_IMGFLAG_READABLE) fprintf(stderr,"arcpgufb: read image set\n");
	}

	buf = vbuf_alloc();

	frame_count++;
	if (verbose) fprintf(stderr,"arcpgufb:get_image. Writing frame %d to buffer %d\n",frame_count,buf);

	switch(in_format) {
		case IMGFMT_YV12:
		case IMGFMT_I420:
		case IMGFMT_IYUV:
			if(!(mpi->flags & MP_IMGFLAG_ACCEPT_STRIDE) && mpi->w != in_width)
			{
				fprintf(stderr,"arcpgufb:get_image. Stride mismatch\n");
				return VO_FALSE;
			}

			mpi->planes[0] = planes.Y[buf];
			mpi->planes[1] = planes.U[buf];
			mpi->planes[2] = planes.V[buf];
			mpi->priv = (void*) ((frame_count << 16) | buf);

			mpi->stride[0] = in_width;
			mpi->stride[1] = (in_width/2);
			mpi->stride[2] = (in_width/2);
			break;

		default:
			fprintf(stderr,"arcpgufb:get_image - unsupported csp.\n");
			return VO_FALSE;
	}

	direct_rendering = 1;
	mpi->width = in_width;
	mpi->height = in_height;
	mpi->flags |= MP_IMGFLAG_DIRECT;

	if (verbose) fprintf(stderr,"arcpgufb:get_image. leaving\n");
	return VO_TRUE;
}

//--------------------------------------------------------------------------------

static int draw_image(mp_image_t *mpi)
{
	int buf;
	int this_frame;
	unsigned long statctrl;

	if (verbose) fprintf(stderr,"arcpgufb:draw_image.\n");

#ifndef BUILD_SK2851		//Barry 2008-05-09
#ifdef ARC_DISPLAY_CROPPING
	if ( HW_Scalar == 0 && get_preview == 0 )	// Raymond - disable for HW scalar , 2006/10/26	
	if ((mpi->w != yuv.cropped_width) || (mpi->h != yuv.cropped_height)) {
		if (verbose) fprintf(stderr,"\n\nChange in image size - w now %d, h now %d\n\n",mpi->w,mpi->h);
	#if 0	//Barry 2007-11-23
		if (ioctl(fd, ARCPGUFB_STOP_DISPLAY) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_STOP_DISPLAY ioctl failed due to %s\n", strerror(errno));
		}


		usleep(100);
	#endif
		yuv.cropped_height = mpi->h;
		yuv.cropped_width = mpi->w;
		yuv.x_offset = mpi->x;
		yuv.y_offset = mpi->y;
		//if (ioctl(fd, ARCPGUFB_SET_YUV_RES, &yuv) == -1) {
		if (ioctl(fd, ARCPGUFB_SET_VIDEO_YUV_RES, &yuv) == -1){ 		//Barry 2008-03-06
			fprintf(stderr,"arcpgufb: ARCPGUFB_SET_RESOLUTION ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
		usleep(1000);
	#if 0	//Barry 2007-11-23
		if (ioctl(fd, ARCPGUFB_START_DISPLAY) == -1)
		{
			fprintf(stderr,"arcpgufb: ARCPGUFB_START_DISPLAY ioctl failed due to %s\n", strerror(errno));
		}
	#endif
	}
#endif
#endif

	if (mpi->flags & MP_IMGFLAG_DIRECT)
	{
		buf = ((int) mpi->priv) & 0xff;
		
		if (mpi->type == MP_IMGTYPE_TEMP) //FIXME this breaks unaccelerated MPEG
		{
		    	/* XXX FIXME 
			   Empircally, we get here if we are using the VC1 codec which does not send the 
			   frame to be displayed itself. The codition "works", but should be replaced
			   by something more rational */
			vbuf_send(buf);
			vbuf_free(buf);
		}
		active_buffer = buf;
		if (verbose)
		{
			this_frame = (((int) mpi->priv) & 0xffff0000) >> 16;

			fprintf(stderr,"arcpgufb:draw_image. frame %d directly displayed from buffer %d\n",this_frame,buf);
		}
	}
	else
	{
//		if( ( HW_Scalar == 1 ) && get_preview == 0 && ( ( mpi->width != DispX ) || ( mpi->height != DispY)  ) )
//			active_buffer = 8;
		
		if (verbose)
			fprintf(stderr,"arcpgufb:draw_image. frame %d slowly copied into into buffer %d\n",this_frame,active_buffer);
#if 0		
		memcpy(planes.Y[active_buffer],mpi->planes[0],mpi->width* mpi->height);
		memcpy(planes.U[active_buffer],mpi->planes[1],(mpi->width/2)*(mpi->height/2));
		memcpy(planes.V[active_buffer],mpi->planes[2],(mpi->width/2)*(mpi->height/2));
#else
		int rw = mpi->width;
		int rh = mpi->height;
		int dw = (rw + 0xf) & ~0xf;	

		// Raymond - 2007/01/02
		int a = -1;
		int c = -1;
		char *src[3], *dst[3];

		// Raymond - swap UV for VC-1 decoder output
		src[0] = mpi->planes[0];
		src[1] = mpi->planes[2];
		src[2] = mpi->planes[1];

//		if( rw > DispX || rh > DispY )
		if( rw > DispX || rh > DispY || ( HW_Scalar == 1 ) )	// Raymond 2007/03/21
		{
			if( (rw & 0xf) || (rh & 0xf) )	// Raymond 2008/06/27
			{
				c = vbuf_dec_alloc();	// Raymond 2007/01/02
				dst[0] = dec_planes.Y[c];	
				dst[1] = dec_planes.U[c];	
				dst[2] = dec_planes.V[c];
			}
			else
				c = fb_addr_to_index(mpi->planes[0]);
		}
		else
		{
#if 1		// Raymond 2007/03/13 - use vbuf_alloc() to do A/V sync
			// Raymond 2008/06/26
			if( (rw & 0xf) || (rh & 0xf) )
			{
				a = vbuf_alloc();
				dst[0] = planes.Y[a];
				dst[1] = planes.U[a];
				dst[2] = planes.V[a];
			}
			else
				a = fb_addr_to_index(mpi->planes[0]);			
			
#else	
			a = vbuf_alloc();
			dst[0] = planes.Y[a];	
			dst[1] = planes.U[a];	
			dst[2] = planes.V[a];	
#endif
		}

		if( (rw & 0xf) || (rh & 0xf) )	// Raymond 2008/06/26		
		if( rw & 0xf )	// Raymond 2006/12/21
		{
			// not multiple of 16, copy rw to dw
			int i = 0;
			int rw2 = rw >> 1;
			int dw2 = dw >> 1;
			int rh2  = rh  >> 1;

			// Y		
			for( i = 0 ; i < rh ; i++ )
				memcpy(dst[0]+i*dw, src[0]+i*rw, rw);
			
			// U		
			for( i = 0 ; i < rh2; i++ )
				memcpy(dst[1]+i*dw2, src[1]+i*rw2, rw2);

			// V		
			for( i = 0 ; i < rh2; i++ )
				memcpy(dst[2]+i*dw2, src[2]+i*rw2, rw2);
		
		}
		else
		{
#ifdef BUILD_SK2851	// Raymond 2008/06/24
			int ilen = mpi->width * mpi->height;
			int ilen2 = ilen >> 2;

			if(ilen & 31)
				memcpy(dst[0],src[0],ilen);
			else
				DMA_COPY(VIRTUAL_TO_FB(dst[0]), VIRTUAL_TO_UCM(src[0]), ilen);				
			
			if(ilen2 & 31)
			{
				memcpy(dst[1],src[1],ilen2);
				memcpy(dst[2],src[2],ilen2);				
			}
			else
			{
				DMA_COPY(VIRTUAL_TO_FB(dst[1]), VIRTUAL_TO_UCM(src[1]), ilen2);
				DMA_COPY(VIRTUAL_TO_FB(dst[2]), VIRTUAL_TO_UCM(src[2]), ilen2);
			}
#else
			memcpy(dst[0],src[0],mpi->width* mpi->height);
			memcpy(dst[1],src[1],(mpi->width/2)*(mpi->height/2));
			memcpy(dst[2],src[2],(mpi->width/2)*(mpi->height/2));
#endif			
		}
		
#if 0	//for scalar Barry 2007-05-11
		// Raymond - for VC-1 HW scalar , 2006/11/14
		if( ( HW_Scalar == 1 ) && get_preview == 0 && ( ( mpi->width != DispX ) || ( mpi->height != DispY)  ) )
		{
//			int b = vbuf_alloc();
			int b = vbuf_scale_alloc();	// Raymond 2007/03/21
			
			unsigned int YAddr, UAddr, VAddr;
			unsigned int Yout, Uout, Vout;
					
			Yout = VIRTUAL_TO_FB(planes.Y[b]);
			Uout = VIRTUAL_TO_FB(planes.U[b]);
			Vout = VIRTUAL_TO_FB(planes.V[b]);

			YAddr = VIRTUAL_TO_FB(dst[0]);
			UAddr = VIRTUAL_TO_FB(dst[1]);
			VAddr = VIRTUAL_TO_FB(dst[2]);

			ScaleFrame(YAddr, UAddr, VAddr, Yout, Uout, Vout, c, b);

			// Raymond 2007/03/21
			vbuf_scale_send(b);	
	      		vbuf_scale_free(b);
				
//		 	vbuf_send(b);
//	      		vbuf_free(b);
				
			// Raymond - 2007/01/02	
			if( c != -1 )
				vbuf_dec_free(c);
			
		}
#else
		//for scalar Barry 2007-05-11
		if( HW_Scalar == 1 ) 
		{
			int i = 0;
			if (display_frame == 1)
				vbuf_scalar_start(c);
			else{
				while( (i = vbuf_scalar_status()) != SCALAR_IDLE) 
					usleep(1);
				vbuf_scalar_start(c);
			}
			
			// Raymond - 2007/01/02	
			if( c != -1 )
				vbuf_dec_free(c);
			
		}
#endif	//for scalar Barry 2007-05-11		
		else
		{
			// Raymond 2007/03/13
			if( a != -1 )
				vbuf_send(a);
		}

		// Raymond 2007/03/13
		if( a != -1 )
			vbuf_free(a);
			
#endif		
	}

	//vbuf_send(buf);

	return VO_TRUE;
}

//--------------------------------------------------------------------------------

static uint32_t start_slice(mp_image_t *mpi){
	if (verbose) fprintf(stderr,"arcpgufb:start slice 0x%x %d\n",mpi,(int)mpi->priv);

	mpi->flags &= ~MP_IMGFLAG_DRAW_CALLBACK; // None of that nonsense here

	return VO_TRUE;
}

//--------------------------------------------------------------------------------

static uint32_t control(uint32_t request, void *data, ...)
{
   int val = 0;
   if (verbose) fprintf(stderr,"arcpgufb:control.\n");

	switch(request) {
		case VOCTRL_GET_IMAGE:
			if (verbose) fprintf(stderr,"arcpgufb:control - get image\n");
			return get_image(data);

		case VOCTRL_DRAW_IMAGE:
			if (verbose) fprintf(stderr,"arcpgufb:control - draw image\n");
			return draw_image(data);

		case VOCTRL_QUERY_FORMAT:
			if (verbose) fprintf(stderr,"arcpgufb:control - query format\n");
			switch(*((uint32_t*)data)) {
				case IMGFMT_YV12:
				case IMGFMT_I420:
				case IMGFMT_IYUV:
					return VFCAP_CSP_SUPPORTED | VFCAP_CSP_SUPPORTED_BY_HW | VFCAP_ACCEPT_STRIDE | VFCAP_TIMER;
			}

			return 0;		/* Not supported */

		case VOCTRL_FULLSCREEN:
			if (verbose) fprintf(stderr,"arcpgufb:control - fullscreen\n");
			setup_screen(!vo_fs);
			return 0;

		case VOCTRL_START_SLICE:
			return start_slice(data);
		case VOCTRL_PAUSE:
			if (verbose) fprintf(stderr,"arcpgufb:control - pause\n");
			ioctl(fd, ARCPGUFB_PAUSE, NULL);
			return 0;
		case VOCTRL_RESUME:
			if (verbose) fprintf(stderr,"arcpgufb:control - resume\n");
			ioctl(fd, ARCPGUFB_RESUME, NULL);
			return 0;
      case VOCTRL_PAUSE_MS:
          ioctl(fd, ARCPGUFB_PAUSE_MS, data);
          return 0;
      case VOCTRL_GET_NUM_FREE_BUFS:
           ioctl(fd, ARCPGUFB_NUM_FREE_BUFS, data);
          return *(int *)data;

  	//Barry 2007-07-18
	case VOCTRL_GET_NUM_FREE_SCALE_BUFS:
           ioctl(fd, ARCPGUFB_NUM_FREE_SCALE_BUFS, data);
          return *(int *)data;
	case VOCTRL_GET_NUM_FREE_DEC_BUFS:
           ioctl(fd, ARCPGUFB_NUM_FREE_DEC_BUFS, data);
          return *(int *)data;
		  
      case VOCTRL_DROP_FRAME:
           ioctl(fd, ARCPGUFB_FRAME_DROP, data);
          return 0;
      case VOCTRL_GET_QUEUE_LEN:
           ioctl(fd, ARCPGUFB_QUEUE_LEN_GET, data);
          return *(int *)data;
      case VOCTRL_FLUSH_QUEUE:
	       ioctl(fd, ARCPGUFB_FLUSH, NULL);
          return 0;
      case VOCTRL_SET_FRAME_RATE:
          val = (*(float *)data) * 1000;
          ioctl(fd, ARCPGUFB_FRAMERATE_SET, &val);
          return 0;

	}

	if (verbose) fprintf(stderr,"arcpgufb:control - dunno %d.\n", request);
	return VO_NOTIMPL;
}

//--------------------------------------------------------------------------------

/* Dummy funcs */
static void check_events(void) {}

//--------------------------------------------------------------------------------
#ifndef BUILD_SK2851
unsigned long get_ir_value()
{
	unsigned long value;
	if (fd == -1)
		fd = open(DEFAULT_FB, O_RDWR);
	if (ioctl(fd,  ARCPGUFB_GET_IRVALUE, &value))
		return 0;
	return value;
}
#else
#if 0	//Barry 2008-08-18
/*
 * system call - macro definition
 */
#define MSG_QUEUE_SIZE             64
#define MSG_MAGIC_NUMBER           0xB0322CDA

#define MSG_CMD_OK                 0
#define MSG_CMD_FAIL               MSG_MAGIC_NUMBER

#define READ_MSG                   0x00000001
#define WRITE_MSG                  0x00000002
#define CLEAR_ALL_MSG              0x00000004

#define VALID_TYPE_MASK            0xFFFF0000
#define TYPE_DONT_CARE             0x00000000
#define REMOVE_MSG_AFTER_READ      0x00010000
#define NOT_CHANGE_MSG_QUEUE_DATA  0x00020000
#define FORCED_WRITE_MSG_2_QUEUE   0x00040000
/* 
 * Declaration
 */
unsigned int exchange_msg( unsigned int _iCmd, unsigned int* _piData, unsigned int _iType );
/* 
 * Implement
 */
_syscall3(unsigned int, exchange_msg, unsigned int, _iCmd, unsigned int*, _piData, unsigned int, _iType );
/*
 *
 */
unsigned long get_ir_value( void ) {
     unsigned int value;
     if( exchange_msg( READ_MSG, &value, REMOVE_MSG_AFTER_READ ) == MSG_CMD_OK ) {
          return (unsigned long) value; 
     }
     else {
          return 0;
     }
}
#endif
unsigned long get_ir_value( void )
{
     return syscall_get_ir();	//Barry 2008-08-18
}
#endif
