
#include <signal.h>
#include <stdio.h>

struct ucontext {
        unsigned long     uc_flags;
        struct ucontext  *uc_link;
        stack_t           uc_stack;
        struct sigcontext uc_mcontext;
        sigset_t          uc_sigmask;   /* mask last for extensibility */
};

void segv_action(int signum, siginfo_t *info, void *ptr)
{
  struct ucontext *context = (struct ucontext*) ptr;

  printf("\n\n\nSEGV handler: signum %d info at 0x%x ptr at 0x%x\n", signum, info, ptr);
  printf("  Signal no. %d\n",info->si_signo);
  printf("  Process ID %d\n",info->si_pid);
  printf("  Code : ");
  switch (info->si_code)
  {
    case SEGV_MAPERR:
      printf("SEGV_MAPERR, address not mapped to object\n");
      break;
    case SEGV_ACCERR:
      printf("SEGV_ACCERR, invalid permissions for mapped object\n");
      break;
    default:
      printf("0x%x, Unknown cause\n",info->si_code);
      break;
 }
  printf("  Faulting address: 0x%x\n",info->si_addr);

  printf("  Context at 0x%x, flags %d, next 0x%x, stack ptr 0x%x, size %d\n", context, context->uc_flags, context->uc_link, context->uc_stack.ss_sp, context->uc_stack.ss_size);

  exit(-1);
}

static struct sigaction sigstruct = 
{ 
  segv_action, 
  0, 
  SA_SIGINFO | SA_ONESHOT, 
  0
};

int install_segv_handler()
{
  sigset_t empty = {0,0};
  sigstruct.sa_sigaction = segv_action;
  sigstruct.sa_flags = SA_SIGINFO | SA_ONESHOT;
  sigstruct.sa_mask = empty;
  sigstruct.sa_restorer = 0;
  return sigaction(SIGSEGV,  &sigstruct, 0);
}
int install_ill_handler()
{
  sigset_t empty = {0,0};
  sigstruct.sa_sigaction = segv_action;
  sigstruct.sa_flags = SA_SIGINFO | SA_ONESHOT;
  sigstruct.sa_mask = empty;
  sigstruct.sa_restorer = 0;
  return sigaction(SIGILL,  &sigstruct, 0);
}


/*               siginfo_t { */
/*                   int      si_signo;  /\* Signal number *\/ */
/*                   int      si_errno;  /\* An errno value *\/ */
/*                   int      si_code;   /\* Signal code *\/ */
/*                   pid_t    si_pid;    /\* Sending process ID *\/ */
/*                   uid_t    si_uid;    /\* Real user ID of sending process *\/ */
/*                   int      si_status; /\* Exit value or signal *\/ */
/*                   clock_t  si_utime;  /\* User time consumed *\/ */
/*                   clock_t  si_stime;  /\* System time consumed *\/ */
/*                   sigval_t si_value;  /\* Signal value *\/ */
/*                   int      si_int;    /\* POSIX.1b signal *\/ */
/*                   void *   si_ptr;    /\* POSIX.1b signal *\/ */
/*                   void *   si_addr;   /\* Memory location which caused fault *\/ */
/*                   int      si_band;   /\* Band event *\/ */
/*                   int      si_fd;     /\* File descriptor *\/ */
/*               } */

