/* vi: set sw=4 ts=4: */
/* common.c
 *
 * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
 */

#include "common.h"

const uint8_t MAC_BCAST_ADDR[6] ALIGN2 = {
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};


void xpipe(int filedes[2])
{
  if (pipe(filedes))
      bb_perror_msg_and_die("can't create pipe");
}

// Turn on nonblocking I/O on a fd
int ndelay_on(int fd)
{
  return fcntl(fd, F_SETFL, fcntl(fd,F_GETFL) | O_NONBLOCK);
}

int close_on_exec_on(int fd)
{
  return fcntl(fd, F_SETFD, FD_CLOEXEC);
}

int ndelay_off(int fd)
{
  return fcntl(fd, F_SETFL, fcntl(fd,F_GETFL) & ~O_NONBLOCK);
}

void setsockopt_reuseaddr(int fd)
{
  int optval;
  setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
}

int setsockopt_broadcast(int fd)
{
  int optval;
  return setsockopt(fd, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval));
}

unsigned long long monotonic_us(void)
{
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return tv.tv_sec * 1000000ULL + tv.tv_usec;
}


char *xasprintf(const char *format, ...)
{
	va_list p;
	int r;
	char *string_ptr;

#if 1
	// GNU extension
	va_start(p, format);
	r = vasprintf(&string_ptr, format, p);
	va_end(p);
#else
	// Bloat for systems that haven't got the GNU extension.
	va_start(p, format);
	r = vsnprintf(NULL, 0, format, p);
	va_end(p);
	string_ptr = malloc(r+1);
	va_start(p, format);
	r = vsnprintf(string_ptr, r+1, format, p);
	va_end(p);
#endif

	if (r < 0)
	{
		fprintf(stderr, "bb_msg_memory_exhausted\n");
	}
	return string_ptr;
}

void bb_info_msg(const char *s, ...)
{
	va_list p;
	/* va_copy is used because it is not portable
	 * to use va_list p twice */
	va_list p2;

	va_start(p, s);
	va_copy(p2, p);
//	if (logmode & LOGMODE_STDIO) {
		vprintf(s, p);
		fputs("\r\n", stdout);
//	}
//	if (ENABLE_FEATURE_SYSLOG && (logmode & LOGMODE_SYSLOG))
//		vsyslog(LOG_INFO, s, p2);
	va_end(p2);
	va_end(p);
}


